#define LINUX_VERSION_CODE 199168
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#define RHEL_MAJOR 7
#define RHEL_MINOR 8
#define RHEL_RELEASE_VERSION(a,b) (((a) << 8) + (b))
#define RHEL_RELEASE_CODE 1800
#define RHEL_RELEASE "1127"
#define KERNEL_HEADERS_CHECKSUM "5a424cbcb7f25098cfca5506eb07cf1b5395144f"
