/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_x86_sahf_1 = 33,
  CODE_FOR_insvhi_1 = 94,
  CODE_FOR_insvsi_1 = 95,
  CODE_FOR_insvdi_1 = 96,
  CODE_FOR_zero_extendqidi2 = 115,
  CODE_FOR_zero_extendhidi2 = 116,
  CODE_FOR_zero_extendqisi2_and = 117,
  CODE_FOR_zero_extendhisi2_and = 118,
  CODE_FOR_zero_extendqihi2_and = 121,
  CODE_FOR_extendsidi2_1 = 129,
  CODE_FOR_extendqidi2 = 130,
  CODE_FOR_extendhidi2 = 131,
  CODE_FOR_extendhisi2 = 132,
  CODE_FOR_extendqisi2 = 134,
  CODE_FOR_extendqihi2 = 136,
  CODE_FOR_truncdfsf2 = 140,
  CODE_FOR_truncxfsf2 = 141,
  CODE_FOR_truncxfdf2 = 142,
  CODE_FOR_fixuns_truncsfdi2 = 143,
  CODE_FOR_fixuns_truncdfdi2 = 144,
  CODE_FOR_fixuns_truncsfsi2_avx512f = 145,
  CODE_FOR_fixuns_truncdfsi2_avx512f = 146,
  CODE_FOR_fix_truncsfsi_sse = 151,
  CODE_FOR_fix_truncsfdi_sse = 152,
  CODE_FOR_fix_truncdfsi_sse = 153,
  CODE_FOR_fix_truncdfdi_sse = 154,
  CODE_FOR_fix_trunchi_i387_fisttp = 155,
  CODE_FOR_fix_truncsi_i387_fisttp = 156,
  CODE_FOR_fix_truncdi_i387_fisttp = 157,
  CODE_FOR_fix_truncdi_i387 = 161,
  CODE_FOR_fix_trunchi_i387 = 162,
  CODE_FOR_fix_truncsi_i387 = 163,
  CODE_FOR_x86_fnstcw_1 = 164,
  CODE_FOR_floathisf2 = 165,
  CODE_FOR_floathidf2 = 166,
  CODE_FOR_floathixf2 = 167,
  CODE_FOR_floatsixf2 = 168,
  CODE_FOR_floatdixf2 = 169,
  CODE_FOR_floatdisf2_i387_with_xmm = 176,
  CODE_FOR_floatdidf2_i387_with_xmm = 177,
  CODE_FOR_floatdixf2_i387_with_xmm = 178,
  CODE_FOR_floatunssisf2_i387_with_xmm = 183,
  CODE_FOR_floatunssidf2_i387_with_xmm = 184,
  CODE_FOR_floatunssixf2_i387_with_xmm = 185,
  CODE_FOR_addsi_1_zext = 192,
  CODE_FOR_addqi_ext_1 = 214,
  CODE_FOR_addqi3_carry = 268,
  CODE_FOR_addhi3_carry = 269,
  CODE_FOR_addsi3_carry = 270,
  CODE_FOR_adddi3_carry = 271,
  CODE_FOR_addcarrysi = 278,
  CODE_FOR_addcarrydi = 279,
  CODE_FOR_subqi3_carry = 280,
  CODE_FOR_subhi3_carry = 281,
  CODE_FOR_subsi3_carry = 282,
  CODE_FOR_subdi3_carry = 283,
  CODE_FOR_subsi3_carry_ccc = 290,
  CODE_FOR_subdi3_carry_ccc = 291,
  CODE_FOR_subsi3_carry_ccgz = 294,
  CODE_FOR_subdi3_carry_ccgz = 295,
  CODE_FOR_subborrowsi = 296,
  CODE_FOR_subborrowdi = 297,
  CODE_FOR_divmodsi4_1 = 346,
  CODE_FOR_divmoddi4_1 = 347,
  CODE_FOR_divmodsi4_zext_1 = 348,
  CODE_FOR_divmodsi4_zext_2 = 349,
  CODE_FOR_divmodhiqi3 = 360,
  CODE_FOR_udivmodsi4_1 = 361,
  CODE_FOR_udivmoddi4_1 = 362,
  CODE_FOR_udivmodsi4_zext_1 = 363,
  CODE_FOR_udivmodsi4_zext_2 = 364,
  CODE_FOR_udivmodhiqi3 = 379,
  CODE_FOR_andqi_ext_1 = 405,
  CODE_FOR_copysignsf3_const = 486,
  CODE_FOR_copysigndf3_const = 487,
  CODE_FOR_copysigntf3_const = 488,
  CODE_FOR_copysignsf3_var = 489,
  CODE_FOR_copysigndf3_var = 490,
  CODE_FOR_copysigntf3_var = 491,
  CODE_FOR_xorsignsf3_1 = 492,
  CODE_FOR_xorsigndf3_1 = 493,
  CODE_FOR_x86_64_shld = 511,
  CODE_FOR_x86_shld = 512,
  CODE_FOR_x86_64_shrd = 555,
  CODE_FOR_x86_shrd = 556,
  CODE_FOR_ashrdi3_cvt = 557,
  CODE_FOR_ashrsi3_cvt = 559,
  CODE_FOR_ix86_rotldi3_doubleword = 604,
  CODE_FOR_ix86_rotlti3_doubleword = 605,
  CODE_FOR_ix86_rotrdi3_doubleword = 606,
  CODE_FOR_ix86_rotrti3_doubleword = 607,
  CODE_FOR_setcc_sf_sse = 657,
  CODE_FOR_setcc_df_sse = 658,
  CODE_FOR_jump = 660,
  CODE_FOR_blockage = 687,
  CODE_FOR_prologue_use = 689,
  CODE_FOR_simple_return_internal = 690,
  CODE_FOR_interrupt_return = 691,
  CODE_FOR_simple_return_internal_long = 692,
  CODE_FOR_simple_return_pop_internal = 693,
  CODE_FOR_nop = 696,
  CODE_FOR_nops = 697,
  CODE_FOR_pad = 698,
  CODE_FOR_set_got_rex64 = 701,
  CODE_FOR_set_rip_rex64 = 702,
  CODE_FOR_set_got_offset_rex64 = 703,
  CODE_FOR_eh_return_internal = 704,
  CODE_FOR_leave = 705,
  CODE_FOR_leave_rex64 = 706,
  CODE_FOR_split_stack_return = 707,
  CODE_FOR_ffssi2_no_cmove = 708,
  CODE_FOR_ctzsi2 = 715,
  CODE_FOR_ctzdi2 = 716,
  CODE_FOR_bsr_rex64 = 719,
  CODE_FOR_bsr = 720,
  CODE_FOR_clzsi2_lzcnt = 722,
  CODE_FOR_clzdi2_lzcnt = 723,
  CODE_FOR_tzcnt_si = 726,
  CODE_FOR_lzcnt_si = 727,
  CODE_FOR_tzcnt_di = 728,
  CODE_FOR_lzcnt_di = 729,
  CODE_FOR_tzcnt_hi = 734,
  CODE_FOR_lzcnt_hi = 735,
  CODE_FOR_bmi_bextr_si = 736,
  CODE_FOR_bmi_bextr_di = 737,
  CODE_FOR_bmi2_pdep_si3 = 756,
  CODE_FOR_bmi2_pdep_di3 = 757,
  CODE_FOR_bmi2_pext_si3 = 758,
  CODE_FOR_bmi2_pext_di3 = 759,
  CODE_FOR_popcountsi2 = 780,
  CODE_FOR_popcountdi2 = 781,
  CODE_FOR_popcounthi2 = 785,
  CODE_FOR_bswaphi_lowpart = 791,
  CODE_FOR_paritydi2_cmp = 792,
  CODE_FOR_paritysi2_cmp = 793,
   CODE_FOR_tls_initial_exec_64_sun = CODE_FOR_nothing,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 844,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 845,
  CODE_FOR_sqrtxf2 = 846,
  CODE_FOR_x86_fnstsw_1 = 850,
  CODE_FOR_fpremxf4_i387 = 851,
  CODE_FOR_fprem1xf4_i387 = 852,
  CODE_FOR_sinxf2 = 853,
  CODE_FOR_cosxf2 = 854,
  CODE_FOR_sincosxf3 = 855,
  CODE_FOR_fptanxf4_i387 = 856,
  CODE_FOR_atan2xf3 = 857,
  CODE_FOR_fyl2xxf3_i387 = 858,
  CODE_FOR_fyl2xp1xf3_i387 = 859,
  CODE_FOR_fxtractxf3_i387 = 860,
  CODE_FOR_fscalexf4_i387 = 862,
  CODE_FOR_sse4_1_roundsf2 = 863,
  CODE_FOR_sse4_1_rounddf2 = 864,
  CODE_FOR_rintxf2 = 865,
  CODE_FOR_lrintxfdi2 = 866,
  CODE_FOR_lrintxfhi2 = 867,
  CODE_FOR_lrintxfsi2 = 868,
  CODE_FOR_frndintxf2_floor = 869,
  CODE_FOR_frndintxf2_ceil = 870,
  CODE_FOR_frndintxf2_trunc = 871,
  CODE_FOR_frndintxf2_floor_i387 = 872,
  CODE_FOR_frndintxf2_ceil_i387 = 873,
  CODE_FOR_frndintxf2_trunc_i387 = 874,
  CODE_FOR_fistdi2_floor = 881,
  CODE_FOR_fistdi2_ceil = 882,
  CODE_FOR_fisthi2_floor = 883,
  CODE_FOR_fisthi2_ceil = 884,
  CODE_FOR_fistsi2_floor = 885,
  CODE_FOR_fistsi2_ceil = 886,
  CODE_FOR_fxamsf2_i387 = 887,
  CODE_FOR_fxamdf2_i387 = 888,
  CODE_FOR_fxamxf2_i387 = 889,
  CODE_FOR_movmsk_df = 890,
  CODE_FOR_cld = 891,
  CODE_FOR_smaxsf3 = 950,
  CODE_FOR_sminsf3 = 951,
  CODE_FOR_smaxdf3 = 952,
  CODE_FOR_smindf3 = 953,
  CODE_FOR_pro_epilogue_adjust_stack_si_add = 958,
  CODE_FOR_pro_epilogue_adjust_stack_di_add = 959,
  CODE_FOR_pro_epilogue_adjust_stack_si_sub = 960,
  CODE_FOR_pro_epilogue_adjust_stack_di_sub = 961,
  CODE_FOR_allocate_stack_worker_probe_si = 962,
  CODE_FOR_allocate_stack_worker_probe_di = 963,
  CODE_FOR_probe_stack_si = 964,
  CODE_FOR_probe_stack_di = 965,
  CODE_FOR_adjust_stack_and_probesi = 966,
  CODE_FOR_adjust_stack_and_probedi = 967,
  CODE_FOR_probe_stack_rangesi = 968,
  CODE_FOR_probe_stack_rangedi = 969,
  CODE_FOR_trap = 970,
  CODE_FOR_ud2 = 971,
  CODE_FOR_stack_protect_set_si = 975,
  CODE_FOR_stack_protect_set_di = 976,
  CODE_FOR_stack_protect_test_si = 977,
  CODE_FOR_stack_protect_test_di = 978,
  CODE_FOR_sse4_2_crc32qi = 979,
  CODE_FOR_sse4_2_crc32hi = 980,
  CODE_FOR_sse4_2_crc32si = 981,
  CODE_FOR_sse4_2_crc32di = 982,
  CODE_FOR_rdpmc = 983,
  CODE_FOR_rdpmc_rex64 = 984,
  CODE_FOR_rdtsc = 985,
  CODE_FOR_rdtsc_rex64 = 986,
  CODE_FOR_rdtscp = 987,
  CODE_FOR_rdtscp_rex64 = 988,
  CODE_FOR_fxsave = 989,
  CODE_FOR_fxsave64 = 990,
  CODE_FOR_fxrstor = 991,
  CODE_FOR_fxrstor64 = 992,
  CODE_FOR_xsave = 993,
  CODE_FOR_xsaveopt = 994,
  CODE_FOR_xsavec = 995,
  CODE_FOR_xsaves = 996,
  CODE_FOR_xsave_rex64 = 997,
  CODE_FOR_xsaveopt_rex64 = 998,
  CODE_FOR_xsavec_rex64 = 999,
  CODE_FOR_xsaves_rex64 = 1000,
  CODE_FOR_xsave64 = 1001,
  CODE_FOR_xsaveopt64 = 1002,
  CODE_FOR_xsavec64 = 1003,
  CODE_FOR_xsaves64 = 1004,
  CODE_FOR_xrstor = 1005,
  CODE_FOR_xrstors = 1006,
  CODE_FOR_xrstor_rex64 = 1007,
  CODE_FOR_xrstors_rex64 = 1008,
  CODE_FOR_xrstor64 = 1009,
  CODE_FOR_xrstors64 = 1010,
  CODE_FOR_xsetbv = 1011,
  CODE_FOR_xsetbv_rex64 = 1012,
  CODE_FOR_xgetbv = 1013,
  CODE_FOR_xgetbv_rex64 = 1014,
  CODE_FOR_fnstenv = 1015,
  CODE_FOR_fldenv = 1016,
  CODE_FOR_fnstsw = 1017,
  CODE_FOR_fnclex = 1018,
  CODE_FOR_lwp_slwpcbsi = 1021,
  CODE_FOR_lwp_slwpcbdi = 1022,
  CODE_FOR_rdfsbasesi = 1027,
  CODE_FOR_rdgsbasesi = 1028,
  CODE_FOR_rdfsbasedi = 1029,
  CODE_FOR_rdgsbasedi = 1030,
  CODE_FOR_wrfsbasesi = 1031,
  CODE_FOR_wrgsbasesi = 1032,
  CODE_FOR_wrfsbasedi = 1033,
  CODE_FOR_wrgsbasedi = 1034,
  CODE_FOR_ptwritesi = 1035,
  CODE_FOR_ptwritedi = 1036,
  CODE_FOR_rdrandhi_1 = 1037,
  CODE_FOR_rdrandsi_1 = 1038,
  CODE_FOR_rdranddi_1 = 1039,
  CODE_FOR_rdseedhi_1 = 1040,
  CODE_FOR_rdseedsi_1 = 1041,
  CODE_FOR_rdseeddi_1 = 1042,
  CODE_FOR_rdsspsi = 1044,
  CODE_FOR_rdsspdi = 1045,
  CODE_FOR_incsspsi = 1046,
  CODE_FOR_incsspdi = 1047,
  CODE_FOR_saveprevssp = 1048,
  CODE_FOR_wrsssi = 1051,
  CODE_FOR_wrssdi = 1052,
  CODE_FOR_wrusssi = 1053,
  CODE_FOR_wrussdi = 1054,
  CODE_FOR_setssbsy = 1055,
  CODE_FOR_nop_endbr = 1058,
  CODE_FOR_xbegin_1 = 1059,
  CODE_FOR_xend = 1060,
  CODE_FOR_xabort = 1061,
  CODE_FOR_xtest_1 = 1062,
  CODE_FOR_clwb = 1063,
  CODE_FOR_clflushopt = 1064,
  CODE_FOR_mwaitx = 1065,
  CODE_FOR_monitorx_si = 1066,
  CODE_FOR_monitorx_di = 1067,
  CODE_FOR_clzero_si = 1068,
  CODE_FOR_clzero_di = 1069,
  CODE_FOR_rdpid = 1072,
  CODE_FOR_rdpid_rex64 = 1073,
  CODE_FOR_wbinvd = 1074,
  CODE_FOR_wbnoinvd = 1075,
  CODE_FOR_movdirisi = 1076,
  CODE_FOR_movdiridi = 1077,
  CODE_FOR_movdir64b_si = 1078,
  CODE_FOR_movdir64b_di = 1079,
  CODE_FOR_umwait = 1080,
  CODE_FOR_umwait_rex64 = 1081,
  CODE_FOR_umonitor_si = 1082,
  CODE_FOR_umonitor_di = 1083,
  CODE_FOR_tpause = 1084,
  CODE_FOR_tpause_rex64 = 1085,
  CODE_FOR_cldemote = 1086,
  CODE_FOR_speculation_barrier = 1087,
  CODE_FOR_sse_movntq = 1093,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1099,
  CODE_FOR_mmx_ieee_minv2sf3 = 1100,
  CODE_FOR_mmx_rcpv2sf2 = 1101,
  CODE_FOR_mmx_rcpit1v2sf3 = 1102,
  CODE_FOR_mmx_rcpit2v2sf3 = 1103,
  CODE_FOR_mmx_rsqrtv2sf2 = 1104,
  CODE_FOR_mmx_rsqit1v2sf3 = 1105,
  CODE_FOR_mmx_haddv2sf3 = 1106,
  CODE_FOR_mmx_hsubv2sf3 = 1107,
  CODE_FOR_mmx_addsubv2sf3 = 1108,
  CODE_FOR_mmx_gtv2sf3 = 1110,
  CODE_FOR_mmx_gev2sf3 = 1111,
  CODE_FOR_mmx_pf2id = 1112,
  CODE_FOR_mmx_pf2iw = 1113,
  CODE_FOR_mmx_pi2fw = 1114,
  CODE_FOR_mmx_floatv2si2 = 1115,
  CODE_FOR_mmx_pswapdv2sf2 = 1116,
  CODE_FOR_mmx_ashrv4hi3 = 1147,
  CODE_FOR_mmx_ashrv2si3 = 1148,
  CODE_FOR_mmx_ashlv4hi3 = 1149,
  CODE_FOR_mmx_lshrv4hi3 = 1150,
  CODE_FOR_mmx_ashlv2si3 = 1151,
  CODE_FOR_mmx_lshrv2si3 = 1152,
  CODE_FOR_mmx_ashlv1di3 = 1153,
  CODE_FOR_mmx_lshrv1di3 = 1154,
  CODE_FOR_mmx_gtv8qi3 = 1158,
  CODE_FOR_mmx_gtv4hi3 = 1159,
  CODE_FOR_mmx_gtv2si3 = 1160,
  CODE_FOR_mmx_andnotv8qi3 = 1161,
  CODE_FOR_mmx_andnotv4hi3 = 1162,
  CODE_FOR_mmx_andnotv2si3 = 1163,
  CODE_FOR_mmx_packsswb = 1173,
  CODE_FOR_mmx_packssdw = 1174,
  CODE_FOR_mmx_packuswb = 1175,
  CODE_FOR_mmx_punpckhbw = 1176,
  CODE_FOR_mmx_punpcklbw = 1177,
  CODE_FOR_mmx_punpckhwd = 1178,
  CODE_FOR_mmx_punpcklwd = 1179,
  CODE_FOR_mmx_punpckhdq = 1180,
  CODE_FOR_mmx_punpckldq = 1181,
  CODE_FOR_mmx_pextrw = 1183,
  CODE_FOR_mmx_pshufw_1 = 1184,
  CODE_FOR_mmx_pswapdv2si2 = 1185,
  CODE_FOR_mmx_psadbw = 1194,
  CODE_FOR_mmx_pmovmskb = 1195,
  CODE_FOR_mmx_emms = 1198,
  CODE_FOR_mmx_femms = 1199,
  CODE_FOR_movv64qi_internal = 1200,
  CODE_FOR_movv32qi_internal = 1201,
  CODE_FOR_movv16qi_internal = 1202,
  CODE_FOR_movv32hi_internal = 1203,
  CODE_FOR_movv16hi_internal = 1204,
  CODE_FOR_movv8hi_internal = 1205,
  CODE_FOR_movv16si_internal = 1206,
  CODE_FOR_movv8si_internal = 1207,
  CODE_FOR_movv4si_internal = 1208,
  CODE_FOR_movv8di_internal = 1209,
  CODE_FOR_movv4di_internal = 1210,
  CODE_FOR_movv2di_internal = 1211,
  CODE_FOR_movv4ti_internal = 1212,
  CODE_FOR_movv2ti_internal = 1213,
  CODE_FOR_movv1ti_internal = 1214,
  CODE_FOR_movv16sf_internal = 1215,
  CODE_FOR_movv8sf_internal = 1216,
  CODE_FOR_movv4sf_internal = 1217,
  CODE_FOR_movv8df_internal = 1218,
  CODE_FOR_movv4df_internal = 1219,
  CODE_FOR_movv2df_internal = 1220,
  CODE_FOR_avx512f_loadv16si_mask = 1221,
  CODE_FOR_avx512vl_loadv8si_mask = 1222,
  CODE_FOR_avx512vl_loadv4si_mask = 1223,
  CODE_FOR_avx512f_loadv8di_mask = 1224,
  CODE_FOR_avx512vl_loadv4di_mask = 1225,
  CODE_FOR_avx512vl_loadv2di_mask = 1226,
  CODE_FOR_avx512f_loadv16sf_mask = 1227,
  CODE_FOR_avx512vl_loadv8sf_mask = 1228,
  CODE_FOR_avx512vl_loadv4sf_mask = 1229,
  CODE_FOR_avx512f_loadv8df_mask = 1230,
  CODE_FOR_avx512vl_loadv4df_mask = 1231,
  CODE_FOR_avx512vl_loadv2df_mask = 1232,
  CODE_FOR_avx512bw_loadv64qi_mask = 1233,
  CODE_FOR_avx512vl_loadv16qi_mask = 1234,
  CODE_FOR_avx512vl_loadv32qi_mask = 1235,
  CODE_FOR_avx512bw_loadv32hi_mask = 1236,
  CODE_FOR_avx512vl_loadv16hi_mask = 1237,
  CODE_FOR_avx512vl_loadv8hi_mask = 1238,
  CODE_FOR_avx512f_movsf_mask = 1239,
  CODE_FOR_avx512f_movdf_mask = 1240,
  CODE_FOR_avx512f_storesf_mask = 1243,
  CODE_FOR_avx512f_storedf_mask = 1244,
  CODE_FOR_avx512f_blendmv16si = 1245,
  CODE_FOR_avx512vl_blendmv8si = 1246,
  CODE_FOR_avx512vl_blendmv4si = 1247,
  CODE_FOR_avx512f_blendmv8di = 1248,
  CODE_FOR_avx512vl_blendmv4di = 1249,
  CODE_FOR_avx512vl_blendmv2di = 1250,
  CODE_FOR_avx512f_blendmv16sf = 1251,
  CODE_FOR_avx512vl_blendmv8sf = 1252,
  CODE_FOR_avx512vl_blendmv4sf = 1253,
  CODE_FOR_avx512f_blendmv8df = 1254,
  CODE_FOR_avx512vl_blendmv4df = 1255,
  CODE_FOR_avx512vl_blendmv2df = 1256,
  CODE_FOR_avx512bw_blendmv64qi = 1257,
  CODE_FOR_avx512vl_blendmv16qi = 1258,
  CODE_FOR_avx512vl_blendmv32qi = 1259,
  CODE_FOR_avx512bw_blendmv32hi = 1260,
  CODE_FOR_avx512vl_blendmv16hi = 1261,
  CODE_FOR_avx512vl_blendmv8hi = 1262,
  CODE_FOR_avx512f_storev16si_mask = 1263,
  CODE_FOR_avx512vl_storev8si_mask = 1264,
  CODE_FOR_avx512vl_storev4si_mask = 1265,
  CODE_FOR_avx512f_storev8di_mask = 1266,
  CODE_FOR_avx512vl_storev4di_mask = 1267,
  CODE_FOR_avx512vl_storev2di_mask = 1268,
  CODE_FOR_avx512f_storev16sf_mask = 1269,
  CODE_FOR_avx512vl_storev8sf_mask = 1270,
  CODE_FOR_avx512vl_storev4sf_mask = 1271,
  CODE_FOR_avx512f_storev8df_mask = 1272,
  CODE_FOR_avx512vl_storev4df_mask = 1273,
  CODE_FOR_avx512vl_storev2df_mask = 1274,
  CODE_FOR_avx512bw_storev64qi_mask = 1275,
  CODE_FOR_avx512vl_storev16qi_mask = 1276,
  CODE_FOR_avx512vl_storev32qi_mask = 1277,
  CODE_FOR_avx512bw_storev32hi_mask = 1278,
  CODE_FOR_avx512vl_storev16hi_mask = 1279,
  CODE_FOR_avx512vl_storev8hi_mask = 1280,
  CODE_FOR_sse2_movq128 = 1281,
  CODE_FOR_movdi_to_sse = 1282,
  CODE_FOR_avx_lddqu256 = 1283,
  CODE_FOR_sse3_lddqu = 1284,
  CODE_FOR_sse2_movntisi = 1285,
  CODE_FOR_sse2_movntidi = 1286,
  CODE_FOR_avx512f_movntv16sf = 1287,
  CODE_FOR_avx_movntv8sf = 1288,
  CODE_FOR_sse_movntv4sf = 1289,
  CODE_FOR_avx512f_movntv8df = 1290,
  CODE_FOR_avx_movntv4df = 1291,
  CODE_FOR_sse2_movntv2df = 1292,
  CODE_FOR_avx512f_movntv8di = 1293,
  CODE_FOR_avx_movntv4di = 1294,
  CODE_FOR_sse2_movntv2di = 1295,
  CODE_FOR_kandqi = 1296,
  CODE_FOR_kiorqi = 1297,
  CODE_FOR_kxorqi = 1298,
  CODE_FOR_kandhi = 1299,
  CODE_FOR_kiorhi = 1300,
  CODE_FOR_kxorhi = 1301,
  CODE_FOR_kandsi = 1302,
  CODE_FOR_kiorsi = 1303,
  CODE_FOR_kxorsi = 1304,
  CODE_FOR_kanddi = 1305,
  CODE_FOR_kiordi = 1306,
  CODE_FOR_kxordi = 1307,
  CODE_FOR_kandnqi = 1308,
  CODE_FOR_kandnhi = 1309,
  CODE_FOR_kandnsi = 1310,
  CODE_FOR_kandndi = 1311,
  CODE_FOR_kxnorqi = 1312,
  CODE_FOR_kxnorhi = 1313,
  CODE_FOR_kxnorsi = 1314,
  CODE_FOR_kxnordi = 1315,
  CODE_FOR_knotqi = 1316,
  CODE_FOR_knothi = 1317,
  CODE_FOR_knotsi = 1318,
  CODE_FOR_knotdi = 1319,
  CODE_FOR_kaddqi = 1320,
  CODE_FOR_kaddhi = 1321,
  CODE_FOR_kaddsi = 1322,
  CODE_FOR_kadddi = 1323,
  CODE_FOR_kashiftqi = 1324,
  CODE_FOR_klshiftrtqi = 1325,
  CODE_FOR_kashifthi = 1326,
  CODE_FOR_klshiftrthi = 1327,
  CODE_FOR_kashiftsi = 1328,
  CODE_FOR_klshiftrtsi = 1329,
  CODE_FOR_kashiftdi = 1330,
  CODE_FOR_klshiftrtdi = 1331,
  CODE_FOR_ktestqi = 1332,
  CODE_FOR_ktesthi = 1333,
  CODE_FOR_ktestsi = 1334,
  CODE_FOR_ktestdi = 1335,
  CODE_FOR_kortestqi = 1336,
  CODE_FOR_kortesthi = 1337,
  CODE_FOR_kortestsi = 1338,
  CODE_FOR_kortestdi = 1339,
  CODE_FOR_kunpckhi = 1340,
  CODE_FOR_kunpcksi = 1341,
  CODE_FOR_kunpckdi = 1342,
  CODE_FOR_sse_vmaddv4sf3 = 1421,
  CODE_FOR_sse_vmaddv4sf3_round = 1422,
  CODE_FOR_sse_vmaddv4sf3_mask = 1423,
  CODE_FOR_sse_vmaddv4sf3_mask_round = 1424,
  CODE_FOR_sse_vmsubv4sf3 = 1425,
  CODE_FOR_sse_vmsubv4sf3_round = 1426,
  CODE_FOR_sse_vmsubv4sf3_mask = 1427,
  CODE_FOR_sse_vmsubv4sf3_mask_round = 1428,
  CODE_FOR_sse2_vmaddv2df3 = 1429,
  CODE_FOR_sse2_vmaddv2df3_round = 1430,
  CODE_FOR_sse2_vmaddv2df3_mask = 1431,
  CODE_FOR_sse2_vmaddv2df3_mask_round = 1432,
  CODE_FOR_sse2_vmsubv2df3 = 1433,
  CODE_FOR_sse2_vmsubv2df3_round = 1434,
  CODE_FOR_sse2_vmsubv2df3_mask = 1435,
  CODE_FOR_sse2_vmsubv2df3_mask_round = 1436,
  CODE_FOR_sse_vmmulv4sf3 = 1465,
  CODE_FOR_sse_vmmulv4sf3_round = 1466,
  CODE_FOR_sse_vmmulv4sf3_mask = 1467,
  CODE_FOR_sse_vmmulv4sf3_mask_round = 1468,
  CODE_FOR_sse_vmdivv4sf3 = 1469,
  CODE_FOR_sse_vmdivv4sf3_round = 1470,
  CODE_FOR_sse_vmdivv4sf3_mask = 1471,
  CODE_FOR_sse_vmdivv4sf3_mask_round = 1472,
  CODE_FOR_sse2_vmmulv2df3 = 1473,
  CODE_FOR_sse2_vmmulv2df3_round = 1474,
  CODE_FOR_sse2_vmmulv2df3_mask = 1475,
  CODE_FOR_sse2_vmmulv2df3_mask_round = 1476,
  CODE_FOR_sse2_vmdivv2df3 = 1477,
  CODE_FOR_sse2_vmdivv2df3_round = 1478,
  CODE_FOR_sse2_vmdivv2df3_mask = 1479,
  CODE_FOR_sse2_vmdivv2df3_mask_round = 1480,
  CODE_FOR_avx512f_divv16sf3 = 1481,
  CODE_FOR_avx512f_divv16sf3_round = 1482,
  CODE_FOR_avx512f_divv16sf3_mask = 1483,
  CODE_FOR_avx512f_divv16sf3_mask_round = 1484,
  CODE_FOR_avx_divv8sf3 = 1485,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 1486,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 1487,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 1488,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 1489,
  CODE_FOR_avx512f_divv8df3_round = 1490,
  CODE_FOR_avx512f_divv8df3_mask = 1491,
  CODE_FOR_avx512f_divv8df3_mask_round = 1492,
  CODE_FOR_avx_divv4df3 = 1493,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 1494,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 1495,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 1496,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 1509,
  CODE_FOR_sse_rcpv4sf2 = 1510,
  CODE_FOR_sse_vmrcpv4sf2 = 1511,
  CODE_FOR_rcp14v16sf_mask = 1513,
  CODE_FOR_rcp14v8sf_mask = 1515,
  CODE_FOR_rcp14v4sf_mask = 1517,
  CODE_FOR_rcp14v8df_mask = 1519,
  CODE_FOR_rcp14v4df_mask = 1521,
  CODE_FOR_rcp14v2df_mask = 1523,
  CODE_FOR_srcp14v4sf = 1524,
  CODE_FOR_srcp14v2df = 1525,
  CODE_FOR_srcp14v4sf_mask = 1526,
  CODE_FOR_srcp14v2df_mask = 1527,
  CODE_FOR_avx512f_sqrtv16sf2 = 1528,
  CODE_FOR_avx512f_sqrtv16sf2_round = 1529,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 1530,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 1531,
  CODE_FOR_avx_sqrtv8sf2 = 1532,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 1533,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 1534,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 1535,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 1536,
  CODE_FOR_avx512f_sqrtv8df2_round = 1537,
  CODE_FOR_avx512f_sqrtv8df2_mask = 1538,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 1539,
  CODE_FOR_avx_sqrtv4df2 = 1540,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 1541,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 1542,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 1543,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsqrtv4sf2 = 1544,
  CODE_FOR_sse_vmsqrtv4sf2_round = 1545,
  CODE_FOR_sse_vmsqrtv4sf2_mask = 1546,
  CODE_FOR_sse_vmsqrtv4sf2_mask_round = 1547,
  CODE_FOR_sse2_vmsqrtv2df2 = 1548,
  CODE_FOR_sse2_vmsqrtv2df2_round = 1549,
  CODE_FOR_sse2_vmsqrtv2df2_mask = 1550,
  CODE_FOR_sse2_vmsqrtv2df2_mask_round = 1551,
  CODE_FOR_avx_rsqrtv8sf2 = 1552,
  CODE_FOR_sse_rsqrtv4sf2 = 1553,
  CODE_FOR_rsqrt14v16sf_mask = 1555,
  CODE_FOR_rsqrt14v8sf_mask = 1557,
  CODE_FOR_rsqrt14v4sf_mask = 1559,
  CODE_FOR_rsqrt14v8df_mask = 1561,
  CODE_FOR_rsqrt14v4df_mask = 1563,
  CODE_FOR_rsqrt14v2df_mask = 1565,
  CODE_FOR_rsqrt14v4sf = 1566,
  CODE_FOR_rsqrt14v2df = 1567,
  CODE_FOR_rsqrt14_v4sf_mask = 1568,
  CODE_FOR_rsqrt14_v2df_mask = 1569,
  CODE_FOR_sse_vmrsqrtv4sf2 = 1570,
  CODE_FOR_ieee_maxv16sf3 = 1603,
  CODE_FOR_ieee_maxv16sf3_round = 1604,
  CODE_FOR_ieee_maxv16sf3_mask = 1605,
  CODE_FOR_ieee_maxv16sf3_mask_round = 1606,
  CODE_FOR_ieee_minv16sf3 = 1607,
  CODE_FOR_ieee_minv16sf3_round = 1608,
  CODE_FOR_ieee_minv16sf3_mask = 1609,
  CODE_FOR_ieee_minv16sf3_mask_round = 1610,
  CODE_FOR_ieee_maxv8sf3 = 1611,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 1612,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 1613,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 1614,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 1615,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 1616,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 1617,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 1618,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 1619,
  CODE_FOR_ieee_maxv8df3_round = 1620,
  CODE_FOR_ieee_maxv8df3_mask = 1621,
  CODE_FOR_ieee_maxv8df3_mask_round = 1622,
  CODE_FOR_ieee_minv8df3 = 1623,
  CODE_FOR_ieee_minv8df3_round = 1624,
  CODE_FOR_ieee_minv8df3_mask = 1625,
  CODE_FOR_ieee_minv8df3_mask_round = 1626,
  CODE_FOR_ieee_maxv4df3 = 1627,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 1628,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 1629,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 1630,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 1631,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 1632,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 1633,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 1634,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsmaxv4sf3 = 1635,
  CODE_FOR_sse_vmsmaxv4sf3_mask = 1636,
  CODE_FOR_sse_vmsmaxv4sf3_round = 1637,
  CODE_FOR_sse_vmsmaxv4sf3_mask_round = 1638,
  CODE_FOR_sse_vmsminv4sf3 = 1639,
  CODE_FOR_sse_vmsminv4sf3_mask = 1640,
  CODE_FOR_sse_vmsminv4sf3_round = 1641,
  CODE_FOR_sse_vmsminv4sf3_mask_round = 1642,
  CODE_FOR_sse2_vmsmaxv2df3 = 1643,
  CODE_FOR_sse2_vmsmaxv2df3_mask = 1644,
  CODE_FOR_sse2_vmsmaxv2df3_round = 1645,
  CODE_FOR_sse2_vmsmaxv2df3_mask_round = 1646,
  CODE_FOR_sse2_vmsminv2df3 = 1647,
  CODE_FOR_sse2_vmsminv2df3_mask = 1648,
  CODE_FOR_sse2_vmsminv2df3_round = 1649,
  CODE_FOR_sse2_vmsminv2df3_mask_round = 1650,
  CODE_FOR_avx_addsubv4df3 = 1651,
  CODE_FOR_sse3_addsubv2df3 = 1652,
  CODE_FOR_avx_addsubv8sf3 = 1653,
  CODE_FOR_sse3_addsubv4sf3 = 1654,
  CODE_FOR_avx_haddv4df3 = 1655,
  CODE_FOR_avx_hsubv4df3 = 1656,
  CODE_FOR_sse3_hsubv2df3 = 1658,
  CODE_FOR_avx_haddv8sf3 = 1661,
  CODE_FOR_avx_hsubv8sf3 = 1662,
  CODE_FOR_sse3_haddv4sf3 = 1663,
  CODE_FOR_sse3_hsubv4sf3 = 1664,
  CODE_FOR_reducepv16sf_mask = 1666,
  CODE_FOR_reducepv8sf_mask = 1668,
  CODE_FOR_reducepv4sf_mask = 1670,
  CODE_FOR_reducepv8df_mask = 1672,
  CODE_FOR_reducepv4df_mask = 1674,
  CODE_FOR_reducepv2df_mask = 1676,
  CODE_FOR_reducesv4sf = 1677,
  CODE_FOR_reducesv4sf_mask = 1678,
  CODE_FOR_reducesv2df = 1679,
  CODE_FOR_reducesv2df_mask = 1680,
  CODE_FOR_avx_cmpv8sf3 = 1681,
  CODE_FOR_avx_cmpv4sf3 = 1682,
  CODE_FOR_avx_cmpv4df3 = 1683,
  CODE_FOR_avx_cmpv2df3 = 1684,
  CODE_FOR_avx_vmcmpv4sf3 = 1685,
  CODE_FOR_avx_vmcmpv2df3 = 1686,
  CODE_FOR_avx_maskcmpv8sf3 = 1691,
  CODE_FOR_sse_maskcmpv4sf3 = 1692,
  CODE_FOR_avx_maskcmpv4df3 = 1693,
  CODE_FOR_sse2_maskcmpv2df3 = 1694,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 1695,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 1696,
  CODE_FOR_avx512f_cmpv16si3 = 1697,
  CODE_FOR_avx512f_cmpv16si3_round = 1698,
  CODE_FOR_avx512f_cmpv16si3_mask = 1699,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 1700,
  CODE_FOR_avx512vl_cmpv8si3 = 1701,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8si3_mask = 1702,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 1703,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3_mask = 1704,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 1705,
  CODE_FOR_avx512f_cmpv8di3_round = 1706,
  CODE_FOR_avx512f_cmpv8di3_mask = 1707,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 1708,
  CODE_FOR_avx512vl_cmpv4di3 = 1709,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4di3_mask = 1710,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 1711,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3_mask = 1712,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 1713,
  CODE_FOR_avx512f_cmpv16sf3_round = 1714,
  CODE_FOR_avx512f_cmpv16sf3_mask = 1715,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 1716,
  CODE_FOR_avx512vl_cmpv8sf3 = 1717,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 1718,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 1719,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 1720,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 1721,
  CODE_FOR_avx512f_cmpv8df3_round = 1722,
  CODE_FOR_avx512f_cmpv8df3_mask = 1723,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 1724,
  CODE_FOR_avx512vl_cmpv4df3 = 1725,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4df3_mask = 1726,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 1727,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3_mask = 1728,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 1729,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 1730,
  CODE_FOR_avx512vl_cmpv16qi3 = 1731,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 1732,
  CODE_FOR_avx512vl_cmpv32qi3 = 1733,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 1734,
  CODE_FOR_avx512bw_cmpv32hi3 = 1735,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 1736,
  CODE_FOR_avx512vl_cmpv16hi3 = 1737,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 1738,
  CODE_FOR_avx512vl_cmpv8hi3 = 1739,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 1740,
  CODE_FOR_avx512bw_ucmpv64qi3 = 1741,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 1742,
  CODE_FOR_avx512vl_ucmpv16qi3 = 1743,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 1744,
  CODE_FOR_avx512vl_ucmpv32qi3 = 1745,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 1746,
  CODE_FOR_avx512bw_ucmpv32hi3 = 1747,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 1748,
  CODE_FOR_avx512vl_ucmpv16hi3 = 1749,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 1750,
  CODE_FOR_avx512vl_ucmpv8hi3 = 1751,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 1752,
  CODE_FOR_avx512f_ucmpv16si3 = 1753,
  CODE_FOR_avx512f_ucmpv16si3_mask = 1754,
  CODE_FOR_avx512vl_ucmpv8si3 = 1755,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 1756,
  CODE_FOR_avx512vl_ucmpv4si3 = 1757,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 1758,
  CODE_FOR_avx512f_ucmpv8di3 = 1759,
  CODE_FOR_avx512f_ucmpv8di3_mask = 1760,
  CODE_FOR_avx512vl_ucmpv4di3 = 1761,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 1762,
  CODE_FOR_avx512vl_ucmpv2di3 = 1763,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 1764,
  CODE_FOR_avx512f_vmcmpv4sf3 = 1765,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 1766,
  CODE_FOR_avx512f_vmcmpv2df3 = 1767,
  CODE_FOR_avx512f_vmcmpv2df3_round = 1768,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 1769,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 1770,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 1771,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 1772,
  CODE_FOR_avx512f_maskcmpv16sf3 = 1773,
  CODE_FOR_avx512f_maskcmpv8sf3 = 1774,
  CODE_FOR_avx512f_maskcmpv4sf3 = 1775,
  CODE_FOR_avx512f_maskcmpv8df3 = 1776,
  CODE_FOR_avx512f_maskcmpv4df3 = 1777,
  CODE_FOR_avx512f_maskcmpv2df3 = 1778,
  CODE_FOR_sse_comi = 1779,
  CODE_FOR_sse_comi_round = 1780,
  CODE_FOR_sse_ucomi = 1781,
  CODE_FOR_sse_ucomi_round = 1782,
  CODE_FOR_sse2_comi = 1783,
  CODE_FOR_sse2_comi_round = 1784,
  CODE_FOR_sse2_ucomi = 1785,
  CODE_FOR_sse2_ucomi_round = 1786,
  CODE_FOR_avx_andnotv8sf3 = 1787,
  CODE_FOR_avx_andnotv8sf3_mask = 1788,
  CODE_FOR_sse_andnotv4sf3 = 1789,
  CODE_FOR_sse_andnotv4sf3_mask = 1790,
  CODE_FOR_avx_andnotv4df3 = 1791,
  CODE_FOR_avx_andnotv4df3_mask = 1792,
  CODE_FOR_sse2_andnotv2df3 = 1793,
  CODE_FOR_sse2_andnotv2df3_mask = 1794,
  CODE_FOR_avx512f_andnotv16sf3 = 1795,
  CODE_FOR_avx512f_andnotv16sf3_mask = 1796,
  CODE_FOR_avx512f_andnotv8df3 = 1797,
  CODE_FOR_avx512f_andnotv8df3_mask = 1798,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 1855,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 1857,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 1859,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 1861,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 1864,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 1866,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 1868,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 1870,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 1907,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 1908,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 1909,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 1910,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 1911,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 1912,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 1913,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 1914,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 1915,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 1916,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 1917,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 1918,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 1919,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 1920,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 1921,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 1922,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 1923,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 1924,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 1925,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 1926,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 1935,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 1937,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 1939,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 1941,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 1944,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 1946,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 1948,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 1950,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 1987,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 1988,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 1989,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 1990,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 1991,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 1992,
  CODE_FOR_avx512f_fmsub_v8df_mask = 1993,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 1994,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 1995,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 1996,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 1997,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 1998,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 1999,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 2000,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 2001,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 2002,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 2003,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 2004,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 2005,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 2006,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 2015,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 2017,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 2019,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 2021,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 2024,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 2026,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 2028,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 2030,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 2067,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 2068,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 2069,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 2070,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 2071,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 2072,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 2073,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 2074,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 2075,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 2076,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 2077,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 2078,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 2079,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 2080,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 2081,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 2082,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 2105,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 2107,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 2109,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 2111,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 2114,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 2116,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 2118,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 2120,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 2157,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 2158,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 2159,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 2160,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 2161,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 2162,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 2163,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 2164,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 2165,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 2166,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 2167,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 2168,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 2169,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 2170,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 2171,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 2172,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 2173,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 2174,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 2175,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 2176,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 2183,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 2185,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 2187,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 2189,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 2192,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 2194,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 2196,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 2198,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 2199,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 2200,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 2201,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 2202,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 2203,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 2204,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 2205,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 2206,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 2207,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 2208,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 2209,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 2210,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 2211,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 2212,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 2213,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 2214,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 2215,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 2216,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 2217,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 2218,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 2219,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 2220,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 2221,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 2222,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 2229,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 2231,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 2233,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 2235,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 2238,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 2240,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 2242,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 2244,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 2245,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 2246,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 2247,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 2248,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 2249,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 2250,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 2251,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 2252,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 2253,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 2254,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 2255,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 2256,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 2257,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 2258,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 2259,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 2260,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 2261,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 2262,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 2263,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 2264,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 2265,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 2266,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 2267,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 2268,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask = 2285,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask_round = 2286,
  CODE_FOR_avx512f_vmfmadd_v2df_mask = 2287,
  CODE_FOR_avx512f_vmfmadd_v2df_mask_round = 2288,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3 = 2289,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3_round = 2290,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3 = 2291,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3_round = 2292,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1 = 2293,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1_round = 2294,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1 = 2295,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1_round = 2296,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3 = 2301,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3_round = 2302,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3 = 2303,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3_round = 2304,
  CODE_FOR_sse_cvtpi2ps = 2341,
  CODE_FOR_sse_cvtps2pi = 2342,
  CODE_FOR_sse_cvttps2pi = 2343,
  CODE_FOR_sse_cvtsi2ss = 2344,
  CODE_FOR_sse_cvtsi2ss_round = 2345,
  CODE_FOR_sse_cvtsi2ssq = 2346,
  CODE_FOR_sse_cvtsi2ssq_round = 2347,
  CODE_FOR_sse_cvtss2si = 2348,
  CODE_FOR_sse_cvtss2si_round = 2349,
  CODE_FOR_sse_cvtss2siq = 2350,
  CODE_FOR_sse_cvtss2siq_round = 2351,
  CODE_FOR_sse_cvtss2si_2 = 2352,
  CODE_FOR_sse_cvtss2siq_2 = 2353,
  CODE_FOR_sse_cvttss2si = 2354,
  CODE_FOR_sse_cvttss2si_round = 2355,
  CODE_FOR_sse_cvttss2siq = 2356,
  CODE_FOR_sse_cvttss2siq_round = 2357,
  CODE_FOR_cvtusi2ss32 = 2358,
  CODE_FOR_cvtusi2ss32_round = 2359,
  CODE_FOR_cvtusi2sd32 = 2360,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss64 = 2361,
  CODE_FOR_cvtusi2ss64_round = 2362,
  CODE_FOR_cvtusi2sd64 = 2363,
  CODE_FOR_cvtusi2sd64_round = 2364,
  CODE_FOR_floatv16siv16sf2 = 2365,
  CODE_FOR_floatv16siv16sf2_round = 2366,
  CODE_FOR_floatv16siv16sf2_mask = 2367,
  CODE_FOR_floatv16siv16sf2_mask_round = 2368,
  CODE_FOR_floatv8siv8sf2 = 2369,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 2370,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 2371,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 2372,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv16siv16sf2 = 2373,
  CODE_FOR_ufloatv16siv16sf2_round = 2374,
  CODE_FOR_ufloatv16siv16sf2_mask = 2375,
  CODE_FOR_ufloatv16siv16sf2_mask_round = 2376,
  CODE_FOR_ufloatv8siv8sf2 = 2377,
  CODE_FOR_ufloatv8siv8sf2_round = 2378,
  CODE_FOR_ufloatv8siv8sf2_mask = 2379,
  CODE_FOR_ufloatv8siv8sf2_mask_round = 2380,
  CODE_FOR_ufloatv4siv4sf2 = 2381,
  CODE_FOR_ufloatv4siv4sf2_round = 2382,
  CODE_FOR_ufloatv4siv4sf2_mask = 2383,
  CODE_FOR_ufloatv4siv4sf2_mask_round = 2384,
  CODE_FOR_avx_fix_notruncv8sfv8si = 2385,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 2386,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 2387,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 2388,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 2389,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 2390,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 2391,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 2392,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask = 2395,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask_round = 2396,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask = 2399,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask_round = 2400,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask = 2403,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask_round = 2404,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 2407,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 2408,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 2410,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 2412,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 2415,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 2416,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 2418,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 2420,
  CODE_FOR_fix_truncv16sfv16si2 = 2421,
  CODE_FOR_fix_truncv16sfv16si2_round = 2422,
  CODE_FOR_fix_truncv16sfv16si2_mask = 2423,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 2424,
  CODE_FOR_ufix_truncv16sfv16si2 = 2425,
  CODE_FOR_ufix_truncv16sfv16si2_round = 2426,
  CODE_FOR_ufix_truncv16sfv16si2_mask = 2427,
  CODE_FOR_ufix_truncv16sfv16si2_mask_round = 2428,
  CODE_FOR_fix_truncv8sfv8si2 = 2429,
  CODE_FOR_fix_truncv8sfv8si2_mask = 2430,
  CODE_FOR_fix_truncv4sfv4si2 = 2431,
  CODE_FOR_fix_truncv4sfv4si2_mask = 2432,
  CODE_FOR_sse2_cvtpi2pd = 2433,
  CODE_FOR_sse2_cvtpd2pi = 2434,
  CODE_FOR_sse2_cvttpd2pi = 2435,
  CODE_FOR_sse2_cvtsi2sd = 2436,
  CODE_FOR_sse2_cvtsi2sdq = 2437,
  CODE_FOR_sse2_cvtsi2sdq_round = 2438,
  CODE_FOR_avx512f_vcvtss2usi = 2439,
  CODE_FOR_avx512f_vcvtss2usi_round = 2440,
  CODE_FOR_avx512f_vcvtss2usiq = 2441,
  CODE_FOR_avx512f_vcvtss2usiq_round = 2442,
  CODE_FOR_avx512f_vcvttss2usi = 2443,
  CODE_FOR_avx512f_vcvttss2usi_round = 2444,
  CODE_FOR_avx512f_vcvttss2usiq = 2445,
  CODE_FOR_avx512f_vcvttss2usiq_round = 2446,
  CODE_FOR_avx512f_vcvtsd2usi = 2447,
  CODE_FOR_avx512f_vcvtsd2usi_round = 2448,
  CODE_FOR_avx512f_vcvtsd2usiq = 2449,
  CODE_FOR_avx512f_vcvtsd2usiq_round = 2450,
  CODE_FOR_avx512f_vcvttsd2usi = 2451,
  CODE_FOR_avx512f_vcvttsd2usi_round = 2452,
  CODE_FOR_avx512f_vcvttsd2usiq = 2453,
  CODE_FOR_avx512f_vcvttsd2usiq_round = 2454,
  CODE_FOR_sse2_cvtsd2si = 2455,
  CODE_FOR_sse2_cvtsd2si_round = 2456,
  CODE_FOR_sse2_cvtsd2siq = 2457,
  CODE_FOR_sse2_cvtsd2siq_round = 2458,
  CODE_FOR_sse2_cvtsd2si_2 = 2459,
  CODE_FOR_sse2_cvtsd2siq_2 = 2460,
  CODE_FOR_sse2_cvttsd2si = 2461,
  CODE_FOR_sse2_cvttsd2si_round = 2462,
  CODE_FOR_sse2_cvttsd2siq = 2463,
  CODE_FOR_sse2_cvttsd2siq_round = 2464,
  CODE_FOR_floatv8siv8df2 = 2465,
  CODE_FOR_floatv8siv8df2_mask = 2466,
  CODE_FOR_floatv4siv4df2 = 2467,
  CODE_FOR_floatv4siv4df2_mask = 2468,
  CODE_FOR_floatv8div8df2 = 2469,
  CODE_FOR_floatv8div8df2_round = 2470,
  CODE_FOR_floatv8div8df2_mask = 2471,
  CODE_FOR_floatv8div8df2_mask_round = 2472,
  CODE_FOR_floatunsv8div8df2 = 2473,
  CODE_FOR_floatunsv8div8df2_round = 2474,
  CODE_FOR_floatunsv8div8df2_mask = 2475,
  CODE_FOR_floatunsv8div8df2_mask_round = 2476,
  CODE_FOR_floatv4div4df2 = 2477,
  CODE_FOR_floatv4div4df2_round = 2478,
  CODE_FOR_floatv4div4df2_mask = 2479,
  CODE_FOR_floatv4div4df2_mask_round = 2480,
  CODE_FOR_floatunsv4div4df2 = 2481,
  CODE_FOR_floatunsv4div4df2_round = 2482,
  CODE_FOR_floatunsv4div4df2_mask = 2483,
  CODE_FOR_floatunsv4div4df2_mask_round = 2484,
  CODE_FOR_floatv2div2df2 = 2485,
  CODE_FOR_floatv2div2df2_round = 2486,
  CODE_FOR_floatv2div2df2_mask = 2487,
  CODE_FOR_floatv2div2df2_mask_round = 2488,
  CODE_FOR_floatunsv2div2df2 = 2489,
  CODE_FOR_floatunsv2div2df2_round = 2490,
  CODE_FOR_floatunsv2div2df2_mask = 2491,
  CODE_FOR_floatunsv2div2df2_mask_round = 2492,
  CODE_FOR_floatv8div8sf2 = 2493,
  CODE_FOR_floatv8div8sf2_round = 2494,
  CODE_FOR_floatv8div8sf2_mask = 2495,
  CODE_FOR_floatv8div8sf2_mask_round = 2496,
  CODE_FOR_floatunsv8div8sf2 = 2497,
  CODE_FOR_floatunsv8div8sf2_round = 2498,
  CODE_FOR_floatunsv8div8sf2_mask = 2499,
  CODE_FOR_floatunsv8div8sf2_mask_round = 2500,
  CODE_FOR_floatv4div4sf2 = 2501,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 2502,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2 = 2503,
   CODE_FOR_floatunsv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2_mask = 2504,
   CODE_FOR_floatunsv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv8siv8df2 = 2511,
  CODE_FOR_ufloatv8siv8df2_mask = 2512,
  CODE_FOR_ufloatv4siv4df2 = 2513,
  CODE_FOR_ufloatv4siv4df2_mask = 2514,
  CODE_FOR_ufloatv2siv2df2 = 2515,
  CODE_FOR_ufloatv2siv2df2_mask = 2516,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 2517,
  CODE_FOR_avx_cvtdq2pd256_2 = 2518,
  CODE_FOR_sse2_cvtdq2pd = 2519,
  CODE_FOR_sse2_cvtdq2pd_mask = 2520,
  CODE_FOR_avx512f_cvtpd2dq512 = 2521,
  CODE_FOR_avx512f_cvtpd2dq512_round = 2522,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 2523,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 2524,
  CODE_FOR_avx_cvtpd2dq256 = 2525,
  CODE_FOR_avx_cvtpd2dq256_mask = 2526,
  CODE_FOR_sse2_cvtpd2dq = 2528,
  CODE_FOR_sse2_cvtpd2dq_mask = 2529,
  CODE_FOR_ufix_notruncv8dfv8si2 = 2530,
  CODE_FOR_ufix_notruncv8dfv8si2_round = 2531,
  CODE_FOR_ufix_notruncv8dfv8si2_mask = 2532,
  CODE_FOR_ufix_notruncv8dfv8si2_mask_round = 2533,
  CODE_FOR_ufix_notruncv4dfv4si2 = 2534,
  CODE_FOR_ufix_notruncv4dfv4si2_round = 2535,
  CODE_FOR_ufix_notruncv4dfv4si2_mask = 2536,
  CODE_FOR_ufix_notruncv4dfv4si2_mask_round = 2537,
  CODE_FOR_ufix_notruncv2dfv2si2 = 2538,
  CODE_FOR_ufix_notruncv2dfv2si2_mask = 2539,
  CODE_FOR_fix_truncv8dfv8si2 = 2540,
  CODE_FOR_fix_truncv8dfv8si2_round = 2541,
  CODE_FOR_fix_truncv8dfv8si2_mask = 2542,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 2543,
  CODE_FOR_fixuns_truncv8dfv8si2 = 2544,
  CODE_FOR_fixuns_truncv8dfv8si2_round = 2545,
  CODE_FOR_fixuns_truncv8dfv8si2_mask = 2546,
  CODE_FOR_fixuns_truncv8dfv8si2_mask_round = 2547,
  CODE_FOR_ufix_truncv2dfv2si2 = 2548,
  CODE_FOR_ufix_truncv2dfv2si2_mask = 2549,
  CODE_FOR_fix_truncv4dfv4si2 = 2550,
  CODE_FOR_fix_truncv4dfv4si2_mask = 2551,
  CODE_FOR_ufix_truncv4dfv4si2 = 2552,
  CODE_FOR_ufix_truncv4dfv4si2_mask = 2553,
  CODE_FOR_fix_truncv8dfv8di2 = 2554,
  CODE_FOR_fix_truncv8dfv8di2_round = 2555,
  CODE_FOR_fix_truncv8dfv8di2_mask = 2556,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 2557,
  CODE_FOR_fixuns_truncv8dfv8di2 = 2558,
  CODE_FOR_fixuns_truncv8dfv8di2_round = 2559,
  CODE_FOR_fixuns_truncv8dfv8di2_mask = 2560,
  CODE_FOR_fixuns_truncv8dfv8di2_mask_round = 2561,
  CODE_FOR_fix_truncv4dfv4di2 = 2562,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 2563,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2 = 2564,
   CODE_FOR_fixuns_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2_mask = 2565,
   CODE_FOR_fixuns_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 2566,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 2567,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2 = 2568,
   CODE_FOR_fixuns_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2_mask = 2569,
   CODE_FOR_fixuns_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 2570,
  CODE_FOR_fix_notruncv8dfv8di2_round = 2571,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 2572,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 2573,
  CODE_FOR_fix_notruncv4dfv4di2 = 2574,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 2575,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 2576,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 2577,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv8dfv8di2 = 2578,
  CODE_FOR_ufix_notruncv8dfv8di2_round = 2579,
  CODE_FOR_ufix_notruncv8dfv8di2_mask = 2580,
  CODE_FOR_ufix_notruncv8dfv8di2_mask_round = 2581,
  CODE_FOR_ufix_notruncv4dfv4di2 = 2582,
   CODE_FOR_ufix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv4dfv4di2_mask = 2583,
   CODE_FOR_ufix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2 = 2584,
   CODE_FOR_ufix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2_mask = 2585,
   CODE_FOR_ufix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 2586,
  CODE_FOR_fix_truncv8sfv8di2_round = 2587,
  CODE_FOR_fix_truncv8sfv8di2_mask = 2588,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 2589,
  CODE_FOR_fixuns_truncv8sfv8di2 = 2590,
  CODE_FOR_fixuns_truncv8sfv8di2_round = 2591,
  CODE_FOR_fixuns_truncv8sfv8di2_mask = 2592,
  CODE_FOR_fixuns_truncv8sfv8di2_mask_round = 2593,
  CODE_FOR_fix_truncv4sfv4di2 = 2594,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 2595,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2 = 2596,
   CODE_FOR_fixuns_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2_mask = 2597,
   CODE_FOR_fixuns_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2sfv2di2 = 2598,
  CODE_FOR_fix_truncv2sfv2di2_mask = 2599,
  CODE_FOR_fixuns_truncv2sfv2di2 = 2600,
  CODE_FOR_fixuns_truncv2sfv2di2_mask = 2601,
  CODE_FOR_ufix_truncv8sfv8si2 = 2602,
  CODE_FOR_ufix_truncv8sfv8si2_mask = 2603,
  CODE_FOR_ufix_truncv4sfv4si2 = 2604,
  CODE_FOR_ufix_truncv4sfv4si2_mask = 2605,
  CODE_FOR_sse2_cvttpd2dq = 2606,
  CODE_FOR_sse2_cvttpd2dq_mask = 2607,
  CODE_FOR_sse2_cvtsd2ss = 2608,
  CODE_FOR_sse2_cvtsd2ss_round = 2609,
  CODE_FOR_sse2_cvtss2sd = 2611,
  CODE_FOR_sse2_cvtss2sd_round = 2612,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 2616,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 2617,
  CODE_FOR_avx_cvtpd2ps256 = 2618,
  CODE_FOR_avx_cvtpd2ps256_mask = 2619,
  CODE_FOR_avx512f_cvtps2pd512 = 2622,
  CODE_FOR_avx512f_cvtps2pd512_round = 2623,
  CODE_FOR_avx512f_cvtps2pd512_mask = 2624,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 2625,
  CODE_FOR_avx_cvtps2pd256 = 2626,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 2627,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 2629,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 2630,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 2631,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 2632,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 2633,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 2634,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 2635,
  CODE_FOR_avx512f_cvtd2maskv16si = 2636,
  CODE_FOR_avx512vl_cvtd2maskv8si = 2637,
  CODE_FOR_avx512vl_cvtd2maskv4si = 2638,
  CODE_FOR_avx512f_cvtq2maskv8di = 2639,
  CODE_FOR_avx512vl_cvtq2maskv4di = 2640,
  CODE_FOR_avx512vl_cvtq2maskv2di = 2641,
  CODE_FOR_sse2_cvtps2pd = 2654,
  CODE_FOR_sse2_cvtps2pd_mask = 2655,
  CODE_FOR_sse_movhlps = 2656,
  CODE_FOR_sse_movlhps = 2657,
  CODE_FOR_avx512f_unpckhps512_mask = 2659,
  CODE_FOR_avx_unpckhps256 = 2660,
  CODE_FOR_avx_unpckhps256_mask = 2661,
  CODE_FOR_vec_interleave_highv4sf = 2662,
  CODE_FOR_vec_interleave_highv4sf_mask = 2663,
  CODE_FOR_avx512f_unpcklps512_mask = 2665,
  CODE_FOR_avx_unpcklps256 = 2666,
  CODE_FOR_avx_unpcklps256_mask = 2667,
  CODE_FOR_unpcklps128_mask = 2668,
  CODE_FOR_vec_interleave_lowv4sf = 2669,
  CODE_FOR_avx_movshdup256 = 2670,
  CODE_FOR_avx_movshdup256_mask = 2671,
  CODE_FOR_sse3_movshdup = 2672,
  CODE_FOR_sse3_movshdup_mask = 2673,
  CODE_FOR_avx512f_movshdup512_mask = 2675,
  CODE_FOR_avx_movsldup256 = 2676,
  CODE_FOR_avx_movsldup256_mask = 2677,
  CODE_FOR_sse3_movsldup = 2678,
  CODE_FOR_sse3_movsldup_mask = 2679,
  CODE_FOR_avx512f_movsldup512_mask = 2681,
  CODE_FOR_avx_shufps256_1 = 2682,
  CODE_FOR_avx_shufps256_1_mask = 2683,
  CODE_FOR_sse_shufps_v4sf_mask = 2684,
  CODE_FOR_sse_shufps_v4si = 2685,
  CODE_FOR_sse_shufps_v4sf = 2686,
  CODE_FOR_sse_storehps = 2687,
  CODE_FOR_sse_loadhps = 2688,
  CODE_FOR_sse_storelps = 2689,
  CODE_FOR_sse_loadlps = 2690,
  CODE_FOR_sse_movss = 2691,
  CODE_FOR_avx2_vec_dupv8sf = 2692,
  CODE_FOR_avx2_vec_dupv4sf = 2693,
  CODE_FOR_avx2_vec_dupv8sf_1 = 2694,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 2695,
  CODE_FOR_avx512f_vec_dupv8df_1 = 2696,
  CODE_FOR_vec_setv4si_0 = 2701,
  CODE_FOR_vec_setv4sf_0 = 2702,
  CODE_FOR_vec_setv8si_0 = 2704,
  CODE_FOR_vec_setv8sf_0 = 2705,
  CODE_FOR_vec_setv16si_0 = 2706,
  CODE_FOR_vec_setv16sf_0 = 2707,
  CODE_FOR_sse4_1_insertps = 2708,
  CODE_FOR_avx512dq_vextractf64x2_1_maskm = 2712,
  CODE_FOR_avx512dq_vextracti64x2_1_maskm = 2713,
  CODE_FOR_avx512f_vextractf32x4_1_maskm = 2714,
  CODE_FOR_avx512f_vextracti32x4_1_maskm = 2715,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 2717,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 2719,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 2721,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 2723,
  CODE_FOR_vec_extract_lo_v8df_maskm = 2724,
  CODE_FOR_vec_extract_lo_v8di_maskm = 2725,
  CODE_FOR_vec_extract_lo_v8df = 2726,
  CODE_FOR_vec_extract_lo_v8df_mask = 2727,
  CODE_FOR_vec_extract_lo_v8di = 2728,
  CODE_FOR_vec_extract_lo_v8di_mask = 2729,
  CODE_FOR_vec_extract_hi_v8df_maskm = 2730,
  CODE_FOR_vec_extract_hi_v8di_maskm = 2731,
  CODE_FOR_vec_extract_hi_v8df = 2732,
  CODE_FOR_vec_extract_hi_v8df_mask = 2733,
  CODE_FOR_vec_extract_hi_v8di = 2734,
  CODE_FOR_vec_extract_hi_v8di_mask = 2735,
  CODE_FOR_vec_extract_hi_v16sf_maskm = 2736,
  CODE_FOR_vec_extract_hi_v16si_maskm = 2737,
  CODE_FOR_vec_extract_hi_v16sf = 2738,
  CODE_FOR_vec_extract_hi_v16sf_mask = 2739,
  CODE_FOR_vec_extract_hi_v16si = 2740,
  CODE_FOR_vec_extract_hi_v16si_mask = 2741,
  CODE_FOR_vec_extract_lo_v16sf = 2742,
  CODE_FOR_vec_extract_lo_v16sf_mask = 2743,
  CODE_FOR_vec_extract_lo_v16si = 2744,
  CODE_FOR_vec_extract_lo_v16si_mask = 2745,
  CODE_FOR_vec_extract_lo_v4di = 2746,
  CODE_FOR_vec_extract_lo_v4di_mask = 2747,
  CODE_FOR_vec_extract_lo_v4df = 2748,
  CODE_FOR_vec_extract_lo_v4df_mask = 2749,
  CODE_FOR_vec_extract_hi_v4di = 2750,
  CODE_FOR_vec_extract_hi_v4di_mask = 2751,
  CODE_FOR_vec_extract_hi_v4df = 2752,
  CODE_FOR_vec_extract_hi_v4df_mask = 2753,
  CODE_FOR_vec_extract_lo_v8si = 2754,
  CODE_FOR_vec_extract_lo_v8si_mask = 2755,
  CODE_FOR_vec_extract_lo_v8sf = 2756,
  CODE_FOR_vec_extract_lo_v8sf_mask = 2757,
  CODE_FOR_vec_extract_lo_v8si_maskm = 2758,
  CODE_FOR_vec_extract_lo_v8sf_maskm = 2759,
  CODE_FOR_vec_extract_hi_v8si_maskm = 2760,
  CODE_FOR_vec_extract_hi_v8sf_maskm = 2761,
  CODE_FOR_vec_extract_hi_v8si_mask = 2762,
  CODE_FOR_vec_extract_hi_v8sf_mask = 2763,
  CODE_FOR_vec_extract_hi_v8si = 2764,
  CODE_FOR_vec_extract_hi_v8sf = 2765,
  CODE_FOR_vec_extract_lo_v32hi = 2766,
  CODE_FOR_vec_extract_hi_v32hi = 2767,
  CODE_FOR_vec_extract_lo_v16hi = 2768,
  CODE_FOR_vec_extract_hi_v16hi = 2769,
  CODE_FOR_vec_extract_lo_v64qi = 2770,
  CODE_FOR_vec_extract_hi_v64qi = 2771,
  CODE_FOR_vec_extract_lo_v32qi = 2772,
  CODE_FOR_vec_extract_hi_v32qi = 2773,
  CODE_FOR_avx512f_unpckhpd512_mask = 2775,
  CODE_FOR_avx_unpckhpd256 = 2776,
  CODE_FOR_avx_unpckhpd256_mask = 2777,
  CODE_FOR_avx512vl_unpckhpd128_mask = 2778,
  CODE_FOR_avx512vl_unpcklpd128_mask = 2784,
  CODE_FOR_avx512f_vmscalefv4sf = 2786,
  CODE_FOR_avx512f_vmscalefv4sf_round = 2787,
  CODE_FOR_avx512f_vmscalefv4sf_mask = 2788,
  CODE_FOR_avx512f_vmscalefv4sf_mask_round = 2789,
  CODE_FOR_avx512f_vmscalefv2df = 2790,
  CODE_FOR_avx512f_vmscalefv2df_round = 2791,
  CODE_FOR_avx512f_vmscalefv2df_mask = 2792,
  CODE_FOR_avx512f_vmscalefv2df_mask_round = 2793,
  CODE_FOR_avx512f_scalefv16sf = 2794,
  CODE_FOR_avx512f_scalefv16sf_round = 2795,
  CODE_FOR_avx512f_scalefv16sf_mask = 2796,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 2797,
  CODE_FOR_avx512vl_scalefv8sf = 2798,
  CODE_FOR_avx512vl_scalefv8sf_round = 2799,
  CODE_FOR_avx512vl_scalefv8sf_mask = 2800,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 2801,
  CODE_FOR_avx512vl_scalefv4sf = 2802,
  CODE_FOR_avx512vl_scalefv4sf_round = 2803,
  CODE_FOR_avx512vl_scalefv4sf_mask = 2804,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 2805,
  CODE_FOR_avx512f_scalefv8df = 2806,
  CODE_FOR_avx512f_scalefv8df_round = 2807,
  CODE_FOR_avx512f_scalefv8df_mask = 2808,
  CODE_FOR_avx512f_scalefv8df_mask_round = 2809,
  CODE_FOR_avx512vl_scalefv4df = 2810,
  CODE_FOR_avx512vl_scalefv4df_round = 2811,
  CODE_FOR_avx512vl_scalefv4df_mask = 2812,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 2813,
  CODE_FOR_avx512vl_scalefv2df = 2814,
  CODE_FOR_avx512vl_scalefv2df_round = 2815,
  CODE_FOR_avx512vl_scalefv2df_mask = 2816,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 2817,
  CODE_FOR_avx512f_vternlogv16si = 2818,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 2819,
  CODE_FOR_avx512vl_vternlogv8si = 2820,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 2821,
  CODE_FOR_avx512vl_vternlogv4si = 2822,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 2823,
  CODE_FOR_avx512f_vternlogv8di = 2824,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 2825,
  CODE_FOR_avx512vl_vternlogv4di = 2826,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 2827,
  CODE_FOR_avx512vl_vternlogv2di = 2828,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 2829,
  CODE_FOR_avx512f_vternlogv16si_mask = 2830,
  CODE_FOR_avx512vl_vternlogv8si_mask = 2831,
  CODE_FOR_avx512vl_vternlogv4si_mask = 2832,
  CODE_FOR_avx512f_vternlogv8di_mask = 2833,
  CODE_FOR_avx512vl_vternlogv4di_mask = 2834,
  CODE_FOR_avx512vl_vternlogv2di_mask = 2835,
  CODE_FOR_avx512f_getexpv16sf = 2836,
  CODE_FOR_avx512f_getexpv16sf_round = 2837,
  CODE_FOR_avx512f_getexpv16sf_mask = 2838,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 2839,
  CODE_FOR_avx512vl_getexpv8sf = 2840,
  CODE_FOR_avx512vl_getexpv8sf_round = 2841,
  CODE_FOR_avx512vl_getexpv8sf_mask = 2842,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 2843,
  CODE_FOR_avx512vl_getexpv4sf = 2844,
  CODE_FOR_avx512vl_getexpv4sf_round = 2845,
  CODE_FOR_avx512vl_getexpv4sf_mask = 2846,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 2847,
  CODE_FOR_avx512f_getexpv8df = 2848,
  CODE_FOR_avx512f_getexpv8df_round = 2849,
  CODE_FOR_avx512f_getexpv8df_mask = 2850,
  CODE_FOR_avx512f_getexpv8df_mask_round = 2851,
  CODE_FOR_avx512vl_getexpv4df = 2852,
  CODE_FOR_avx512vl_getexpv4df_round = 2853,
  CODE_FOR_avx512vl_getexpv4df_mask = 2854,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 2855,
  CODE_FOR_avx512vl_getexpv2df = 2856,
  CODE_FOR_avx512vl_getexpv2df_round = 2857,
  CODE_FOR_avx512vl_getexpv2df_mask = 2858,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 2859,
  CODE_FOR_avx512f_sgetexpv4sf = 2860,
  CODE_FOR_avx512f_sgetexpv4sf_mask = 2861,
  CODE_FOR_avx512f_sgetexpv4sf_round = 2862,
  CODE_FOR_avx512f_sgetexpv4sf_mask_round = 2863,
  CODE_FOR_avx512f_sgetexpv2df = 2864,
  CODE_FOR_avx512f_sgetexpv2df_mask = 2865,
  CODE_FOR_avx512f_sgetexpv2df_round = 2866,
  CODE_FOR_avx512f_sgetexpv2df_mask_round = 2867,
  CODE_FOR_avx512f_alignv16si_mask = 2869,
  CODE_FOR_avx512vl_alignv8si_mask = 2871,
  CODE_FOR_avx512vl_alignv4si_mask = 2873,
  CODE_FOR_avx512f_alignv8di_mask = 2875,
  CODE_FOR_avx512vl_alignv4di_mask = 2877,
  CODE_FOR_avx512vl_alignv2di_mask = 2879,
  CODE_FOR_avx512f_fixupimmv16sf = 2880,
  CODE_FOR_avx512f_fixupimmv16sf_round = 2881,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 2882,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 2883,
  CODE_FOR_avx512vl_fixupimmv8sf = 2884,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 2885,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 2886,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 2887,
  CODE_FOR_avx512vl_fixupimmv4sf = 2888,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 2889,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 2890,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 2891,
  CODE_FOR_avx512f_fixupimmv8df = 2892,
  CODE_FOR_avx512f_fixupimmv8df_round = 2893,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 2894,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 2895,
  CODE_FOR_avx512vl_fixupimmv4df = 2896,
  CODE_FOR_avx512vl_fixupimmv4df_round = 2897,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 2898,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 2899,
  CODE_FOR_avx512vl_fixupimmv2df = 2900,
  CODE_FOR_avx512vl_fixupimmv2df_round = 2901,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 2902,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 2903,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 2904,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 2905,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 2906,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 2907,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 2908,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 2909,
  CODE_FOR_avx512f_fixupimmv8df_mask = 2910,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 2911,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 2912,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 2913,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 2914,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 2915,
  CODE_FOR_avx512f_sfixupimmv4sf = 2916,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 2917,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 2918,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 2919,
  CODE_FOR_avx512f_sfixupimmv2df = 2920,
  CODE_FOR_avx512f_sfixupimmv2df_round = 2921,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 2922,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 2923,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 2924,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 2925,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 2926,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 2927,
  CODE_FOR_avx512f_rndscalev16sf = 2928,
  CODE_FOR_avx512f_rndscalev16sf_round = 2929,
  CODE_FOR_avx512f_rndscalev16sf_mask = 2930,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 2931,
  CODE_FOR_avx512vl_rndscalev8sf = 2932,
  CODE_FOR_avx512vl_rndscalev8sf_round = 2933,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 2934,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 2935,
  CODE_FOR_avx512vl_rndscalev4sf = 2936,
  CODE_FOR_avx512vl_rndscalev4sf_round = 2937,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 2938,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 2939,
  CODE_FOR_avx512f_rndscalev8df = 2940,
  CODE_FOR_avx512f_rndscalev8df_round = 2941,
  CODE_FOR_avx512f_rndscalev8df_mask = 2942,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 2943,
  CODE_FOR_avx512vl_rndscalev4df = 2944,
  CODE_FOR_avx512vl_rndscalev4df_round = 2945,
  CODE_FOR_avx512vl_rndscalev4df_mask = 2946,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 2947,
  CODE_FOR_avx512vl_rndscalev2df = 2948,
  CODE_FOR_avx512vl_rndscalev2df_round = 2949,
  CODE_FOR_avx512vl_rndscalev2df_mask = 2950,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 2951,
  CODE_FOR_avx512f_rndscalev4sf = 2952,
  CODE_FOR_avx512f_rndscalev4sf_round = 2953,
  CODE_FOR_avx512f_rndscalev2df = 2954,
  CODE_FOR_avx512f_rndscalev2df_round = 2955,
  CODE_FOR_avx512f_shufps512_1 = 2956,
  CODE_FOR_avx512f_shufps512_1_mask = 2957,
  CODE_FOR_avx512f_shufpd512_1 = 2958,
  CODE_FOR_avx512f_shufpd512_1_mask = 2959,
  CODE_FOR_avx_shufpd256_1 = 2960,
  CODE_FOR_avx_shufpd256_1_mask = 2961,
  CODE_FOR_sse2_shufpd_v2df_mask = 2962,
  CODE_FOR_avx2_interleave_highv4di = 2963,
  CODE_FOR_avx2_interleave_highv4di_mask = 2964,
  CODE_FOR_avx512f_interleave_highv8di_mask = 2966,
  CODE_FOR_vec_interleave_highv2di = 2967,
  CODE_FOR_vec_interleave_highv2di_mask = 2968,
  CODE_FOR_avx2_interleave_lowv4di = 2969,
  CODE_FOR_avx2_interleave_lowv4di_mask = 2970,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 2972,
  CODE_FOR_vec_interleave_lowv2di = 2973,
  CODE_FOR_vec_interleave_lowv2di_mask = 2974,
  CODE_FOR_sse2_shufpd_v2di = 2975,
  CODE_FOR_sse2_shufpd_v2df = 2976,
  CODE_FOR_sse2_storehpd = 2977,
  CODE_FOR_sse2_storelpd = 2979,
  CODE_FOR_sse2_loadhpd = 2981,
  CODE_FOR_sse2_loadlpd = 2982,
  CODE_FOR_sse2_movsd = 2983,
  CODE_FOR_vec_dupv2df = 2984,
  CODE_FOR_vec_dupv2df_mask = 2985,
  CODE_FOR_vec_concatv2df = 2986,
  CODE_FOR_vec_setv8df_0 = 2987,
  CODE_FOR_vec_setv4df_0 = 2988,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 3001,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 3002,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 3003,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 3004,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 3005,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 3006,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 3007,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 3008,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 3009,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 3010,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 3011,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 3012,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 3013,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 3014,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 3015,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 3016,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 3017,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 3018,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 3028,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 3029,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 3030,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 3031,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 3032,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 3033,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 3034,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 3035,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 3036,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 3055,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 3056,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 3057,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store = 3061,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store = 3062,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store = 3063,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 3070,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 3071,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 3072,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 3073,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 3074,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 3075,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store = 3082,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store = 3083,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store = 3084,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store = 3085,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store = 3086,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store = 3087,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 3094,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 3095,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 3096,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 3097,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 3098,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 3099,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store = 3106,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store = 3107,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store = 3108,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store = 3109,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store = 3110,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store = 3111,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 3127,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 3128,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 3129,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 3130,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 3131,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 3132,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store = 3139,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store = 3140,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store = 3141,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store = 3142,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store = 3143,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store = 3144,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 3148,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 3149,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 3150,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store = 3154,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store = 3155,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store = 3156,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 3163,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 3164,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 3165,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store = 3169,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store = 3170,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store = 3171,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 3178,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 3179,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 3180,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store = 3184,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store = 3185,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store = 3186,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 3325,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 3326,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 3327,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 3328,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 3329,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 3330,
  CODE_FOR_avx512dq_mulv8di3 = 3333,
  CODE_FOR_avx512dq_mulv8di3_mask = 3334,
  CODE_FOR_avx512dq_mulv4di3 = 3335,
  CODE_FOR_avx512dq_mulv4di3_mask = 3336,
  CODE_FOR_avx512dq_mulv2di3 = 3337,
  CODE_FOR_avx512dq_mulv2di3_mask = 3338,
  CODE_FOR_ashrv16hi3_mask = 3346,
  CODE_FOR_ashrv8hi3_mask = 3348,
  CODE_FOR_ashrv8si3_mask = 3350,
  CODE_FOR_ashrv4si3_mask = 3352,
  CODE_FOR_ashrv2di3_mask = 3354,
  CODE_FOR_ashrv16hi3 = 3355,
  CODE_FOR_ashrv8hi3 = 3356,
  CODE_FOR_ashrv8si3 = 3357,
  CODE_FOR_ashrv4si3 = 3358,
  CODE_FOR_ashrv32hi3 = 3359,
  CODE_FOR_ashrv32hi3_mask = 3360,
  CODE_FOR_ashrv4di3 = 3361,
  CODE_FOR_ashrv4di3_mask = 3362,
  CODE_FOR_ashrv16si3 = 3363,
  CODE_FOR_ashrv16si3_mask = 3364,
  CODE_FOR_ashrv8di3 = 3365,
  CODE_FOR_ashrv8di3_mask = 3366,
  CODE_FOR_ashlv16hi3_mask = 3368,
  CODE_FOR_lshrv16hi3_mask = 3370,
  CODE_FOR_ashlv8hi3_mask = 3372,
  CODE_FOR_lshrv8hi3_mask = 3374,
  CODE_FOR_ashlv8si3_mask = 3376,
  CODE_FOR_lshrv8si3_mask = 3378,
  CODE_FOR_ashlv4si3_mask = 3380,
  CODE_FOR_lshrv4si3_mask = 3382,
  CODE_FOR_ashlv4di3_mask = 3384,
  CODE_FOR_lshrv4di3_mask = 3386,
  CODE_FOR_ashlv2di3_mask = 3388,
  CODE_FOR_lshrv2di3_mask = 3390,
  CODE_FOR_ashlv16hi3 = 3391,
  CODE_FOR_lshrv16hi3 = 3392,
  CODE_FOR_ashlv8hi3 = 3393,
  CODE_FOR_lshrv8hi3 = 3394,
  CODE_FOR_ashlv8si3 = 3395,
  CODE_FOR_lshrv8si3 = 3396,
  CODE_FOR_ashlv4si3 = 3397,
  CODE_FOR_lshrv4si3 = 3398,
  CODE_FOR_ashlv4di3 = 3399,
  CODE_FOR_lshrv4di3 = 3400,
  CODE_FOR_ashlv2di3 = 3401,
  CODE_FOR_lshrv2di3 = 3402,
  CODE_FOR_ashlv32hi3 = 3403,
  CODE_FOR_ashlv32hi3_mask = 3404,
  CODE_FOR_lshrv32hi3 = 3405,
  CODE_FOR_lshrv32hi3_mask = 3406,
  CODE_FOR_ashlv16si3 = 3407,
  CODE_FOR_ashlv16si3_mask = 3408,
  CODE_FOR_lshrv16si3 = 3409,
  CODE_FOR_lshrv16si3_mask = 3410,
  CODE_FOR_ashlv8di3 = 3411,
  CODE_FOR_ashlv8di3_mask = 3412,
  CODE_FOR_lshrv8di3 = 3413,
  CODE_FOR_lshrv8di3_mask = 3414,
  CODE_FOR_avx512bw_ashlv4ti3 = 3415,
  CODE_FOR_avx512bw_lshrv4ti3 = 3416,
  CODE_FOR_avx512bw_ashlv2ti3 = 3417,
  CODE_FOR_avx512bw_lshrv2ti3 = 3418,
  CODE_FOR_avx512bw_ashlv1ti3 = 3419,
  CODE_FOR_avx512bw_lshrv1ti3 = 3420,
  CODE_FOR_avx2_ashlv2ti3 = 3421,
  CODE_FOR_avx2_lshrv2ti3 = 3422,
  CODE_FOR_sse2_ashlv1ti3 = 3423,
  CODE_FOR_sse2_lshrv1ti3 = 3424,
  CODE_FOR_avx512f_rolvv16si = 3425,
  CODE_FOR_avx512f_rolvv16si_mask = 3426,
  CODE_FOR_avx512f_rorvv16si = 3427,
  CODE_FOR_avx512f_rorvv16si_mask = 3428,
  CODE_FOR_avx512vl_rolvv8si = 3429,
  CODE_FOR_avx512vl_rolvv8si_mask = 3430,
  CODE_FOR_avx512vl_rorvv8si = 3431,
  CODE_FOR_avx512vl_rorvv8si_mask = 3432,
  CODE_FOR_avx512vl_rolvv4si = 3433,
  CODE_FOR_avx512vl_rolvv4si_mask = 3434,
  CODE_FOR_avx512vl_rorvv4si = 3435,
  CODE_FOR_avx512vl_rorvv4si_mask = 3436,
  CODE_FOR_avx512f_rolvv8di = 3437,
  CODE_FOR_avx512f_rolvv8di_mask = 3438,
  CODE_FOR_avx512f_rorvv8di = 3439,
  CODE_FOR_avx512f_rorvv8di_mask = 3440,
  CODE_FOR_avx512vl_rolvv4di = 3441,
  CODE_FOR_avx512vl_rolvv4di_mask = 3442,
  CODE_FOR_avx512vl_rorvv4di = 3443,
  CODE_FOR_avx512vl_rorvv4di_mask = 3444,
  CODE_FOR_avx512vl_rolvv2di = 3445,
  CODE_FOR_avx512vl_rolvv2di_mask = 3446,
  CODE_FOR_avx512vl_rorvv2di = 3447,
  CODE_FOR_avx512vl_rorvv2di_mask = 3448,
  CODE_FOR_avx512f_rolv16si = 3449,
  CODE_FOR_avx512f_rolv16si_mask = 3450,
  CODE_FOR_avx512f_rorv16si = 3451,
  CODE_FOR_avx512f_rorv16si_mask = 3452,
  CODE_FOR_avx512vl_rolv8si = 3453,
  CODE_FOR_avx512vl_rolv8si_mask = 3454,
  CODE_FOR_avx512vl_rorv8si = 3455,
  CODE_FOR_avx512vl_rorv8si_mask = 3456,
  CODE_FOR_avx512vl_rolv4si = 3457,
  CODE_FOR_avx512vl_rolv4si_mask = 3458,
  CODE_FOR_avx512vl_rorv4si = 3459,
  CODE_FOR_avx512vl_rorv4si_mask = 3460,
  CODE_FOR_avx512f_rolv8di = 3461,
  CODE_FOR_avx512f_rolv8di_mask = 3462,
  CODE_FOR_avx512f_rorv8di = 3463,
  CODE_FOR_avx512f_rorv8di_mask = 3464,
  CODE_FOR_avx512vl_rolv4di = 3465,
  CODE_FOR_avx512vl_rolv4di_mask = 3466,
  CODE_FOR_avx512vl_rorv4di = 3467,
  CODE_FOR_avx512vl_rorv4di_mask = 3468,
  CODE_FOR_avx512vl_rolv2di = 3469,
  CODE_FOR_avx512vl_rolv2di_mask = 3470,
  CODE_FOR_avx512vl_rorv2di = 3471,
  CODE_FOR_avx512vl_rorv2di_mask = 3472,
  CODE_FOR_smaxv64qi3_mask = 3534,
  CODE_FOR_sminv64qi3_mask = 3536,
  CODE_FOR_umaxv64qi3_mask = 3538,
  CODE_FOR_uminv64qi3_mask = 3540,
  CODE_FOR_smaxv16qi3_mask = 3542,
  CODE_FOR_sminv16qi3_mask = 3544,
  CODE_FOR_umaxv16qi3_mask = 3546,
  CODE_FOR_uminv16qi3_mask = 3548,
  CODE_FOR_smaxv32qi3_mask = 3550,
  CODE_FOR_sminv32qi3_mask = 3552,
  CODE_FOR_umaxv32qi3_mask = 3554,
  CODE_FOR_uminv32qi3_mask = 3556,
  CODE_FOR_smaxv32hi3_mask = 3558,
  CODE_FOR_sminv32hi3_mask = 3560,
  CODE_FOR_umaxv32hi3_mask = 3562,
  CODE_FOR_uminv32hi3_mask = 3564,
  CODE_FOR_smaxv16hi3_mask = 3566,
  CODE_FOR_sminv16hi3_mask = 3568,
  CODE_FOR_umaxv16hi3_mask = 3570,
  CODE_FOR_uminv16hi3_mask = 3572,
  CODE_FOR_smaxv8hi3_mask = 3574,
  CODE_FOR_sminv8hi3_mask = 3576,
  CODE_FOR_umaxv8hi3_mask = 3578,
  CODE_FOR_uminv8hi3_mask = 3580,
  CODE_FOR_avx512bw_eqv64qi3_1 = 3605,
  CODE_FOR_avx512bw_eqv64qi3_mask_1 = 3606,
  CODE_FOR_avx512vl_eqv16qi3_1 = 3607,
  CODE_FOR_avx512vl_eqv16qi3_mask_1 = 3608,
  CODE_FOR_avx512vl_eqv32qi3_1 = 3609,
  CODE_FOR_avx512vl_eqv32qi3_mask_1 = 3610,
  CODE_FOR_avx512bw_eqv32hi3_1 = 3611,
  CODE_FOR_avx512bw_eqv32hi3_mask_1 = 3612,
  CODE_FOR_avx512vl_eqv16hi3_1 = 3613,
  CODE_FOR_avx512vl_eqv16hi3_mask_1 = 3614,
  CODE_FOR_avx512vl_eqv8hi3_1 = 3615,
  CODE_FOR_avx512vl_eqv8hi3_mask_1 = 3616,
  CODE_FOR_avx512f_eqv16si3_1 = 3617,
  CODE_FOR_avx512f_eqv16si3_mask_1 = 3618,
  CODE_FOR_avx512vl_eqv8si3_1 = 3619,
  CODE_FOR_avx512vl_eqv8si3_mask_1 = 3620,
  CODE_FOR_avx512vl_eqv4si3_1 = 3621,
  CODE_FOR_avx512vl_eqv4si3_mask_1 = 3622,
  CODE_FOR_avx512f_eqv8di3_1 = 3623,
  CODE_FOR_avx512f_eqv8di3_mask_1 = 3624,
  CODE_FOR_avx512vl_eqv4di3_1 = 3625,
  CODE_FOR_avx512vl_eqv4di3_mask_1 = 3626,
  CODE_FOR_avx512vl_eqv2di3_1 = 3627,
  CODE_FOR_avx512vl_eqv2di3_mask_1 = 3628,
  CODE_FOR_sse4_2_gtv2di3 = 3633,
  CODE_FOR_avx2_gtv32qi3 = 3634,
  CODE_FOR_avx2_gtv16hi3 = 3635,
  CODE_FOR_avx2_gtv8si3 = 3636,
  CODE_FOR_avx2_gtv4di3 = 3637,
  CODE_FOR_avx512f_gtv16si3 = 3638,
  CODE_FOR_avx512f_gtv16si3_mask = 3639,
  CODE_FOR_avx512vl_gtv8si3 = 3640,
  CODE_FOR_avx512vl_gtv8si3_mask = 3641,
  CODE_FOR_avx512vl_gtv4si3 = 3642,
  CODE_FOR_avx512vl_gtv4si3_mask = 3643,
  CODE_FOR_avx512f_gtv8di3 = 3644,
  CODE_FOR_avx512f_gtv8di3_mask = 3645,
  CODE_FOR_avx512vl_gtv4di3 = 3646,
  CODE_FOR_avx512vl_gtv4di3_mask = 3647,
  CODE_FOR_avx512vl_gtv2di3 = 3648,
  CODE_FOR_avx512vl_gtv2di3_mask = 3649,
  CODE_FOR_avx512bw_gtv64qi3 = 3650,
  CODE_FOR_avx512bw_gtv64qi3_mask = 3651,
  CODE_FOR_avx512vl_gtv16qi3 = 3652,
  CODE_FOR_avx512vl_gtv16qi3_mask = 3653,
  CODE_FOR_avx512vl_gtv32qi3 = 3654,
  CODE_FOR_avx512vl_gtv32qi3_mask = 3655,
  CODE_FOR_avx512bw_gtv32hi3 = 3656,
  CODE_FOR_avx512bw_gtv32hi3_mask = 3657,
  CODE_FOR_avx512vl_gtv16hi3 = 3658,
  CODE_FOR_avx512vl_gtv16hi3_mask = 3659,
  CODE_FOR_avx512vl_gtv8hi3 = 3660,
  CODE_FOR_avx512vl_gtv8hi3_mask = 3661,
  CODE_FOR_sse2_gtv16qi3 = 3662,
  CODE_FOR_sse2_gtv8hi3 = 3663,
  CODE_FOR_sse2_gtv4si3 = 3664,
  CODE_FOR_andv16si3_mask = 3690,
  CODE_FOR_iorv16si3_mask = 3692,
  CODE_FOR_xorv16si3_mask = 3694,
  CODE_FOR_andv8si3_mask = 3696,
  CODE_FOR_iorv8si3_mask = 3698,
  CODE_FOR_xorv8si3_mask = 3700,
  CODE_FOR_andv4si3_mask = 3702,
  CODE_FOR_iorv4si3_mask = 3704,
  CODE_FOR_xorv4si3_mask = 3706,
  CODE_FOR_andv8di3_mask = 3708,
  CODE_FOR_iorv8di3_mask = 3710,
  CODE_FOR_xorv8di3_mask = 3712,
  CODE_FOR_andv4di3_mask = 3714,
  CODE_FOR_iorv4di3_mask = 3716,
  CODE_FOR_xorv4di3_mask = 3718,
  CODE_FOR_andv2di3_mask = 3720,
  CODE_FOR_iorv2di3_mask = 3722,
  CODE_FOR_xorv2di3_mask = 3724,
  CODE_FOR_avx512bw_testmv64qi3 = 3779,
  CODE_FOR_avx512bw_testmv64qi3_mask = 3780,
  CODE_FOR_avx512vl_testmv32qi3 = 3781,
  CODE_FOR_avx512vl_testmv32qi3_mask = 3782,
  CODE_FOR_avx512vl_testmv16qi3 = 3783,
  CODE_FOR_avx512vl_testmv16qi3_mask = 3784,
  CODE_FOR_avx512bw_testmv32hi3 = 3785,
  CODE_FOR_avx512bw_testmv32hi3_mask = 3786,
  CODE_FOR_avx512vl_testmv16hi3 = 3787,
  CODE_FOR_avx512vl_testmv16hi3_mask = 3788,
  CODE_FOR_avx512vl_testmv8hi3 = 3789,
  CODE_FOR_avx512vl_testmv8hi3_mask = 3790,
  CODE_FOR_avx512f_testmv16si3 = 3791,
  CODE_FOR_avx512f_testmv16si3_mask = 3792,
  CODE_FOR_avx512vl_testmv8si3 = 3793,
  CODE_FOR_avx512vl_testmv8si3_mask = 3794,
  CODE_FOR_avx512vl_testmv4si3 = 3795,
  CODE_FOR_avx512vl_testmv4si3_mask = 3796,
  CODE_FOR_avx512f_testmv8di3 = 3797,
  CODE_FOR_avx512f_testmv8di3_mask = 3798,
  CODE_FOR_avx512vl_testmv4di3 = 3799,
  CODE_FOR_avx512vl_testmv4di3_mask = 3800,
  CODE_FOR_avx512vl_testmv2di3 = 3801,
  CODE_FOR_avx512vl_testmv2di3_mask = 3802,
  CODE_FOR_avx512bw_testnmv64qi3 = 3803,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 3804,
  CODE_FOR_avx512vl_testnmv32qi3 = 3805,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 3806,
  CODE_FOR_avx512vl_testnmv16qi3 = 3807,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 3808,
  CODE_FOR_avx512bw_testnmv32hi3 = 3809,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 3810,
  CODE_FOR_avx512vl_testnmv16hi3 = 3811,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 3812,
  CODE_FOR_avx512vl_testnmv8hi3 = 3813,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 3814,
  CODE_FOR_avx512f_testnmv16si3 = 3815,
  CODE_FOR_avx512f_testnmv16si3_mask = 3816,
  CODE_FOR_avx512vl_testnmv8si3 = 3817,
  CODE_FOR_avx512vl_testnmv8si3_mask = 3818,
  CODE_FOR_avx512vl_testnmv4si3 = 3819,
  CODE_FOR_avx512vl_testnmv4si3_mask = 3820,
  CODE_FOR_avx512f_testnmv8di3 = 3821,
  CODE_FOR_avx512f_testnmv8di3_mask = 3822,
  CODE_FOR_avx512vl_testnmv4di3 = 3823,
  CODE_FOR_avx512vl_testnmv4di3_mask = 3824,
  CODE_FOR_avx512vl_testnmv2di3 = 3825,
  CODE_FOR_avx512vl_testnmv2di3_mask = 3826,
  CODE_FOR_avx512bw_packsswb = 3971,
  CODE_FOR_avx512bw_packsswb_mask = 3972,
  CODE_FOR_avx2_packsswb = 3973,
  CODE_FOR_avx2_packsswb_mask = 3974,
  CODE_FOR_sse2_packsswb = 3975,
  CODE_FOR_sse2_packsswb_mask = 3976,
  CODE_FOR_avx512bw_packssdw = 3977,
  CODE_FOR_avx512bw_packssdw_mask = 3978,
  CODE_FOR_avx2_packssdw = 3979,
  CODE_FOR_avx2_packssdw_mask = 3980,
  CODE_FOR_sse2_packssdw = 3981,
  CODE_FOR_sse2_packssdw_mask = 3982,
  CODE_FOR_avx512bw_packuswb = 3983,
  CODE_FOR_avx512bw_packuswb_mask = 3984,
  CODE_FOR_avx2_packuswb = 3985,
  CODE_FOR_avx2_packuswb_mask = 3986,
  CODE_FOR_sse2_packuswb = 3987,
  CODE_FOR_sse2_packuswb_mask = 3988,
  CODE_FOR_avx512bw_interleave_highv64qi = 3989,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 3990,
  CODE_FOR_avx2_interleave_highv32qi = 3991,
  CODE_FOR_avx2_interleave_highv32qi_mask = 3992,
  CODE_FOR_vec_interleave_highv16qi = 3993,
  CODE_FOR_vec_interleave_highv16qi_mask = 3994,
  CODE_FOR_avx512bw_interleave_lowv64qi = 3995,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 3996,
  CODE_FOR_avx2_interleave_lowv32qi = 3997,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 3998,
  CODE_FOR_vec_interleave_lowv16qi = 3999,
  CODE_FOR_vec_interleave_lowv16qi_mask = 4000,
  CODE_FOR_avx512bw_interleave_highv32hi = 4001,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 4002,
  CODE_FOR_avx2_interleave_highv16hi = 4003,
  CODE_FOR_avx2_interleave_highv16hi_mask = 4004,
  CODE_FOR_vec_interleave_highv8hi = 4005,
  CODE_FOR_vec_interleave_highv8hi_mask = 4006,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 4008,
  CODE_FOR_avx2_interleave_lowv16hi = 4009,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 4010,
  CODE_FOR_vec_interleave_lowv8hi = 4011,
  CODE_FOR_vec_interleave_lowv8hi_mask = 4012,
  CODE_FOR_avx2_interleave_highv8si = 4013,
  CODE_FOR_avx2_interleave_highv8si_mask = 4014,
  CODE_FOR_avx512f_interleave_highv16si_mask = 4016,
  CODE_FOR_vec_interleave_highv4si = 4017,
  CODE_FOR_vec_interleave_highv4si_mask = 4018,
  CODE_FOR_avx2_interleave_lowv8si = 4019,
  CODE_FOR_avx2_interleave_lowv8si_mask = 4020,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 4022,
  CODE_FOR_vec_interleave_lowv4si = 4023,
  CODE_FOR_vec_interleave_lowv4si_mask = 4024,
  CODE_FOR_sse4_1_pinsrb = 4025,
  CODE_FOR_sse2_pinsrw = 4026,
  CODE_FOR_sse4_1_pinsrd = 4027,
  CODE_FOR_sse4_1_pinsrq = 4028,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 4034,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 4036,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 4038,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 4040,
  CODE_FOR_vec_set_lo_v16sf = 4041,
  CODE_FOR_vec_set_lo_v16sf_mask = 4042,
  CODE_FOR_vec_set_lo_v16si = 4043,
  CODE_FOR_vec_set_lo_v16si_mask = 4044,
  CODE_FOR_vec_set_hi_v16sf = 4045,
  CODE_FOR_vec_set_hi_v16sf_mask = 4046,
  CODE_FOR_vec_set_hi_v16si = 4047,
  CODE_FOR_vec_set_hi_v16si_mask = 4048,
  CODE_FOR_vec_set_lo_v8df = 4049,
  CODE_FOR_vec_set_lo_v8df_mask = 4050,
  CODE_FOR_vec_set_lo_v8di = 4051,
  CODE_FOR_vec_set_lo_v8di_mask = 4052,
  CODE_FOR_vec_set_hi_v8df = 4053,
  CODE_FOR_vec_set_hi_v8df_mask = 4054,
  CODE_FOR_vec_set_hi_v8di = 4055,
  CODE_FOR_vec_set_hi_v8di_mask = 4056,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 4058,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 4060,
  CODE_FOR_avx512f_shuf_f64x2_1 = 4061,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 4062,
  CODE_FOR_avx512f_shuf_i64x2_1 = 4063,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 4064,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 4065,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 4066,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 4067,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 4068,
  CODE_FOR_avx512f_shuf_f32x4_1 = 4069,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 4070,
  CODE_FOR_avx512f_shuf_i32x4_1 = 4071,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 4072,
  CODE_FOR_avx512f_pshufd_1 = 4073,
  CODE_FOR_avx512f_pshufd_1_mask = 4074,
  CODE_FOR_avx2_pshufd_1 = 4075,
  CODE_FOR_avx2_pshufd_1_mask = 4076,
  CODE_FOR_sse2_pshufd_1 = 4077,
  CODE_FOR_sse2_pshufd_1_mask = 4078,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 4080,
  CODE_FOR_avx2_pshuflw_1 = 4081,
  CODE_FOR_avx2_pshuflw_1_mask = 4082,
  CODE_FOR_sse2_pshuflw_1 = 4083,
  CODE_FOR_sse2_pshuflw_1_mask = 4084,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 4086,
  CODE_FOR_avx2_pshufhw_1 = 4087,
  CODE_FOR_avx2_pshufhw_1_mask = 4088,
  CODE_FOR_sse2_pshufhw_1 = 4089,
  CODE_FOR_sse2_pshufhw_1_mask = 4090,
  CODE_FOR_sse2_loadld = 4091,
  CODE_FOR_vec_concatv2di = 4116,
  CODE_FOR_vec_setv8di_0 = 4118,
  CODE_FOR_vec_setv4di_0 = 4119,
  CODE_FOR_avx512f_psadbw = 4132,
  CODE_FOR_avx2_psadbw = 4133,
  CODE_FOR_sse2_psadbw = 4134,
  CODE_FOR_avx_movmskps256 = 4135,
  CODE_FOR_sse_movmskps = 4136,
  CODE_FOR_avx_movmskpd256 = 4137,
  CODE_FOR_sse2_movmskpd = 4138,
  CODE_FOR_avx2_pmovmskb = 4159,
  CODE_FOR_sse2_pmovmskb = 4160,
  CODE_FOR_sse_ldmxcsr = 4169,
  CODE_FOR_sse_stmxcsr = 4170,
  CODE_FOR_sse2_clflush = 4171,
  CODE_FOR_sse3_mwait = 4172,
  CODE_FOR_sse3_monitor_si = 4173,
  CODE_FOR_sse3_monitor_di = 4174,
  CODE_FOR_avx2_phaddwv16hi3 = 4175,
  CODE_FOR_avx2_phaddswv16hi3 = 4176,
  CODE_FOR_avx2_phsubwv16hi3 = 4177,
  CODE_FOR_avx2_phsubswv16hi3 = 4178,
  CODE_FOR_ssse3_phaddwv8hi3 = 4179,
  CODE_FOR_ssse3_phaddswv8hi3 = 4180,
  CODE_FOR_ssse3_phsubwv8hi3 = 4181,
  CODE_FOR_ssse3_phsubswv8hi3 = 4182,
  CODE_FOR_ssse3_phaddwv4hi3 = 4183,
  CODE_FOR_ssse3_phaddswv4hi3 = 4184,
  CODE_FOR_ssse3_phsubwv4hi3 = 4185,
  CODE_FOR_ssse3_phsubswv4hi3 = 4186,
  CODE_FOR_avx2_phadddv8si3 = 4187,
  CODE_FOR_avx2_phsubdv8si3 = 4188,
  CODE_FOR_ssse3_phadddv4si3 = 4189,
  CODE_FOR_ssse3_phsubdv4si3 = 4190,
  CODE_FOR_ssse3_phadddv2si3 = 4191,
  CODE_FOR_ssse3_phsubdv2si3 = 4192,
  CODE_FOR_avx2_pmaddubsw256 = 4193,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 4194,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 4195,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 4196,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 4197,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 4198,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 4199,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 4200,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 4201,
  CODE_FOR_ssse3_pmaddubsw128 = 4202,
  CODE_FOR_ssse3_pmaddubsw = 4203,
  CODE_FOR_avx512bw_pshufbv64qi3 = 4211,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 4212,
  CODE_FOR_avx2_pshufbv32qi3 = 4213,
  CODE_FOR_avx2_pshufbv32qi3_mask = 4214,
  CODE_FOR_ssse3_pshufbv16qi3 = 4215,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 4216,
  CODE_FOR_ssse3_pshufbv8qi3 = 4217,
  CODE_FOR_avx2_psignv32qi3 = 4218,
  CODE_FOR_ssse3_psignv16qi3 = 4219,
  CODE_FOR_avx2_psignv16hi3 = 4220,
  CODE_FOR_ssse3_psignv8hi3 = 4221,
  CODE_FOR_avx2_psignv8si3 = 4222,
  CODE_FOR_ssse3_psignv4si3 = 4223,
  CODE_FOR_ssse3_psignv8qi3 = 4224,
  CODE_FOR_ssse3_psignv4hi3 = 4225,
  CODE_FOR_ssse3_psignv2si3 = 4226,
  CODE_FOR_avx512bw_palignrv64qi_mask = 4227,
  CODE_FOR_avx2_palignrv32qi_mask = 4228,
  CODE_FOR_ssse3_palignrv16qi_mask = 4229,
  CODE_FOR_avx512bw_palignrv4ti = 4230,
  CODE_FOR_avx2_palignrv2ti = 4231,
  CODE_FOR_ssse3_palignrti = 4232,
  CODE_FOR_ssse3_palignrdi = 4233,
  CODE_FOR_absv16si2_mask = 4246,
  CODE_FOR_absv8si2_mask = 4247,
  CODE_FOR_absv4si2_mask = 4248,
  CODE_FOR_absv8di2_mask = 4249,
  CODE_FOR_absv4di2_mask = 4250,
  CODE_FOR_absv2di2_mask = 4251,
  CODE_FOR_absv64qi2_mask = 4252,
  CODE_FOR_absv16qi2_mask = 4253,
  CODE_FOR_absv32qi2_mask = 4254,
  CODE_FOR_absv32hi2_mask = 4255,
  CODE_FOR_absv16hi2_mask = 4256,
  CODE_FOR_absv8hi2_mask = 4257,
  CODE_FOR_ssse3_absv8qi2 = 4258,
  CODE_FOR_ssse3_absv4hi2 = 4259,
  CODE_FOR_ssse3_absv2si2 = 4260,
  CODE_FOR_sse4a_movntsf = 4261,
  CODE_FOR_sse4a_movntdf = 4262,
  CODE_FOR_sse4a_vmmovntv4sf = 4263,
  CODE_FOR_sse4a_vmmovntv2df = 4264,
  CODE_FOR_sse4a_extrqi = 4265,
  CODE_FOR_sse4a_extrq = 4266,
  CODE_FOR_sse4a_insertqi = 4267,
  CODE_FOR_sse4a_insertq = 4268,
  CODE_FOR_avx_blendps256 = 4269,
  CODE_FOR_sse4_1_blendps = 4270,
  CODE_FOR_avx_blendpd256 = 4271,
  CODE_FOR_sse4_1_blendpd = 4272,
  CODE_FOR_avx_blendvps256 = 4273,
  CODE_FOR_sse4_1_blendvps = 4274,
  CODE_FOR_avx_blendvpd256 = 4275,
  CODE_FOR_sse4_1_blendvpd = 4276,
  CODE_FOR_sse4_1_blendvss = 4277,
  CODE_FOR_sse4_1_blendvsd = 4278,
  CODE_FOR_avx_dpps256 = 4287,
  CODE_FOR_sse4_1_dpps = 4288,
  CODE_FOR_avx_dppd256 = 4289,
  CODE_FOR_sse4_1_dppd = 4290,
  CODE_FOR_avx512f_movntdqa = 4291,
  CODE_FOR_avx2_movntdqa = 4292,
  CODE_FOR_sse4_1_movntdqa = 4293,
  CODE_FOR_avx2_mpsadbw = 4294,
  CODE_FOR_sse4_1_mpsadbw = 4295,
  CODE_FOR_avx512bw_packusdw = 4296,
  CODE_FOR_avx512bw_packusdw_mask = 4297,
  CODE_FOR_avx2_packusdw = 4298,
  CODE_FOR_avx2_packusdw_mask = 4299,
  CODE_FOR_sse4_1_packusdw = 4300,
  CODE_FOR_sse4_1_packusdw_mask = 4301,
  CODE_FOR_avx2_pblendvb = 4302,
  CODE_FOR_sse4_1_pblendvb = 4303,
  CODE_FOR_sse4_1_pblendw = 4306,
  CODE_FOR_avx2_pblenddv8si = 4308,
  CODE_FOR_avx2_pblenddv4si = 4309,
  CODE_FOR_sse4_1_phminposuw = 4310,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 4311,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 4312,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 4313,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 4314,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 4315,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 4316,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 4317,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 4318,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 4319,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 4320,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 4321,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 4322,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 4332,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 4334,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 4335,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 4336,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 4337,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 4338,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 4347,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 4348,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 4349,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 4350,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 4359,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 4360,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 4361,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 4362,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 4363,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 4364,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 4365,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 4366,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 4367,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 4368,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 4369,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 4370,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 4379,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 4380,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 4381,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 4382,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 4391,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 4392,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 4393,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 4394,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 4403,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 4404,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 4405,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 4406,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 4407,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 4408,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 4409,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 4410,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 4411,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 4412,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 4413,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 4414,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 4423,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 4424,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 4425,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 4426,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 4435,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 4436,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 4437,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 4438,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 4439,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 4440,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 4441,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 4442,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 4443,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 4444,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 4445,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 4446,
  CODE_FOR_avx_vtestps256 = 4455,
  CODE_FOR_avx_vtestps = 4456,
  CODE_FOR_avx_vtestpd256 = 4457,
  CODE_FOR_avx_vtestpd = 4458,
  CODE_FOR_sse4_1_ptestv16qi = 4459,
  CODE_FOR_sse4_1_ptestv8hi = 4460,
  CODE_FOR_sse4_1_ptestv4si = 4461,
  CODE_FOR_sse4_1_ptestv2di = 4462,
  CODE_FOR_sse4_1_ptestv4sf = 4463,
  CODE_FOR_sse4_1_ptestv2df = 4464,
  CODE_FOR_avx_ptestv32qi = 4465,
  CODE_FOR_avx_ptestv16hi = 4466,
  CODE_FOR_avx_ptestv8si = 4467,
  CODE_FOR_avx_ptestv4di = 4468,
  CODE_FOR_avx_ptestv8sf = 4469,
  CODE_FOR_avx_ptestv4df = 4470,
  CODE_FOR_ptesttf2 = 4471,
  CODE_FOR_avx_roundps256 = 4472,
  CODE_FOR_sse4_1_roundps = 4473,
  CODE_FOR_avx_roundpd256 = 4474,
  CODE_FOR_sse4_1_roundpd = 4475,
  CODE_FOR_sse4_1_roundss = 4476,
  CODE_FOR_sse4_1_roundsd = 4477,
  CODE_FOR_sse4_2_pcmpestr = 4478,
  CODE_FOR_sse4_2_pcmpestri = 4479,
  CODE_FOR_sse4_2_pcmpestrm = 4480,
  CODE_FOR_sse4_2_pcmpestr_cconly = 4481,
  CODE_FOR_sse4_2_pcmpistr = 4482,
  CODE_FOR_sse4_2_pcmpistri = 4483,
  CODE_FOR_sse4_2_pcmpistrm = 4484,
  CODE_FOR_sse4_2_pcmpistr_cconly = 4485,
  CODE_FOR_avx512er_exp2v16sf = 4502,
  CODE_FOR_avx512er_exp2v16sf_round = 4503,
  CODE_FOR_avx512er_exp2v16sf_mask = 4504,
  CODE_FOR_avx512er_exp2v16sf_mask_round = 4505,
  CODE_FOR_avx512er_exp2v8df = 4506,
  CODE_FOR_avx512er_exp2v8df_round = 4507,
  CODE_FOR_avx512er_exp2v8df_mask = 4508,
  CODE_FOR_avx512er_exp2v8df_mask_round = 4509,
  CODE_FOR_avx512er_rcp28v16sf_mask = 4512,
  CODE_FOR_avx512er_rcp28v16sf_mask_round = 4513,
  CODE_FOR_avx512er_rcp28v8df_mask = 4516,
  CODE_FOR_avx512er_rcp28v8df_mask_round = 4517,
  CODE_FOR_avx512er_vmrcp28v4sf = 4518,
  CODE_FOR_avx512er_vmrcp28v4sf_round = 4519,
  CODE_FOR_avx512er_vmrcp28v2df = 4520,
  CODE_FOR_avx512er_vmrcp28v2df_round = 4521,
  CODE_FOR_avx512er_rsqrt28v16sf_mask = 4524,
  CODE_FOR_avx512er_rsqrt28v16sf_mask_round = 4525,
  CODE_FOR_avx512er_rsqrt28v8df_mask = 4528,
  CODE_FOR_avx512er_rsqrt28v8df_mask_round = 4529,
  CODE_FOR_avx512er_vmrsqrt28v4sf = 4530,
  CODE_FOR_avx512er_vmrsqrt28v4sf_round = 4531,
  CODE_FOR_avx512er_vmrsqrt28v2df = 4532,
  CODE_FOR_avx512er_vmrsqrt28v2df_round = 4533,
  CODE_FOR_xop_pmacsww = 4534,
  CODE_FOR_xop_pmacssww = 4535,
  CODE_FOR_xop_pmacsdd = 4536,
  CODE_FOR_xop_pmacssdd = 4537,
  CODE_FOR_xop_pmacsdql = 4538,
  CODE_FOR_xop_pmacssdql = 4539,
  CODE_FOR_xop_pmacsdqh = 4540,
  CODE_FOR_xop_pmacssdqh = 4541,
  CODE_FOR_xop_pmacswd = 4542,
  CODE_FOR_xop_pmacsswd = 4543,
  CODE_FOR_xop_pmadcswd = 4544,
  CODE_FOR_xop_pmadcsswd = 4545,
  CODE_FOR_xop_pcmov_v32qi256 = 4546,
  CODE_FOR_xop_pcmov_v16qi = 4547,
  CODE_FOR_xop_pcmov_v16hi256 = 4548,
  CODE_FOR_xop_pcmov_v8hi = 4549,
  CODE_FOR_xop_pcmov_v8si256 = 4550,
  CODE_FOR_xop_pcmov_v4si = 4551,
  CODE_FOR_xop_pcmov_v4di256 = 4552,
  CODE_FOR_xop_pcmov_v2di = 4553,
  CODE_FOR_xop_pcmov_v8sf256 = 4554,
  CODE_FOR_xop_pcmov_v4sf = 4555,
  CODE_FOR_xop_pcmov_v4df256 = 4556,
  CODE_FOR_xop_pcmov_v2df = 4557,
  CODE_FOR_xop_phaddbw = 4558,
  CODE_FOR_xop_phaddubw = 4559,
  CODE_FOR_xop_phaddbd = 4560,
  CODE_FOR_xop_phaddubd = 4561,
  CODE_FOR_xop_phaddbq = 4562,
  CODE_FOR_xop_phaddubq = 4563,
  CODE_FOR_xop_phaddwd = 4564,
  CODE_FOR_xop_phadduwd = 4565,
  CODE_FOR_xop_phaddwq = 4566,
  CODE_FOR_xop_phadduwq = 4567,
  CODE_FOR_xop_phadddq = 4568,
  CODE_FOR_xop_phaddudq = 4569,
  CODE_FOR_xop_phsubbw = 4570,
  CODE_FOR_xop_phsubwd = 4571,
  CODE_FOR_xop_phsubdq = 4572,
  CODE_FOR_xop_pperm = 4573,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 4574,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 4575,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 4576,
  CODE_FOR_xop_rotlv16qi3 = 4577,
  CODE_FOR_xop_rotlv8hi3 = 4578,
  CODE_FOR_xop_rotlv4si3 = 4579,
  CODE_FOR_xop_rotlv2di3 = 4580,
  CODE_FOR_xop_rotrv16qi3 = 4581,
  CODE_FOR_xop_rotrv8hi3 = 4582,
  CODE_FOR_xop_rotrv4si3 = 4583,
  CODE_FOR_xop_rotrv2di3 = 4584,
  CODE_FOR_xop_vrotlv16qi3 = 4585,
  CODE_FOR_xop_vrotlv8hi3 = 4586,
  CODE_FOR_xop_vrotlv4si3 = 4587,
  CODE_FOR_xop_vrotlv2di3 = 4588,
  CODE_FOR_xop_shav16qi3 = 4589,
  CODE_FOR_xop_shav8hi3 = 4590,
  CODE_FOR_xop_shav4si3 = 4591,
  CODE_FOR_xop_shav2di3 = 4592,
  CODE_FOR_xop_shlv16qi3 = 4593,
  CODE_FOR_xop_shlv8hi3 = 4594,
  CODE_FOR_xop_shlv4si3 = 4595,
  CODE_FOR_xop_shlv2di3 = 4596,
  CODE_FOR_xop_frczsf2 = 4597,
  CODE_FOR_xop_frczdf2 = 4598,
  CODE_FOR_xop_frczv4sf2 = 4599,
  CODE_FOR_xop_frczv2df2 = 4600,
  CODE_FOR_xop_frczv8sf2 = 4601,
  CODE_FOR_xop_frczv4df2 = 4602,
  CODE_FOR_xop_maskcmpv16qi3 = 4605,
  CODE_FOR_xop_maskcmpv8hi3 = 4606,
  CODE_FOR_xop_maskcmpv4si3 = 4607,
  CODE_FOR_xop_maskcmpv2di3 = 4608,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 4609,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 4610,
  CODE_FOR_xop_maskcmp_unsv4si3 = 4611,
  CODE_FOR_xop_maskcmp_unsv2di3 = 4612,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 4613,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 4614,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 4615,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 4616,
  CODE_FOR_xop_pcom_tfv16qi3 = 4617,
  CODE_FOR_xop_pcom_tfv8hi3 = 4618,
  CODE_FOR_xop_pcom_tfv4si3 = 4619,
  CODE_FOR_xop_pcom_tfv2di3 = 4620,
  CODE_FOR_xop_vpermil2v8sf3 = 4621,
  CODE_FOR_xop_vpermil2v4sf3 = 4622,
  CODE_FOR_xop_vpermil2v4df3 = 4623,
  CODE_FOR_xop_vpermil2v2df3 = 4624,
  CODE_FOR_aesenc = 4625,
  CODE_FOR_aesenclast = 4626,
  CODE_FOR_aesdec = 4627,
  CODE_FOR_aesdeclast = 4628,
  CODE_FOR_aesimc = 4629,
  CODE_FOR_aeskeygenassist = 4630,
  CODE_FOR_pclmulqdq = 4631,
  CODE_FOR_avx_vzeroupper = 4633,
  CODE_FOR_avx2_pbroadcastv16si = 4634,
  CODE_FOR_avx2_pbroadcastv8di = 4635,
  CODE_FOR_avx2_pbroadcastv64qi = 4636,
  CODE_FOR_avx2_pbroadcastv32qi = 4637,
  CODE_FOR_avx2_pbroadcastv16qi = 4638,
  CODE_FOR_avx2_pbroadcastv32hi = 4639,
  CODE_FOR_avx2_pbroadcastv16hi = 4640,
  CODE_FOR_avx2_pbroadcastv8hi = 4641,
  CODE_FOR_avx2_pbroadcastv8si = 4642,
  CODE_FOR_avx2_pbroadcastv4si = 4643,
  CODE_FOR_avx2_pbroadcastv4di = 4644,
  CODE_FOR_avx2_pbroadcastv2di = 4645,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 4646,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 4647,
  CODE_FOR_avx2_pbroadcastv8si_1 = 4648,
  CODE_FOR_avx2_pbroadcastv4di_1 = 4649,
  CODE_FOR_avx2_permvarv8si = 4650,
  CODE_FOR_avx2_permvarv8si_mask = 4651,
  CODE_FOR_avx2_permvarv8sf = 4652,
  CODE_FOR_avx2_permvarv8sf_mask = 4653,
  CODE_FOR_avx512f_permvarv16si = 4654,
  CODE_FOR_avx512f_permvarv16si_mask = 4655,
  CODE_FOR_avx512f_permvarv16sf = 4656,
  CODE_FOR_avx512f_permvarv16sf_mask = 4657,
  CODE_FOR_avx512f_permvarv8di = 4658,
  CODE_FOR_avx512f_permvarv8di_mask = 4659,
  CODE_FOR_avx512f_permvarv8df = 4660,
  CODE_FOR_avx512f_permvarv8df_mask = 4661,
  CODE_FOR_avx2_permvarv4di = 4662,
  CODE_FOR_avx2_permvarv4di_mask = 4663,
  CODE_FOR_avx2_permvarv4df = 4664,
  CODE_FOR_avx2_permvarv4df_mask = 4665,
  CODE_FOR_avx512bw_permvarv64qi = 4666,
  CODE_FOR_avx512bw_permvarv64qi_mask = 4667,
  CODE_FOR_avx512vl_permvarv16qi = 4668,
  CODE_FOR_avx512vl_permvarv16qi_mask = 4669,
  CODE_FOR_avx512vl_permvarv32qi = 4670,
  CODE_FOR_avx512vl_permvarv32qi_mask = 4671,
  CODE_FOR_avx512vl_permvarv8hi = 4672,
  CODE_FOR_avx512vl_permvarv8hi_mask = 4673,
  CODE_FOR_avx512vl_permvarv16hi = 4674,
  CODE_FOR_avx512vl_permvarv16hi_mask = 4675,
  CODE_FOR_avx512bw_permvarv32hi = 4676,
  CODE_FOR_avx512bw_permvarv32hi_mask = 4677,
  CODE_FOR_avx2_permv4di_1 = 4678,
  CODE_FOR_avx2_permv4di_1_mask = 4679,
  CODE_FOR_avx2_permv4df_1 = 4680,
  CODE_FOR_avx2_permv4df_1_mask = 4681,
  CODE_FOR_avx512f_permv8df_1 = 4682,
  CODE_FOR_avx512f_permv8df_1_mask = 4683,
  CODE_FOR_avx512f_permv8di_1 = 4684,
  CODE_FOR_avx512f_permv8di_1_mask = 4685,
  CODE_FOR_avx2_permv2ti = 4686,
  CODE_FOR_avx2_vec_dupv4df = 4687,
  CODE_FOR_avx512f_vec_dupv16si_1 = 4688,
  CODE_FOR_avx512f_vec_dupv8di_1 = 4689,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 4690,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 4691,
  CODE_FOR_avx512f_vec_dupv16si = 4692,
  CODE_FOR_avx512f_vec_dupv16si_mask = 4693,
  CODE_FOR_avx512vl_vec_dupv8si = 4694,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 4695,
  CODE_FOR_avx512vl_vec_dupv4si = 4696,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 4697,
  CODE_FOR_avx512f_vec_dupv8di = 4698,
  CODE_FOR_avx512f_vec_dupv8di_mask = 4699,
  CODE_FOR_avx512vl_vec_dupv4di = 4700,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 4701,
  CODE_FOR_avx512vl_vec_dupv2di = 4702,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 4703,
  CODE_FOR_avx512f_vec_dupv16sf = 4704,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 4705,
  CODE_FOR_avx512vl_vec_dupv8sf = 4706,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 4707,
  CODE_FOR_avx512vl_vec_dupv4sf = 4708,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 4709,
  CODE_FOR_avx512f_vec_dupv8df = 4710,
  CODE_FOR_avx512f_vec_dupv8df_mask = 4711,
  CODE_FOR_avx512vl_vec_dupv4df = 4712,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 4713,
  CODE_FOR_avx512vl_vec_dupv2df = 4714,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 4715,
  CODE_FOR_avx512bw_vec_dupv64qi = 4716,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 4717,
  CODE_FOR_avx512vl_vec_dupv16qi = 4718,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 4719,
  CODE_FOR_avx512vl_vec_dupv32qi = 4720,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 4721,
  CODE_FOR_avx512bw_vec_dupv32hi = 4722,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 4723,
  CODE_FOR_avx512vl_vec_dupv16hi = 4724,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 4725,
  CODE_FOR_avx512vl_vec_dupv8hi = 4726,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 4727,
  CODE_FOR_avx512f_broadcastv16sf_mask = 4729,
  CODE_FOR_avx512f_broadcastv16si_mask = 4731,
  CODE_FOR_avx512f_broadcastv8df_mask = 4733,
  CODE_FOR_avx512f_broadcastv8di_mask = 4735,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 4737,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 4739,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 4741,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 4743,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 4745,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 4747,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 4749,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 4751,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 4753,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 4755,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 4757,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 4759,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 4761,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 4763,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 4765,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 4767,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 4769,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 4771,
  CODE_FOR_vec_dupv4sf = 4772,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 4775,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 4776,
  CODE_FOR_avx2_vbroadcasti128_v8si = 4777,
  CODE_FOR_avx2_vbroadcasti128_v4di = 4778,
  CODE_FOR_vec_dupv8si = 4785,
  CODE_FOR_vec_dupv8sf = 4786,
  CODE_FOR_vec_dupv4di = 4787,
  CODE_FOR_vec_dupv4df = 4788,
  CODE_FOR_avx_vbroadcastf128_v32qi = 4789,
  CODE_FOR_avx_vbroadcastf128_v16hi = 4790,
  CODE_FOR_avx_vbroadcastf128_v8si = 4791,
  CODE_FOR_avx_vbroadcastf128_v4di = 4792,
  CODE_FOR_avx_vbroadcastf128_v8sf = 4793,
  CODE_FOR_avx_vbroadcastf128_v4df = 4794,
  CODE_FOR_avx512dq_broadcastv16si_mask = 4796,
  CODE_FOR_avx512dq_broadcastv8si_mask = 4798,
  CODE_FOR_avx512dq_broadcastv4si_mask = 4800,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 4802,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 4804,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 4806,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 4808,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 4810,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 4812,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 4814,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 4816,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 4818,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 4820,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 4821,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 4822,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 4823,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 4824,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 4825,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 4826,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 4842,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 4843,
  CODE_FOR_avx_vpermilvarv8sf3 = 4844,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 4845,
  CODE_FOR_avx_vpermilvarv4sf3 = 4846,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 4847,
  CODE_FOR_avx512f_vpermilvarv8df3 = 4848,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 4849,
  CODE_FOR_avx_vpermilvarv4df3 = 4850,
  CODE_FOR_avx_vpermilvarv4df3_mask = 4851,
  CODE_FOR_avx_vpermilvarv2df3 = 4852,
  CODE_FOR_avx_vpermilvarv2df3_mask = 4853,
  CODE_FOR_avx512f_vpermt2varv16si3 = 4872,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 4873,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 4874,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 4875,
  CODE_FOR_avx512f_vpermt2varv8di3 = 4876,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 4877,
  CODE_FOR_avx512f_vpermt2varv8df3 = 4878,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 4879,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 4880,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 4881,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 4882,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 4883,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 4884,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 4885,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 4886,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 4887,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 4888,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 4889,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 4890,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 4891,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 4892,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 4893,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 4894,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 4895,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 4896,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 4897,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 4898,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 4899,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 4900,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 4901,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 4902,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 4903,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 4904,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 4905,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 4906,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 4907,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 4908,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 4909,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 4910,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 4911,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 4912,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 4913,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 4914,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 4915,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 4916,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 4917,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 4918,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 4919,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 4920,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 4921,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 4922,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 4923,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 4924,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 4925,
  CODE_FOR_vec_set_lo_v4di = 4938,
  CODE_FOR_vec_set_lo_v4di_mask = 4939,
  CODE_FOR_vec_set_lo_v4df = 4940,
  CODE_FOR_vec_set_lo_v4df_mask = 4941,
  CODE_FOR_vec_set_hi_v4di = 4942,
  CODE_FOR_vec_set_hi_v4di_mask = 4943,
  CODE_FOR_vec_set_hi_v4df = 4944,
  CODE_FOR_vec_set_hi_v4df_mask = 4945,
  CODE_FOR_vec_set_lo_v8si = 4946,
  CODE_FOR_vec_set_lo_v8si_mask = 4947,
  CODE_FOR_vec_set_lo_v8sf = 4948,
  CODE_FOR_vec_set_lo_v8sf_mask = 4949,
  CODE_FOR_vec_set_hi_v8si = 4950,
  CODE_FOR_vec_set_hi_v8si_mask = 4951,
  CODE_FOR_vec_set_hi_v8sf = 4952,
  CODE_FOR_vec_set_hi_v8sf_mask = 4953,
  CODE_FOR_vec_set_lo_v16hi = 4954,
  CODE_FOR_vec_set_hi_v16hi = 4955,
  CODE_FOR_vec_set_lo_v32qi = 4956,
  CODE_FOR_vec_set_hi_v32qi = 4957,
  CODE_FOR_avx_maskloadps = 4958,
  CODE_FOR_avx_maskloadpd = 4959,
  CODE_FOR_avx_maskloadps256 = 4960,
  CODE_FOR_avx_maskloadpd256 = 4961,
  CODE_FOR_avx2_maskloadd = 4962,
  CODE_FOR_avx2_maskloadq = 4963,
  CODE_FOR_avx2_maskloadd256 = 4964,
  CODE_FOR_avx2_maskloadq256 = 4965,
  CODE_FOR_avx_maskstoreps = 4966,
  CODE_FOR_avx_maskstorepd = 4967,
  CODE_FOR_avx_maskstoreps256 = 4968,
  CODE_FOR_avx_maskstorepd256 = 4969,
  CODE_FOR_avx2_maskstored = 4970,
  CODE_FOR_avx2_maskstoreq = 4971,
  CODE_FOR_avx2_maskstored256 = 4972,
  CODE_FOR_avx2_maskstoreq256 = 4973,
  CODE_FOR_avx_si256_si = 4974,
  CODE_FOR_avx_ps256_ps = 4975,
  CODE_FOR_avx_pd256_pd = 4976,
  CODE_FOR_avx2_ashrvv4si = 4977,
  CODE_FOR_avx2_ashrvv4si_mask = 4978,
  CODE_FOR_avx2_ashrvv8si = 4979,
  CODE_FOR_avx2_ashrvv8si_mask = 4980,
  CODE_FOR_avx512f_ashrvv16si = 4981,
  CODE_FOR_avx512f_ashrvv16si_mask = 4982,
  CODE_FOR_avx2_ashrvv2di = 4983,
  CODE_FOR_avx2_ashrvv2di_mask = 4984,
  CODE_FOR_avx2_ashrvv4di = 4985,
  CODE_FOR_avx2_ashrvv4di_mask = 4986,
  CODE_FOR_avx512f_ashrvv8di = 4987,
  CODE_FOR_avx512f_ashrvv8di_mask = 4988,
  CODE_FOR_avx512vl_ashrvv8hi = 4989,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 4990,
  CODE_FOR_avx512vl_ashrvv16hi = 4991,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 4992,
  CODE_FOR_avx512bw_ashrvv32hi = 4993,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 4994,
  CODE_FOR_avx512f_ashlvv16si = 4995,
  CODE_FOR_avx512f_ashlvv16si_mask = 4996,
  CODE_FOR_avx512f_lshrvv16si = 4997,
  CODE_FOR_avx512f_lshrvv16si_mask = 4998,
  CODE_FOR_avx2_ashlvv8si = 4999,
  CODE_FOR_avx2_ashlvv8si_mask = 5000,
  CODE_FOR_avx2_lshrvv8si = 5001,
  CODE_FOR_avx2_lshrvv8si_mask = 5002,
  CODE_FOR_avx2_ashlvv4si = 5003,
  CODE_FOR_avx2_ashlvv4si_mask = 5004,
  CODE_FOR_avx2_lshrvv4si = 5005,
  CODE_FOR_avx2_lshrvv4si_mask = 5006,
  CODE_FOR_avx512f_ashlvv8di = 5007,
  CODE_FOR_avx512f_ashlvv8di_mask = 5008,
  CODE_FOR_avx512f_lshrvv8di = 5009,
  CODE_FOR_avx512f_lshrvv8di_mask = 5010,
  CODE_FOR_avx2_ashlvv4di = 5011,
  CODE_FOR_avx2_ashlvv4di_mask = 5012,
  CODE_FOR_avx2_lshrvv4di = 5013,
  CODE_FOR_avx2_lshrvv4di_mask = 5014,
  CODE_FOR_avx2_ashlvv2di = 5015,
  CODE_FOR_avx2_ashlvv2di_mask = 5016,
  CODE_FOR_avx2_lshrvv2di = 5017,
  CODE_FOR_avx2_lshrvv2di_mask = 5018,
  CODE_FOR_avx512vl_ashlvv8hi = 5019,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 5020,
  CODE_FOR_avx512vl_lshrvv8hi = 5021,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 5022,
  CODE_FOR_avx512vl_ashlvv16hi = 5023,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 5024,
  CODE_FOR_avx512vl_lshrvv16hi = 5025,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 5026,
  CODE_FOR_avx512bw_ashlvv32hi = 5027,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 5028,
  CODE_FOR_avx512bw_lshrvv32hi = 5029,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 5030,
  CODE_FOR_avx_vec_concatv32qi = 5031,
  CODE_FOR_avx_vec_concatv16hi = 5032,
  CODE_FOR_avx_vec_concatv8si = 5033,
  CODE_FOR_avx_vec_concatv4di = 5034,
  CODE_FOR_avx_vec_concatv8sf = 5035,
  CODE_FOR_avx_vec_concatv4df = 5036,
  CODE_FOR_avx_vec_concatv64qi = 5037,
  CODE_FOR_avx_vec_concatv32hi = 5038,
  CODE_FOR_avx_vec_concatv16si = 5039,
  CODE_FOR_avx_vec_concatv8di = 5040,
  CODE_FOR_avx_vec_concatv16sf = 5041,
  CODE_FOR_avx_vec_concatv8df = 5042,
  CODE_FOR_vcvtph2ps = 5043,
  CODE_FOR_vcvtph2ps_mask = 5044,
  CODE_FOR_vcvtph2ps256 = 5047,
  CODE_FOR_vcvtph2ps256_mask = 5048,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 5051,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 5052,
  CODE_FOR_vcvtps2ph256 = 5056,
  CODE_FOR_vcvtps2ph256_mask = 5057,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 5059,
  CODE_FOR_avx512f_compressv16si_mask = 5276,
  CODE_FOR_avx512f_compressv16sf_mask = 5277,
  CODE_FOR_avx512f_compressv8di_mask = 5278,
  CODE_FOR_avx512f_compressv8df_mask = 5279,
  CODE_FOR_avx512vl_compressv8si_mask = 5280,
  CODE_FOR_avx512vl_compressv8sf_mask = 5281,
  CODE_FOR_avx512vl_compressv4di_mask = 5282,
  CODE_FOR_avx512vl_compressv4df_mask = 5283,
  CODE_FOR_avx512vl_compressv4si_mask = 5284,
  CODE_FOR_avx512vl_compressv4sf_mask = 5285,
  CODE_FOR_avx512vl_compressv2di_mask = 5286,
  CODE_FOR_avx512vl_compressv2df_mask = 5287,
  CODE_FOR_compressv64qi_mask = 5288,
  CODE_FOR_compressv16qi_mask = 5289,
  CODE_FOR_compressv32qi_mask = 5290,
  CODE_FOR_compressv32hi_mask = 5291,
  CODE_FOR_compressv16hi_mask = 5292,
  CODE_FOR_compressv8hi_mask = 5293,
  CODE_FOR_avx512f_compressstorev16si_mask = 5294,
  CODE_FOR_avx512f_compressstorev16sf_mask = 5295,
  CODE_FOR_avx512f_compressstorev8di_mask = 5296,
  CODE_FOR_avx512f_compressstorev8df_mask = 5297,
  CODE_FOR_avx512vl_compressstorev8si_mask = 5298,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 5299,
  CODE_FOR_avx512vl_compressstorev4di_mask = 5300,
  CODE_FOR_avx512vl_compressstorev4df_mask = 5301,
  CODE_FOR_avx512vl_compressstorev4si_mask = 5302,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 5303,
  CODE_FOR_avx512vl_compressstorev2di_mask = 5304,
  CODE_FOR_avx512vl_compressstorev2df_mask = 5305,
  CODE_FOR_compressstorev64qi_mask = 5306,
  CODE_FOR_compressstorev16qi_mask = 5307,
  CODE_FOR_compressstorev32qi_mask = 5308,
  CODE_FOR_compressstorev32hi_mask = 5309,
  CODE_FOR_compressstorev16hi_mask = 5310,
  CODE_FOR_compressstorev8hi_mask = 5311,
  CODE_FOR_avx512f_expandv16si_mask = 5312,
  CODE_FOR_avx512f_expandv16sf_mask = 5313,
  CODE_FOR_avx512f_expandv8di_mask = 5314,
  CODE_FOR_avx512f_expandv8df_mask = 5315,
  CODE_FOR_avx512vl_expandv8si_mask = 5316,
  CODE_FOR_avx512vl_expandv8sf_mask = 5317,
  CODE_FOR_avx512vl_expandv4di_mask = 5318,
  CODE_FOR_avx512vl_expandv4df_mask = 5319,
  CODE_FOR_avx512vl_expandv4si_mask = 5320,
  CODE_FOR_avx512vl_expandv4sf_mask = 5321,
  CODE_FOR_avx512vl_expandv2di_mask = 5322,
  CODE_FOR_avx512vl_expandv2df_mask = 5323,
  CODE_FOR_expandv64qi_mask = 5324,
  CODE_FOR_expandv16qi_mask = 5325,
  CODE_FOR_expandv32qi_mask = 5326,
  CODE_FOR_expandv32hi_mask = 5327,
  CODE_FOR_expandv16hi_mask = 5328,
  CODE_FOR_expandv8hi_mask = 5329,
  CODE_FOR_avx512dq_rangepv16sf = 5330,
  CODE_FOR_avx512dq_rangepv16sf_round = 5331,
  CODE_FOR_avx512dq_rangepv16sf_mask = 5332,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 5333,
  CODE_FOR_avx512dq_rangepv8sf = 5334,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 5335,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 5336,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 5337,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 5338,
  CODE_FOR_avx512dq_rangepv8df_round = 5339,
  CODE_FOR_avx512dq_rangepv8df_mask = 5340,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 5341,
  CODE_FOR_avx512dq_rangepv4df = 5342,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 5343,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 5344,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 5345,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 5346,
  CODE_FOR_avx512dq_rangesv4sf_mask = 5347,
  CODE_FOR_avx512dq_rangesv4sf_round = 5348,
  CODE_FOR_avx512dq_rangesv4sf_mask_round = 5349,
  CODE_FOR_avx512dq_rangesv2df = 5350,
  CODE_FOR_avx512dq_rangesv2df_mask = 5351,
  CODE_FOR_avx512dq_rangesv2df_round = 5352,
  CODE_FOR_avx512dq_rangesv2df_mask_round = 5353,
  CODE_FOR_avx512dq_fpclassv16sf = 5354,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 5355,
  CODE_FOR_avx512dq_fpclassv8sf = 5356,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 5357,
  CODE_FOR_avx512dq_fpclassv4sf = 5358,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 5359,
  CODE_FOR_avx512dq_fpclassv8df = 5360,
  CODE_FOR_avx512dq_fpclassv8df_mask = 5361,
  CODE_FOR_avx512dq_fpclassv4df = 5362,
  CODE_FOR_avx512dq_fpclassv4df_mask = 5363,
  CODE_FOR_avx512dq_fpclassv2df = 5364,
  CODE_FOR_avx512dq_fpclassv2df_mask = 5365,
  CODE_FOR_avx512dq_vmfpclassv4sf = 5366,
  CODE_FOR_avx512dq_vmfpclassv2df = 5367,
  CODE_FOR_avx512f_getmantv16sf = 5368,
  CODE_FOR_avx512f_getmantv16sf_round = 5369,
  CODE_FOR_avx512f_getmantv16sf_mask = 5370,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 5371,
  CODE_FOR_avx512vl_getmantv8sf = 5372,
  CODE_FOR_avx512vl_getmantv8sf_round = 5373,
  CODE_FOR_avx512vl_getmantv8sf_mask = 5374,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 5375,
  CODE_FOR_avx512vl_getmantv4sf = 5376,
  CODE_FOR_avx512vl_getmantv4sf_round = 5377,
  CODE_FOR_avx512vl_getmantv4sf_mask = 5378,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 5379,
  CODE_FOR_avx512f_getmantv8df = 5380,
  CODE_FOR_avx512f_getmantv8df_round = 5381,
  CODE_FOR_avx512f_getmantv8df_mask = 5382,
  CODE_FOR_avx512f_getmantv8df_mask_round = 5383,
  CODE_FOR_avx512vl_getmantv4df = 5384,
  CODE_FOR_avx512vl_getmantv4df_round = 5385,
  CODE_FOR_avx512vl_getmantv4df_mask = 5386,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 5387,
  CODE_FOR_avx512vl_getmantv2df = 5388,
  CODE_FOR_avx512vl_getmantv2df_round = 5389,
  CODE_FOR_avx512vl_getmantv2df_mask = 5390,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 5391,
  CODE_FOR_avx512f_vgetmantv4sf = 5392,
  CODE_FOR_avx512f_vgetmantv4sf_mask = 5393,
  CODE_FOR_avx512f_vgetmantv4sf_round = 5394,
  CODE_FOR_avx512f_vgetmantv4sf_mask_round = 5395,
  CODE_FOR_avx512f_vgetmantv2df = 5396,
  CODE_FOR_avx512f_vgetmantv2df_mask = 5397,
  CODE_FOR_avx512f_vgetmantv2df_round = 5398,
  CODE_FOR_avx512f_vgetmantv2df_mask_round = 5399,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 5401,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 5403,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 5405,
  CODE_FOR_clzv16si2 = 5406,
  CODE_FOR_clzv16si2_mask = 5407,
  CODE_FOR_clzv8si2 = 5408,
  CODE_FOR_clzv8si2_mask = 5409,
  CODE_FOR_clzv4si2 = 5410,
  CODE_FOR_clzv4si2_mask = 5411,
  CODE_FOR_clzv8di2 = 5412,
  CODE_FOR_clzv8di2_mask = 5413,
  CODE_FOR_clzv4di2 = 5414,
  CODE_FOR_clzv4di2_mask = 5415,
  CODE_FOR_clzv2di2 = 5416,
  CODE_FOR_clzv2di2_mask = 5417,
  CODE_FOR_conflictv16si_mask = 5419,
  CODE_FOR_conflictv8si_mask = 5421,
  CODE_FOR_conflictv4si_mask = 5423,
  CODE_FOR_conflictv8di_mask = 5425,
  CODE_FOR_conflictv4di_mask = 5427,
  CODE_FOR_conflictv2di_mask = 5429,
  CODE_FOR_sha1msg1 = 5430,
  CODE_FOR_sha1msg2 = 5431,
  CODE_FOR_sha1nexte = 5432,
  CODE_FOR_sha1rnds4 = 5433,
  CODE_FOR_sha256msg1 = 5434,
  CODE_FOR_sha256msg2 = 5435,
  CODE_FOR_sha256rnds2 = 5436,
  CODE_FOR_avx512f_si512_si = 5437,
  CODE_FOR_avx512f_ps512_ps = 5438,
  CODE_FOR_avx512f_pd512_pd = 5439,
  CODE_FOR_avx512f_si512_256si = 5440,
  CODE_FOR_avx512f_ps512_256ps = 5441,
  CODE_FOR_avx512f_pd512_256pd = 5442,
  CODE_FOR_vpamdd52luqv8di = 5443,
  CODE_FOR_vpamdd52luqv8di_maskz_1 = 5444,
  CODE_FOR_vpamdd52huqv8di = 5445,
  CODE_FOR_vpamdd52huqv8di_maskz_1 = 5446,
  CODE_FOR_vpamdd52luqv4di = 5447,
  CODE_FOR_vpamdd52luqv4di_maskz_1 = 5448,
  CODE_FOR_vpamdd52huqv4di = 5449,
  CODE_FOR_vpamdd52huqv4di_maskz_1 = 5450,
  CODE_FOR_vpamdd52luqv2di = 5451,
  CODE_FOR_vpamdd52luqv2di_maskz_1 = 5452,
  CODE_FOR_vpamdd52huqv2di = 5453,
  CODE_FOR_vpamdd52huqv2di_maskz_1 = 5454,
  CODE_FOR_vpamdd52luqv8di_mask = 5455,
  CODE_FOR_vpamdd52huqv8di_mask = 5456,
  CODE_FOR_vpamdd52luqv4di_mask = 5457,
  CODE_FOR_vpamdd52huqv4di_mask = 5458,
  CODE_FOR_vpamdd52luqv2di_mask = 5459,
  CODE_FOR_vpamdd52huqv2di_mask = 5460,
  CODE_FOR_vpmultishiftqbv64qi = 5461,
  CODE_FOR_vpmultishiftqbv64qi_mask = 5462,
  CODE_FOR_vpmultishiftqbv16qi = 5463,
  CODE_FOR_vpmultishiftqbv16qi_mask = 5464,
  CODE_FOR_vpmultishiftqbv32qi = 5465,
  CODE_FOR_vpmultishiftqbv32qi_mask = 5466,
  CODE_FOR_avx5124fmaddps_4fmaddps = 5469,
  CODE_FOR_avx5124fmaddps_4fmaddps_mask = 5470,
  CODE_FOR_avx5124fmaddps_4fmaddps_maskz = 5471,
  CODE_FOR_avx5124fmaddps_4fmaddss = 5472,
  CODE_FOR_avx5124fmaddps_4fmaddss_mask = 5473,
  CODE_FOR_avx5124fmaddps_4fmaddss_maskz = 5474,
  CODE_FOR_avx5124fmaddps_4fnmaddps = 5475,
  CODE_FOR_avx5124fmaddps_4fnmaddps_mask = 5476,
  CODE_FOR_avx5124fmaddps_4fnmaddps_maskz = 5477,
  CODE_FOR_avx5124fmaddps_4fnmaddss = 5478,
  CODE_FOR_avx5124fmaddps_4fnmaddss_mask = 5479,
  CODE_FOR_avx5124fmaddps_4fnmaddss_maskz = 5480,
  CODE_FOR_avx5124vnniw_vp4dpwssd = 5481,
  CODE_FOR_avx5124vnniw_vp4dpwssd_mask = 5482,
  CODE_FOR_avx5124vnniw_vp4dpwssd_maskz = 5483,
  CODE_FOR_avx5124vnniw_vp4dpwssds = 5484,
  CODE_FOR_avx5124vnniw_vp4dpwssds_mask = 5485,
  CODE_FOR_avx5124vnniw_vp4dpwssds_maskz = 5486,
  CODE_FOR_vpopcountv16si = 5487,
  CODE_FOR_vpopcountv16si_mask = 5488,
  CODE_FOR_vpopcountv8si = 5489,
  CODE_FOR_vpopcountv8si_mask = 5490,
  CODE_FOR_vpopcountv4si = 5491,
  CODE_FOR_vpopcountv4si_mask = 5492,
  CODE_FOR_vpopcountv8di = 5493,
  CODE_FOR_vpopcountv8di_mask = 5494,
  CODE_FOR_vpopcountv4di = 5495,
  CODE_FOR_vpopcountv4di_mask = 5496,
  CODE_FOR_vpopcountv2di = 5497,
  CODE_FOR_vpopcountv2di_mask = 5498,
  CODE_FOR_save_multiplesi = 5499,
  CODE_FOR_save_multipledi = 5500,
  CODE_FOR_restore_multiplesi = 5501,
  CODE_FOR_restore_multipledi = 5502,
  CODE_FOR_restore_multiple_and_returnsi = 5503,
  CODE_FOR_restore_multiple_and_returndi = 5504,
  CODE_FOR_restore_multiple_leave_returnsi = 5505,
  CODE_FOR_restore_multiple_leave_returndi = 5506,
  CODE_FOR_vpopcountv64qi = 5507,
  CODE_FOR_vpopcountv64qi_mask = 5508,
  CODE_FOR_vpopcountv16qi = 5509,
  CODE_FOR_vpopcountv16qi_mask = 5510,
  CODE_FOR_vpopcountv32qi = 5511,
  CODE_FOR_vpopcountv32qi_mask = 5512,
  CODE_FOR_vpopcountv32hi = 5513,
  CODE_FOR_vpopcountv32hi_mask = 5514,
  CODE_FOR_vpopcountv16hi = 5515,
  CODE_FOR_vpopcountv16hi_mask = 5516,
  CODE_FOR_vpopcountv8hi = 5517,
  CODE_FOR_vpopcountv8hi_mask = 5518,
  CODE_FOR_vgf2p8affineinvqb_v64qi = 5519,
  CODE_FOR_vgf2p8affineinvqb_v64qi_mask = 5520,
  CODE_FOR_vgf2p8affineinvqb_v32qi = 5521,
  CODE_FOR_vgf2p8affineinvqb_v32qi_mask = 5522,
  CODE_FOR_vgf2p8affineinvqb_v16qi = 5523,
  CODE_FOR_vgf2p8affineinvqb_v16qi_mask = 5524,
  CODE_FOR_vgf2p8affineqb_v64qi = 5525,
  CODE_FOR_vgf2p8affineqb_v64qi_mask = 5526,
  CODE_FOR_vgf2p8affineqb_v32qi = 5527,
  CODE_FOR_vgf2p8affineqb_v32qi_mask = 5528,
  CODE_FOR_vgf2p8affineqb_v16qi = 5529,
  CODE_FOR_vgf2p8affineqb_v16qi_mask = 5530,
  CODE_FOR_vgf2p8mulb_v64qi = 5531,
  CODE_FOR_vgf2p8mulb_v64qi_mask = 5532,
  CODE_FOR_vgf2p8mulb_v32qi = 5533,
  CODE_FOR_vgf2p8mulb_v32qi_mask = 5534,
  CODE_FOR_vgf2p8mulb_v16qi = 5535,
  CODE_FOR_vgf2p8mulb_v16qi_mask = 5536,
  CODE_FOR_vpshrd_v32hi = 5537,
  CODE_FOR_vpshrd_v32hi_mask = 5538,
  CODE_FOR_vpshrd_v16si = 5539,
  CODE_FOR_vpshrd_v16si_mask = 5540,
  CODE_FOR_vpshrd_v8di = 5541,
  CODE_FOR_vpshrd_v8di_mask = 5542,
  CODE_FOR_vpshrd_v16hi = 5543,
  CODE_FOR_vpshrd_v16hi_mask = 5544,
  CODE_FOR_vpshrd_v8si = 5545,
  CODE_FOR_vpshrd_v8si_mask = 5546,
  CODE_FOR_vpshrd_v4di = 5547,
  CODE_FOR_vpshrd_v4di_mask = 5548,
  CODE_FOR_vpshrd_v8hi = 5549,
  CODE_FOR_vpshrd_v8hi_mask = 5550,
  CODE_FOR_vpshrd_v4si = 5551,
  CODE_FOR_vpshrd_v4si_mask = 5552,
  CODE_FOR_vpshrd_v2di = 5553,
  CODE_FOR_vpshrd_v2di_mask = 5554,
  CODE_FOR_vpshld_v32hi = 5555,
  CODE_FOR_vpshld_v32hi_mask = 5556,
  CODE_FOR_vpshld_v16si = 5557,
  CODE_FOR_vpshld_v16si_mask = 5558,
  CODE_FOR_vpshld_v8di = 5559,
  CODE_FOR_vpshld_v8di_mask = 5560,
  CODE_FOR_vpshld_v16hi = 5561,
  CODE_FOR_vpshld_v16hi_mask = 5562,
  CODE_FOR_vpshld_v8si = 5563,
  CODE_FOR_vpshld_v8si_mask = 5564,
  CODE_FOR_vpshld_v4di = 5565,
  CODE_FOR_vpshld_v4di_mask = 5566,
  CODE_FOR_vpshld_v8hi = 5567,
  CODE_FOR_vpshld_v8hi_mask = 5568,
  CODE_FOR_vpshld_v4si = 5569,
  CODE_FOR_vpshld_v4si_mask = 5570,
  CODE_FOR_vpshld_v2di = 5571,
  CODE_FOR_vpshld_v2di_mask = 5572,
  CODE_FOR_vpshrdv_v32hi = 5573,
  CODE_FOR_vpshrdv_v16si = 5574,
  CODE_FOR_vpshrdv_v8di = 5575,
  CODE_FOR_vpshrdv_v16hi = 5576,
  CODE_FOR_vpshrdv_v8si = 5577,
  CODE_FOR_vpshrdv_v4di = 5578,
  CODE_FOR_vpshrdv_v8hi = 5579,
  CODE_FOR_vpshrdv_v4si = 5580,
  CODE_FOR_vpshrdv_v2di = 5581,
  CODE_FOR_vpshrdv_v32hi_mask = 5582,
  CODE_FOR_vpshrdv_v16si_mask = 5583,
  CODE_FOR_vpshrdv_v8di_mask = 5584,
  CODE_FOR_vpshrdv_v16hi_mask = 5585,
  CODE_FOR_vpshrdv_v8si_mask = 5586,
  CODE_FOR_vpshrdv_v4di_mask = 5587,
  CODE_FOR_vpshrdv_v8hi_mask = 5588,
  CODE_FOR_vpshrdv_v4si_mask = 5589,
  CODE_FOR_vpshrdv_v2di_mask = 5590,
  CODE_FOR_vpshrdv_v32hi_maskz_1 = 5591,
  CODE_FOR_vpshrdv_v16si_maskz_1 = 5592,
  CODE_FOR_vpshrdv_v8di_maskz_1 = 5593,
  CODE_FOR_vpshrdv_v16hi_maskz_1 = 5594,
  CODE_FOR_vpshrdv_v8si_maskz_1 = 5595,
  CODE_FOR_vpshrdv_v4di_maskz_1 = 5596,
  CODE_FOR_vpshrdv_v8hi_maskz_1 = 5597,
  CODE_FOR_vpshrdv_v4si_maskz_1 = 5598,
  CODE_FOR_vpshrdv_v2di_maskz_1 = 5599,
  CODE_FOR_vpshldv_v32hi = 5600,
  CODE_FOR_vpshldv_v16si = 5601,
  CODE_FOR_vpshldv_v8di = 5602,
  CODE_FOR_vpshldv_v16hi = 5603,
  CODE_FOR_vpshldv_v8si = 5604,
  CODE_FOR_vpshldv_v4di = 5605,
  CODE_FOR_vpshldv_v8hi = 5606,
  CODE_FOR_vpshldv_v4si = 5607,
  CODE_FOR_vpshldv_v2di = 5608,
  CODE_FOR_vpshldv_v32hi_mask = 5609,
  CODE_FOR_vpshldv_v16si_mask = 5610,
  CODE_FOR_vpshldv_v8di_mask = 5611,
  CODE_FOR_vpshldv_v16hi_mask = 5612,
  CODE_FOR_vpshldv_v8si_mask = 5613,
  CODE_FOR_vpshldv_v4di_mask = 5614,
  CODE_FOR_vpshldv_v8hi_mask = 5615,
  CODE_FOR_vpshldv_v4si_mask = 5616,
  CODE_FOR_vpshldv_v2di_mask = 5617,
  CODE_FOR_vpshldv_v32hi_maskz_1 = 5618,
  CODE_FOR_vpshldv_v16si_maskz_1 = 5619,
  CODE_FOR_vpshldv_v8di_maskz_1 = 5620,
  CODE_FOR_vpshldv_v16hi_maskz_1 = 5621,
  CODE_FOR_vpshldv_v8si_maskz_1 = 5622,
  CODE_FOR_vpshldv_v4di_maskz_1 = 5623,
  CODE_FOR_vpshldv_v8hi_maskz_1 = 5624,
  CODE_FOR_vpshldv_v4si_maskz_1 = 5625,
  CODE_FOR_vpshldv_v2di_maskz_1 = 5626,
  CODE_FOR_vpdpbusd_v16si = 5627,
  CODE_FOR_vpdpbusd_v8si = 5628,
  CODE_FOR_vpdpbusd_v4si = 5629,
  CODE_FOR_vpdpbusd_v16si_mask = 5630,
  CODE_FOR_vpdpbusd_v8si_mask = 5631,
  CODE_FOR_vpdpbusd_v4si_mask = 5632,
  CODE_FOR_vpdpbusd_v16si_maskz_1 = 5633,
  CODE_FOR_vpdpbusd_v8si_maskz_1 = 5634,
  CODE_FOR_vpdpbusd_v4si_maskz_1 = 5635,
  CODE_FOR_vpdpbusds_v16si = 5636,
  CODE_FOR_vpdpbusds_v8si = 5637,
  CODE_FOR_vpdpbusds_v4si = 5638,
  CODE_FOR_vpdpbusds_v16si_mask = 5639,
  CODE_FOR_vpdpbusds_v8si_mask = 5640,
  CODE_FOR_vpdpbusds_v4si_mask = 5641,
  CODE_FOR_vpdpbusds_v16si_maskz_1 = 5642,
  CODE_FOR_vpdpbusds_v8si_maskz_1 = 5643,
  CODE_FOR_vpdpbusds_v4si_maskz_1 = 5644,
  CODE_FOR_vpdpwssd_v16si = 5645,
  CODE_FOR_vpdpwssd_v8si = 5646,
  CODE_FOR_vpdpwssd_v4si = 5647,
  CODE_FOR_vpdpwssd_v16si_mask = 5648,
  CODE_FOR_vpdpwssd_v8si_mask = 5649,
  CODE_FOR_vpdpwssd_v4si_mask = 5650,
  CODE_FOR_vpdpwssd_v16si_maskz_1 = 5651,
  CODE_FOR_vpdpwssd_v8si_maskz_1 = 5652,
  CODE_FOR_vpdpwssd_v4si_maskz_1 = 5653,
  CODE_FOR_vpdpwssds_v16si = 5654,
  CODE_FOR_vpdpwssds_v8si = 5655,
  CODE_FOR_vpdpwssds_v4si = 5656,
  CODE_FOR_vpdpwssds_v16si_mask = 5657,
  CODE_FOR_vpdpwssds_v8si_mask = 5658,
  CODE_FOR_vpdpwssds_v4si_mask = 5659,
  CODE_FOR_vpdpwssds_v16si_maskz_1 = 5660,
  CODE_FOR_vpdpwssds_v8si_maskz_1 = 5661,
  CODE_FOR_vpdpwssds_v4si_maskz_1 = 5662,
  CODE_FOR_vaesdec_v32qi = 5663,
  CODE_FOR_vaesdec_v16qi = 5664,
  CODE_FOR_vaesdec_v64qi = 5665,
  CODE_FOR_vaesdeclast_v32qi = 5666,
  CODE_FOR_vaesdeclast_v16qi = 5667,
  CODE_FOR_vaesdeclast_v64qi = 5668,
  CODE_FOR_vaesenc_v32qi = 5669,
  CODE_FOR_vaesenc_v16qi = 5670,
  CODE_FOR_vaesenc_v64qi = 5671,
  CODE_FOR_vaesenclast_v32qi = 5672,
  CODE_FOR_vaesenclast_v16qi = 5673,
  CODE_FOR_vaesenclast_v64qi = 5674,
  CODE_FOR_vpclmulqdq_v8di = 5675,
  CODE_FOR_vpclmulqdq_v4di = 5676,
  CODE_FOR_vpclmulqdq_v2di = 5677,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi = 5678,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi_mask = 5679,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi = 5680,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi_mask = 5681,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi = 5682,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi_mask = 5683,
  CODE_FOR_mfence_sse2 = 5686,
  CODE_FOR_mfence_nosse = 5687,
  CODE_FOR_atomic_loaddi_fpu = 5688,
  CODE_FOR_atomic_storeqi_1 = 5689,
  CODE_FOR_atomic_storehi_1 = 5690,
  CODE_FOR_atomic_storesi_1 = 5691,
  CODE_FOR_atomic_storedi_1 = 5692,
  CODE_FOR_atomic_storedi_fpu = 5693,
  CODE_FOR_loaddi_via_fpu = 5694,
  CODE_FOR_storedi_via_fpu = 5695,
  CODE_FOR_loaddi_via_sse = 5696,
  CODE_FOR_storedi_via_sse = 5697,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 5698,
  CODE_FOR_atomic_compare_and_swapti_doubleword = 5699,
  CODE_FOR_atomic_compare_and_swapqi_1 = 5700,
  CODE_FOR_atomic_compare_and_swaphi_1 = 5701,
  CODE_FOR_atomic_compare_and_swapsi_1 = 5702,
  CODE_FOR_atomic_compare_and_swapdi_1 = 5703,
  CODE_FOR_atomic_fetch_addqi = 5704,
  CODE_FOR_atomic_fetch_addhi = 5705,
  CODE_FOR_atomic_fetch_addsi = 5706,
  CODE_FOR_atomic_fetch_adddi = 5707,
  CODE_FOR_atomic_exchangeqi = 5712,
  CODE_FOR_atomic_exchangehi = 5713,
  CODE_FOR_atomic_exchangesi = 5714,
  CODE_FOR_atomic_exchangedi = 5715,
  CODE_FOR_atomic_addqi = 5716,
  CODE_FOR_atomic_addhi = 5717,
  CODE_FOR_atomic_addsi = 5718,
  CODE_FOR_atomic_adddi = 5719,
  CODE_FOR_atomic_subqi = 5720,
  CODE_FOR_atomic_subhi = 5721,
  CODE_FOR_atomic_subsi = 5722,
  CODE_FOR_atomic_subdi = 5723,
  CODE_FOR_atomic_andqi = 5724,
  CODE_FOR_atomic_orqi = 5725,
  CODE_FOR_atomic_xorqi = 5726,
  CODE_FOR_atomic_andhi = 5727,
  CODE_FOR_atomic_orhi = 5728,
  CODE_FOR_atomic_xorhi = 5729,
  CODE_FOR_atomic_andsi = 5730,
  CODE_FOR_atomic_orsi = 5731,
  CODE_FOR_atomic_xorsi = 5732,
  CODE_FOR_atomic_anddi = 5733,
  CODE_FOR_atomic_ordi = 5734,
  CODE_FOR_atomic_xordi = 5735,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 5736,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 5737,
  CODE_FOR_atomic_bit_test_and_setdi_1 = 5738,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 5739,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 5740,
  CODE_FOR_atomic_bit_test_and_complementdi_1 = 5741,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 5742,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 5743,
  CODE_FOR_atomic_bit_test_and_resetdi_1 = 5744,
  CODE_FOR_cbranchqi4 = 5745,
  CODE_FOR_cbranchhi4 = 5746,
  CODE_FOR_cbranchsi4 = 5747,
  CODE_FOR_cbranchdi4 = 5748,
  CODE_FOR_cbranchti4 = 5749,
  CODE_FOR_cstoreqi4 = 5750,
  CODE_FOR_cstorehi4 = 5751,
  CODE_FOR_cstoresi4 = 5752,
  CODE_FOR_cstoredi4 = 5753,
  CODE_FOR_cmpsi_1 = 5754,
  CODE_FOR_cmpdi_1 = 5755,
  CODE_FOR_cmpqi_ext_3 = 5756,
  CODE_FOR_cbranchxf4 = 5757,
  CODE_FOR_cstorexf4 = 5758,
  CODE_FOR_cbranchsf4 = 5759,
  CODE_FOR_cbranchdf4 = 5760,
  CODE_FOR_cstoresf4 = 5761,
  CODE_FOR_cstoredf4 = 5762,
  CODE_FOR_cbranchcc4 = 5763,
  CODE_FOR_cstorecc4 = 5764,
  CODE_FOR_reload_noff_store = 5765,
  CODE_FOR_reload_noff_load = 5766,
  CODE_FOR_movxi = 5767,
  CODE_FOR_movoi = 5768,
  CODE_FOR_movti = 5769,
  CODE_FOR_movcdi = 5770,
  CODE_FOR_movqi = 5771,
  CODE_FOR_movhi = 5772,
  CODE_FOR_movsi = 5773,
  CODE_FOR_movdi = 5774,
  CODE_FOR_movstrictqi = 5775,
  CODE_FOR_movstricthi = 5776,
  CODE_FOR_extvhi = 5777,
  CODE_FOR_extvsi = 5778,
  CODE_FOR_extzvhi = 5779,
  CODE_FOR_extzvsi = 5780,
  CODE_FOR_extzvdi = 5781,
  CODE_FOR_insvhi = 5782,
  CODE_FOR_insvsi = 5783,
  CODE_FOR_insvdi = 5784,
  CODE_FOR_movtf = 5785,
  CODE_FOR_movsf = 5786,
  CODE_FOR_movdf = 5787,
  CODE_FOR_movxf = 5788,
  CODE_FOR_zero_extendsidi2 = 5789,
  CODE_FOR_zero_extendqisi2 = 5790,
  CODE_FOR_zero_extendhisi2 = 5791,
  CODE_FOR_zero_extendqihi2 = 5792,
  CODE_FOR_extendsidi2 = 5793,
  CODE_FOR_extendsfdf2 = 5794,
  CODE_FOR_extendsfxf2 = 5795,
  CODE_FOR_extenddfxf2 = 5796,
  CODE_FOR_fix_truncxfdi2 = 5797,
  CODE_FOR_fix_truncsfdi2 = 5798,
  CODE_FOR_fix_truncdfdi2 = 5799,
  CODE_FOR_fix_truncxfsi2 = 5800,
  CODE_FOR_fix_truncsfsi2 = 5801,
  CODE_FOR_fix_truncdfsi2 = 5802,
  CODE_FOR_fix_truncsfhi2 = 5803,
  CODE_FOR_fix_truncdfhi2 = 5804,
  CODE_FOR_fix_truncxfhi2 = 5805,
  CODE_FOR_fixuns_truncsfsi2 = 5806,
  CODE_FOR_fixuns_truncdfsi2 = 5807,
  CODE_FOR_fixuns_truncsfhi2 = 5808,
  CODE_FOR_fixuns_truncdfhi2 = 5809,
  CODE_FOR_floatsisf2 = 5810,
  CODE_FOR_floatsidf2 = 5811,
  CODE_FOR_floatdisf2 = 5812,
  CODE_FOR_floatdidf2 = 5813,
  CODE_FOR_floatunsqisf2 = 5814,
  CODE_FOR_floatunshisf2 = 5815,
  CODE_FOR_floatunsqidf2 = 5816,
  CODE_FOR_floatunshidf2 = 5817,
  CODE_FOR_floatunssisf2 = 5818,
  CODE_FOR_floatunssidf2 = 5819,
  CODE_FOR_floatunssixf2 = 5820,
  CODE_FOR_floatunsdisf2 = 5821,
  CODE_FOR_floatunsdidf2 = 5822,
  CODE_FOR_addqi3 = 5823,
  CODE_FOR_addhi3 = 5824,
  CODE_FOR_addsi3 = 5825,
  CODE_FOR_adddi3 = 5826,
  CODE_FOR_addti3 = 5827,
  CODE_FOR_addvqi4 = 5828,
  CODE_FOR_addvhi4 = 5829,
  CODE_FOR_addvsi4 = 5830,
  CODE_FOR_addvdi4 = 5831,
  CODE_FOR_uaddvqi4 = 5832,
  CODE_FOR_uaddvhi4 = 5833,
  CODE_FOR_uaddvsi4 = 5834,
  CODE_FOR_uaddvdi4 = 5835,
  CODE_FOR_subqi3 = 5836,
  CODE_FOR_subhi3 = 5837,
  CODE_FOR_subsi3 = 5838,
  CODE_FOR_subdi3 = 5839,
  CODE_FOR_subti3 = 5840,
  CODE_FOR_subvqi4 = 5841,
  CODE_FOR_subvhi4 = 5842,
  CODE_FOR_subvsi4 = 5843,
  CODE_FOR_subvdi4 = 5844,
  CODE_FOR_usubvqi4 = 5845,
  CODE_FOR_usubvhi4 = 5846,
  CODE_FOR_usubvsi4 = 5847,
  CODE_FOR_usubvdi4 = 5848,
  CODE_FOR_addcarrysi_0 = 5849,
  CODE_FOR_addcarrydi_0 = 5850,
  CODE_FOR_subborrowsi_0 = 5851,
  CODE_FOR_subborrowdi_0 = 5852,
  CODE_FOR_addqi3_cconly_overflow = 5853,
  CODE_FOR_addxf3 = 5854,
  CODE_FOR_subxf3 = 5855,
  CODE_FOR_addsf3 = 5856,
  CODE_FOR_subsf3 = 5857,
  CODE_FOR_adddf3 = 5858,
  CODE_FOR_subdf3 = 5859,
  CODE_FOR_mulhi3 = 5860,
  CODE_FOR_mulsi3 = 5861,
  CODE_FOR_muldi3 = 5862,
  CODE_FOR_mulqi3 = 5863,
  CODE_FOR_mulvhi4 = 5864,
  CODE_FOR_mulvsi4 = 5865,
  CODE_FOR_mulvdi4 = 5866,
  CODE_FOR_umulvhi4 = 5867,
  CODE_FOR_umulvsi4 = 5868,
  CODE_FOR_umulvdi4 = 5869,
  CODE_FOR_mulvqi4 = 5870,
  CODE_FOR_umulvqi4 = 5871,
  CODE_FOR_mulsidi3 = 5872,
  CODE_FOR_umulsidi3 = 5873,
  CODE_FOR_mulditi3 = 5874,
  CODE_FOR_umulditi3 = 5875,
  CODE_FOR_mulqihi3 = 5876,
  CODE_FOR_umulqihi3 = 5877,
  CODE_FOR_smulsi3_highpart = 5878,
  CODE_FOR_umulsi3_highpart = 5879,
  CODE_FOR_smuldi3_highpart = 5880,
  CODE_FOR_umuldi3_highpart = 5881,
  CODE_FOR_mulxf3 = 5882,
  CODE_FOR_mulsf3 = 5883,
  CODE_FOR_muldf3 = 5884,
  CODE_FOR_divxf3 = 5885,
  CODE_FOR_divsf3 = 5886,
  CODE_FOR_divdf3 = 5887,
  CODE_FOR_divmodhi4 = 5888,
  CODE_FOR_divmodsi4 = 5889,
  CODE_FOR_divmoddi4 = 5890,
  CODE_FOR_divmodqi4 = 5891,
  CODE_FOR_udivmodhi4 = 5892,
  CODE_FOR_udivmodsi4 = 5893,
  CODE_FOR_udivmoddi4 = 5894,
  CODE_FOR_udivmodqi4 = 5895,
  CODE_FOR_testsi_ccno_1 = 5896,
  CODE_FOR_testqi_ccz_1 = 5897,
  CODE_FOR_testdi_ccno_1 = 5898,
  CODE_FOR_testqi_ext_1_ccno = 5899,
  CODE_FOR_andqi3 = 5900,
  CODE_FOR_andhi3 = 5901,
  CODE_FOR_andsi3 = 5902,
  CODE_FOR_anddi3 = 5903,
  CODE_FOR_iorqi3 = 5904,
  CODE_FOR_xorqi3 = 5905,
  CODE_FOR_iorhi3 = 5906,
  CODE_FOR_xorhi3 = 5907,
  CODE_FOR_iorsi3 = 5908,
  CODE_FOR_xorsi3 = 5909,
  CODE_FOR_iordi3 = 5910,
  CODE_FOR_xordi3 = 5911,
  CODE_FOR_xorqi_ext_1_cc = 5912,
  CODE_FOR_negqi2 = 5913,
  CODE_FOR_neghi2 = 5914,
  CODE_FOR_negsi2 = 5915,
  CODE_FOR_negdi2 = 5916,
  CODE_FOR_negti2 = 5917,
  CODE_FOR_negvqi3 = 5918,
  CODE_FOR_negvhi3 = 5919,
  CODE_FOR_negvsi3 = 5920,
  CODE_FOR_negvdi3 = 5921,
  CODE_FOR_abssf2 = 5922,
  CODE_FOR_negsf2 = 5923,
  CODE_FOR_absdf2 = 5924,
  CODE_FOR_negdf2 = 5925,
  CODE_FOR_absxf2 = 5926,
  CODE_FOR_negxf2 = 5927,
  CODE_FOR_abstf2 = 5928,
  CODE_FOR_negtf2 = 5929,
  CODE_FOR_copysignsf3 = 5930,
  CODE_FOR_copysigndf3 = 5931,
  CODE_FOR_copysigntf3 = 5932,
  CODE_FOR_xorsignsf3 = 5933,
  CODE_FOR_xorsigndf3 = 5934,
  CODE_FOR_one_cmplqi2 = 5935,
  CODE_FOR_one_cmplhi2 = 5936,
  CODE_FOR_one_cmplsi2 = 5937,
  CODE_FOR_one_cmpldi2 = 5938,
  CODE_FOR_ashlqi3 = 5939,
  CODE_FOR_ashlhi3 = 5940,
  CODE_FOR_ashlsi3 = 5941,
  CODE_FOR_ashldi3 = 5942,
  CODE_FOR_ashlti3 = 5943,
  CODE_FOR_x86_shiftsi_adj_1 = 5944,
  CODE_FOR_x86_shiftdi_adj_1 = 5945,
  CODE_FOR_x86_shiftsi_adj_2 = 5946,
  CODE_FOR_x86_shiftdi_adj_2 = 5947,
  CODE_FOR_lshrqi3 = 5948,
  CODE_FOR_ashrqi3 = 5949,
  CODE_FOR_lshrhi3 = 5950,
  CODE_FOR_ashrhi3 = 5951,
  CODE_FOR_lshrsi3 = 5952,
  CODE_FOR_ashrsi3 = 5953,
  CODE_FOR_lshrdi3 = 5954,
  CODE_FOR_ashrdi3 = 5955,
  CODE_FOR_lshrti3 = 5956,
  CODE_FOR_ashrti3 = 5957,
  CODE_FOR_x86_shiftsi_adj_3 = 5958,
  CODE_FOR_x86_shiftdi_adj_3 = 5959,
  CODE_FOR_rotlti3 = 5960,
  CODE_FOR_rotrti3 = 5961,
  CODE_FOR_rotldi3 = 5962,
  CODE_FOR_rotrdi3 = 5963,
  CODE_FOR_rotlqi3 = 5964,
  CODE_FOR_rotrqi3 = 5965,
  CODE_FOR_rotlhi3 = 5966,
  CODE_FOR_rotrhi3 = 5967,
  CODE_FOR_rotlsi3 = 5968,
  CODE_FOR_rotrsi3 = 5969,
  CODE_FOR_indirect_jump = 5970,
  CODE_FOR_tablejump = 5971,
  CODE_FOR_call = 5972,
  CODE_FOR_sibcall = 5973,
  CODE_FOR_call_pop = 5974,
  CODE_FOR_call_value = 5975,
  CODE_FOR_sibcall_value = 5976,
  CODE_FOR_call_value_pop = 5977,
  CODE_FOR_untyped_call = 5978,
  CODE_FOR_memory_blockage = 5979,
  CODE_FOR_return = 5980,
  CODE_FOR_simple_return = 5981,
  CODE_FOR_simple_return_indirect_internal = 5982,
  CODE_FOR_prologue = 5983,
  CODE_FOR_set_got = 5984,
  CODE_FOR_set_got_labelled = 5985,
  CODE_FOR_epilogue = 5986,
  CODE_FOR_sibcall_epilogue = 5987,
  CODE_FOR_eh_return = 5988,
  CODE_FOR_split_stack_prologue = 5989,
  CODE_FOR_split_stack_space_check = 5990,
  CODE_FOR_ffssi2 = 5991,
  CODE_FOR_ffsdi2 = 5992,
  CODE_FOR_clzsi2 = 5993,
  CODE_FOR_clzdi2 = 5994,
  CODE_FOR_bmi2_bzhi_si3 = 5995,
  CODE_FOR_bmi2_bzhi_di3 = 5996,
  CODE_FOR_tbm_bextri_si = 5997,
  CODE_FOR_tbm_bextri_di = 5998,
  CODE_FOR_bswapdi2 = 5999,
  CODE_FOR_bswapsi2 = 6000,
  CODE_FOR_bswaphi2 = 6001,
  CODE_FOR_paritydi2 = 6002,
  CODE_FOR_paritysi2 = 6003,
  CODE_FOR_tls_global_dynamic_32 = 6004,
  CODE_FOR_tls_global_dynamic_64_si = 6005,
  CODE_FOR_tls_global_dynamic_64_di = 6006,
  CODE_FOR_tls_local_dynamic_base_32 = 6007,
  CODE_FOR_tls_local_dynamic_base_64_si = 6008,
  CODE_FOR_tls_local_dynamic_base_64_di = 6009,
  CODE_FOR_tls_dynamic_gnu2_32 = 6010,
  CODE_FOR_tls_dynamic_gnu2_64 = 6011,
  CODE_FOR_rsqrtsf2 = 6012,
  CODE_FOR_sqrtsf2 = 6013,
  CODE_FOR_sqrtdf2 = 6014,
  CODE_FOR_hypotsf3 = 6015,
  CODE_FOR_hypotdf3 = 6016,
  CODE_FOR_fmodxf3 = 6017,
  CODE_FOR_fmodsf3 = 6018,
  CODE_FOR_fmoddf3 = 6019,
  CODE_FOR_remainderxf3 = 6020,
  CODE_FOR_remaindersf3 = 6021,
  CODE_FOR_remainderdf3 = 6022,
  CODE_FOR_sinsf2 = 6023,
  CODE_FOR_cossf2 = 6024,
  CODE_FOR_sindf2 = 6025,
  CODE_FOR_cosdf2 = 6026,
  CODE_FOR_sincossf3 = 6027,
  CODE_FOR_sincosdf3 = 6028,
  CODE_FOR_tanxf2 = 6029,
  CODE_FOR_tansf2 = 6030,
  CODE_FOR_tandf2 = 6031,
  CODE_FOR_atan2sf3 = 6032,
  CODE_FOR_atan2df3 = 6033,
  CODE_FOR_atanxf2 = 6034,
  CODE_FOR_atansf2 = 6035,
  CODE_FOR_atandf2 = 6036,
  CODE_FOR_asinxf2 = 6037,
  CODE_FOR_asinsf2 = 6038,
  CODE_FOR_asindf2 = 6039,
  CODE_FOR_acosxf2 = 6040,
  CODE_FOR_acossf2 = 6041,
  CODE_FOR_acosdf2 = 6042,
  CODE_FOR_sinhxf2 = 6043,
  CODE_FOR_sinhsf2 = 6044,
  CODE_FOR_sinhdf2 = 6045,
  CODE_FOR_coshxf2 = 6046,
  CODE_FOR_coshsf2 = 6047,
  CODE_FOR_coshdf2 = 6048,
  CODE_FOR_tanhxf2 = 6049,
  CODE_FOR_tanhsf2 = 6050,
  CODE_FOR_tanhdf2 = 6051,
  CODE_FOR_asinhxf2 = 6052,
  CODE_FOR_asinhsf2 = 6053,
  CODE_FOR_asinhdf2 = 6054,
  CODE_FOR_acoshxf2 = 6055,
  CODE_FOR_acoshsf2 = 6056,
  CODE_FOR_acoshdf2 = 6057,
  CODE_FOR_atanhxf2 = 6058,
  CODE_FOR_atanhsf2 = 6059,
  CODE_FOR_atanhdf2 = 6060,
  CODE_FOR_logxf2 = 6061,
  CODE_FOR_logsf2 = 6062,
  CODE_FOR_logdf2 = 6063,
  CODE_FOR_log10xf2 = 6064,
  CODE_FOR_log10sf2 = 6065,
  CODE_FOR_log10df2 = 6066,
  CODE_FOR_log2xf2 = 6067,
  CODE_FOR_log2sf2 = 6068,
  CODE_FOR_log2df2 = 6069,
  CODE_FOR_log1pxf2 = 6070,
  CODE_FOR_log1psf2 = 6071,
  CODE_FOR_log1pdf2 = 6072,
  CODE_FOR_logbxf2 = 6073,
  CODE_FOR_logbsf2 = 6074,
  CODE_FOR_logbdf2 = 6075,
  CODE_FOR_ilogbxf2 = 6076,
  CODE_FOR_ilogbsf2 = 6077,
  CODE_FOR_ilogbdf2 = 6078,
  CODE_FOR_expNcorexf3 = 6079,
  CODE_FOR_expxf2 = 6080,
  CODE_FOR_expsf2 = 6081,
  CODE_FOR_expdf2 = 6082,
  CODE_FOR_exp10xf2 = 6083,
  CODE_FOR_exp10sf2 = 6084,
  CODE_FOR_exp10df2 = 6085,
  CODE_FOR_exp2xf2 = 6086,
  CODE_FOR_exp2sf2 = 6087,
  CODE_FOR_exp2df2 = 6088,
  CODE_FOR_expm1xf2 = 6089,
  CODE_FOR_expm1sf2 = 6090,
  CODE_FOR_expm1df2 = 6091,
  CODE_FOR_ldexpxf3 = 6092,
  CODE_FOR_ldexpsf3 = 6093,
  CODE_FOR_ldexpdf3 = 6094,
  CODE_FOR_scalbxf3 = 6095,
  CODE_FOR_scalbsf3 = 6096,
  CODE_FOR_scalbdf3 = 6097,
  CODE_FOR_significandxf2 = 6098,
  CODE_FOR_significandsf2 = 6099,
  CODE_FOR_significanddf2 = 6100,
  CODE_FOR_rintsf2 = 6101,
  CODE_FOR_rintdf2 = 6102,
  CODE_FOR_nearbyintxf2 = 6103,
  CODE_FOR_nearbyintsf2 = 6104,
  CODE_FOR_nearbyintdf2 = 6105,
  CODE_FOR_roundsf2 = 6106,
  CODE_FOR_rounddf2 = 6107,
  CODE_FOR_roundxf2 = 6108,
  CODE_FOR_lrintsfsi2 = 6109,
  CODE_FOR_lrintsfdi2 = 6110,
  CODE_FOR_lrintdfsi2 = 6111,
  CODE_FOR_lrintdfdi2 = 6112,
  CODE_FOR_lroundsfhi2 = 6113,
  CODE_FOR_lrounddfhi2 = 6114,
  CODE_FOR_lroundxfhi2 = 6115,
  CODE_FOR_lroundsfsi2 = 6116,
  CODE_FOR_lrounddfsi2 = 6117,
  CODE_FOR_lroundxfsi2 = 6118,
  CODE_FOR_lroundsfdi2 = 6119,
  CODE_FOR_lrounddfdi2 = 6120,
  CODE_FOR_lroundxfdi2 = 6121,
  CODE_FOR_floorxf2 = 6122,
  CODE_FOR_ceilxf2 = 6123,
  CODE_FOR_btruncxf2 = 6124,
  CODE_FOR_floorsf2 = 6125,
  CODE_FOR_ceilsf2 = 6126,
  CODE_FOR_btruncsf2 = 6127,
  CODE_FOR_floordf2 = 6128,
  CODE_FOR_ceildf2 = 6129,
  CODE_FOR_btruncdf2 = 6130,
  CODE_FOR_lfloorxfhi2 = 6131,
  CODE_FOR_lceilxfhi2 = 6132,
  CODE_FOR_lfloorxfsi2 = 6133,
  CODE_FOR_lceilxfsi2 = 6134,
  CODE_FOR_lfloorxfdi2 = 6135,
  CODE_FOR_lceilxfdi2 = 6136,
  CODE_FOR_lfloorsfsi2 = 6137,
  CODE_FOR_lceilsfsi2 = 6138,
  CODE_FOR_lfloorsfdi2 = 6139,
  CODE_FOR_lceilsfdi2 = 6140,
  CODE_FOR_lfloordfsi2 = 6141,
  CODE_FOR_lceildfsi2 = 6142,
  CODE_FOR_lfloordfdi2 = 6143,
  CODE_FOR_lceildfdi2 = 6144,
  CODE_FOR_signbittf2 = 6145,
  CODE_FOR_signbitxf2 = 6146,
  CODE_FOR_signbitdf2 = 6147,
  CODE_FOR_signbitsf2 = 6148,
  CODE_FOR_movmemsi = 6149,
  CODE_FOR_movmemdi = 6150,
  CODE_FOR_strmov = 6151,
  CODE_FOR_strmov_singleop = 6152,
  CODE_FOR_rep_mov = 6153,
  CODE_FOR_setmemsi = 6154,
  CODE_FOR_setmemdi = 6155,
  CODE_FOR_strset = 6156,
  CODE_FOR_strset_singleop = 6157,
  CODE_FOR_rep_stos = 6158,
  CODE_FOR_cmpstrnsi = 6159,
  CODE_FOR_cmpintqi = 6160,
  CODE_FOR_cmpstrnqi_nz_1 = 6161,
  CODE_FOR_cmpstrnqi_1 = 6162,
  CODE_FOR_strlensi = 6163,
  CODE_FOR_strlendi = 6164,
  CODE_FOR_strlenqi_1 = 6165,
  CODE_FOR_movqicc = 6166,
  CODE_FOR_movhicc = 6167,
  CODE_FOR_movsicc = 6168,
  CODE_FOR_movdicc = 6169,
  CODE_FOR_x86_movsicc_0_m1 = 6170,
  CODE_FOR_x86_movdicc_0_m1 = 6171,
  CODE_FOR_movsfcc = 6172,
  CODE_FOR_movdfcc = 6173,
  CODE_FOR_movxfcc = 6174,
  CODE_FOR_addqicc = 6175,
  CODE_FOR_addhicc = 6176,
  CODE_FOR_addsicc = 6177,
  CODE_FOR_adddicc = 6178,
  CODE_FOR_allocate_stack = 6179,
  CODE_FOR_probe_stack = 6180,
  CODE_FOR_builtin_setjmp_receiver = 6181,
  CODE_FOR_save_stack_nonlocal = 6182,
  CODE_FOR_restore_stack_nonlocal = 6183,
  CODE_FOR_prefetch = 6184,
  CODE_FOR_stack_protect_set = 6185,
  CODE_FOR_stack_protect_test = 6186,
  CODE_FOR_lwp_llwpcb = 6187,
  CODE_FOR_lwp_slwpcb = 6188,
  CODE_FOR_lwp_lwpvalsi3 = 6189,
  CODE_FOR_lwp_lwpvaldi3 = 6190,
  CODE_FOR_lwp_lwpinssi3 = 6191,
  CODE_FOR_lwp_lwpinsdi3 = 6192,
  CODE_FOR_pause = 6193,
  CODE_FOR_rstorssp = 6194,
  CODE_FOR_clrssbsy = 6195,
  CODE_FOR_xbegin = 6196,
  CODE_FOR_xtest = 6197,
  CODE_FOR_rdpkru = 6198,
  CODE_FOR_wrpkru = 6199,
  CODE_FOR_movv8qi = 6200,
  CODE_FOR_movv4hi = 6201,
  CODE_FOR_movv2si = 6202,
  CODE_FOR_movv1di = 6203,
  CODE_FOR_movv2sf = 6204,
  CODE_FOR_movmisalignv8qi = 6205,
  CODE_FOR_movmisalignv4hi = 6206,
  CODE_FOR_movmisalignv2si = 6207,
  CODE_FOR_movmisalignv1di = 6208,
  CODE_FOR_movmisalignv2sf = 6209,
  CODE_FOR_mmx_addv2sf3 = 6210,
  CODE_FOR_mmx_subv2sf3 = 6211,
  CODE_FOR_mmx_subrv2sf3 = 6212,
  CODE_FOR_mmx_mulv2sf3 = 6213,
  CODE_FOR_mmx_smaxv2sf3 = 6214,
  CODE_FOR_mmx_sminv2sf3 = 6215,
  CODE_FOR_mmx_eqv2sf3 = 6216,
  CODE_FOR_vec_setv2sf = 6217,
  CODE_FOR_vec_extractv2sfsf = 6218,
  CODE_FOR_vec_initv2sfsf = 6219,
  CODE_FOR_mmx_addv8qi3 = 6220,
  CODE_FOR_mmx_subv8qi3 = 6221,
  CODE_FOR_mmx_addv4hi3 = 6222,
  CODE_FOR_mmx_subv4hi3 = 6223,
  CODE_FOR_mmx_addv2si3 = 6224,
  CODE_FOR_mmx_subv2si3 = 6225,
  CODE_FOR_mmx_addv1di3 = 6226,
  CODE_FOR_mmx_subv1di3 = 6227,
  CODE_FOR_mmx_ssaddv8qi3 = 6228,
  CODE_FOR_mmx_usaddv8qi3 = 6229,
  CODE_FOR_mmx_sssubv8qi3 = 6230,
  CODE_FOR_mmx_ussubv8qi3 = 6231,
  CODE_FOR_mmx_ssaddv4hi3 = 6232,
  CODE_FOR_mmx_usaddv4hi3 = 6233,
  CODE_FOR_mmx_sssubv4hi3 = 6234,
  CODE_FOR_mmx_ussubv4hi3 = 6235,
  CODE_FOR_mmx_mulv4hi3 = 6236,
  CODE_FOR_mmx_smulv4hi3_highpart = 6237,
  CODE_FOR_mmx_umulv4hi3_highpart = 6238,
  CODE_FOR_mmx_pmaddwd = 6239,
  CODE_FOR_mmx_pmulhrwv4hi3 = 6240,
  CODE_FOR_sse2_umulv1siv1di3 = 6241,
  CODE_FOR_mmx_smaxv4hi3 = 6242,
  CODE_FOR_mmx_sminv4hi3 = 6243,
  CODE_FOR_mmx_umaxv8qi3 = 6244,
  CODE_FOR_mmx_uminv8qi3 = 6245,
  CODE_FOR_mmx_eqv8qi3 = 6246,
  CODE_FOR_mmx_eqv4hi3 = 6247,
  CODE_FOR_mmx_eqv2si3 = 6248,
  CODE_FOR_mmx_andv8qi3 = 6249,
  CODE_FOR_mmx_iorv8qi3 = 6250,
  CODE_FOR_mmx_xorv8qi3 = 6251,
  CODE_FOR_mmx_andv4hi3 = 6252,
  CODE_FOR_mmx_iorv4hi3 = 6253,
  CODE_FOR_mmx_xorv4hi3 = 6254,
  CODE_FOR_mmx_andv2si3 = 6255,
  CODE_FOR_mmx_iorv2si3 = 6256,
  CODE_FOR_mmx_xorv2si3 = 6257,
  CODE_FOR_mmx_pinsrw = 6258,
  CODE_FOR_mmx_pshufw = 6259,
  CODE_FOR_vec_setv2si = 6260,
  CODE_FOR_vec_extractv2sisi = 6261,
  CODE_FOR_vec_initv2sisi = 6262,
  CODE_FOR_vec_setv4hi = 6263,
  CODE_FOR_vec_extractv4hihi = 6264,
  CODE_FOR_vec_initv4hihi = 6265,
  CODE_FOR_vec_setv8qi = 6266,
  CODE_FOR_vec_extractv8qiqi = 6267,
  CODE_FOR_vec_initv8qiqi = 6268,
  CODE_FOR_mmx_uavgv8qi3 = 6269,
  CODE_FOR_mmx_uavgv4hi3 = 6270,
  CODE_FOR_mmx_maskmovq = 6271,
  CODE_FOR_movv64qi = 6272,
  CODE_FOR_movv32qi = 6273,
  CODE_FOR_movv16qi = 6274,
  CODE_FOR_movv32hi = 6275,
  CODE_FOR_movv16hi = 6276,
  CODE_FOR_movv8hi = 6277,
  CODE_FOR_movv16si = 6278,
  CODE_FOR_movv8si = 6279,
  CODE_FOR_movv4si = 6280,
  CODE_FOR_movv8di = 6281,
  CODE_FOR_movv4di = 6282,
  CODE_FOR_movv2di = 6283,
  CODE_FOR_movv4ti = 6284,
  CODE_FOR_movv2ti = 6285,
  CODE_FOR_movv1ti = 6286,
  CODE_FOR_movv16sf = 6287,
  CODE_FOR_movv8sf = 6288,
  CODE_FOR_movv4sf = 6289,
  CODE_FOR_movv8df = 6290,
  CODE_FOR_movv4df = 6291,
  CODE_FOR_movv2df = 6292,
  CODE_FOR_avx512f_loadsf_mask = 6293,
  CODE_FOR_avx512f_loaddf_mask = 6294,
  CODE_FOR_movmisalignv64qi = 6295,
  CODE_FOR_movmisalignv32qi = 6296,
  CODE_FOR_movmisalignv16qi = 6297,
  CODE_FOR_movmisalignv32hi = 6298,
  CODE_FOR_movmisalignv16hi = 6299,
  CODE_FOR_movmisalignv8hi = 6300,
  CODE_FOR_movmisalignv16si = 6301,
  CODE_FOR_movmisalignv8si = 6302,
  CODE_FOR_movmisalignv4si = 6303,
  CODE_FOR_movmisalignv8di = 6304,
  CODE_FOR_movmisalignv4di = 6305,
  CODE_FOR_movmisalignv2di = 6306,
  CODE_FOR_movmisalignv4ti = 6307,
  CODE_FOR_movmisalignv2ti = 6308,
  CODE_FOR_movmisalignv1ti = 6309,
  CODE_FOR_movmisalignv16sf = 6310,
  CODE_FOR_movmisalignv8sf = 6311,
  CODE_FOR_movmisalignv4sf = 6312,
  CODE_FOR_movmisalignv8df = 6313,
  CODE_FOR_movmisalignv4df = 6314,
  CODE_FOR_movmisalignv2df = 6315,
  CODE_FOR_storentdi = 6316,
  CODE_FOR_storentsi = 6317,
  CODE_FOR_storentsf = 6318,
  CODE_FOR_storentdf = 6319,
  CODE_FOR_storentv8di = 6320,
  CODE_FOR_storentv4di = 6321,
  CODE_FOR_storentv2di = 6322,
  CODE_FOR_storentv16sf = 6323,
  CODE_FOR_storentv8sf = 6324,
  CODE_FOR_storentv4sf = 6325,
  CODE_FOR_storentv8df = 6326,
  CODE_FOR_storentv4df = 6327,
  CODE_FOR_storentv2df = 6328,
  CODE_FOR_kmovb = 6329,
  CODE_FOR_kmovw = 6330,
  CODE_FOR_kmovd = 6331,
  CODE_FOR_kmovq = 6332,
  CODE_FOR_absv16sf2 = 6333,
  CODE_FOR_negv16sf2 = 6334,
  CODE_FOR_absv8sf2 = 6335,
  CODE_FOR_negv8sf2 = 6336,
  CODE_FOR_absv4sf2 = 6337,
  CODE_FOR_negv4sf2 = 6338,
  CODE_FOR_absv8df2 = 6339,
  CODE_FOR_negv8df2 = 6340,
  CODE_FOR_absv4df2 = 6341,
  CODE_FOR_negv4df2 = 6342,
  CODE_FOR_absv2df2 = 6343,
  CODE_FOR_negv2df2 = 6344,
  CODE_FOR_addv16sf3 = 6345,
  CODE_FOR_addv16sf3_round = 6346,
  CODE_FOR_addv16sf3_mask = 6347,
  CODE_FOR_addv16sf3_mask_round = 6348,
  CODE_FOR_subv16sf3 = 6349,
  CODE_FOR_subv16sf3_round = 6350,
  CODE_FOR_subv16sf3_mask = 6351,
  CODE_FOR_subv16sf3_mask_round = 6352,
  CODE_FOR_addv8sf3 = 6353,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 6354,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 6355,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 6356,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 6357,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 6358,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 6359,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 6360,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 6361,
  CODE_FOR_addv8df3_round = 6362,
  CODE_FOR_addv8df3_mask = 6363,
  CODE_FOR_addv8df3_mask_round = 6364,
  CODE_FOR_subv8df3 = 6365,
  CODE_FOR_subv8df3_round = 6366,
  CODE_FOR_subv8df3_mask = 6367,
  CODE_FOR_subv8df3_mask_round = 6368,
  CODE_FOR_addv4df3 = 6369,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 6370,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 6371,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 6372,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 6373,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 6374,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 6375,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 6376,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 6377,
  CODE_FOR_mulv16sf3_round = 6378,
  CODE_FOR_mulv16sf3_mask = 6379,
  CODE_FOR_mulv16sf3_mask_round = 6380,
  CODE_FOR_mulv8sf3 = 6381,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 6382,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 6383,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 6384,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 6385,
  CODE_FOR_mulv8df3_round = 6386,
  CODE_FOR_mulv8df3_mask = 6387,
  CODE_FOR_mulv8df3_mask_round = 6388,
  CODE_FOR_mulv4df3 = 6389,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 6390,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 6391,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 6392,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_divv8df3 = 6393,
  CODE_FOR_divv4df3 = 6394,
  CODE_FOR_divv2df3 = 6395,
  CODE_FOR_divv16sf3 = 6396,
  CODE_FOR_divv8sf3 = 6397,
  CODE_FOR_divv4sf3 = 6398,
  CODE_FOR_sqrtv8df2 = 6399,
  CODE_FOR_sqrtv4df2 = 6400,
  CODE_FOR_sqrtv2df2 = 6401,
  CODE_FOR_sqrtv16sf2 = 6402,
  CODE_FOR_sqrtv8sf2 = 6403,
  CODE_FOR_sqrtv4sf2 = 6404,
  CODE_FOR_rsqrtv8sf2 = 6405,
  CODE_FOR_rsqrtv4sf2 = 6406,
  CODE_FOR_rsqrtv16sf2 = 6407,
  CODE_FOR_smaxv16sf3 = 6408,
  CODE_FOR_smaxv16sf3_round = 6409,
  CODE_FOR_smaxv16sf3_mask = 6410,
  CODE_FOR_smaxv16sf3_mask_round = 6411,
  CODE_FOR_sminv16sf3 = 6412,
  CODE_FOR_sminv16sf3_round = 6413,
  CODE_FOR_sminv16sf3_mask = 6414,
  CODE_FOR_sminv16sf3_mask_round = 6415,
  CODE_FOR_smaxv8sf3 = 6416,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 6417,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 6418,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 6419,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 6420,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 6421,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 6422,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 6423,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 6424,
  CODE_FOR_smaxv8df3_round = 6425,
  CODE_FOR_smaxv8df3_mask = 6426,
  CODE_FOR_smaxv8df3_mask_round = 6427,
  CODE_FOR_sminv8df3 = 6428,
  CODE_FOR_sminv8df3_round = 6429,
  CODE_FOR_sminv8df3_mask = 6430,
  CODE_FOR_sminv8df3_mask_round = 6431,
  CODE_FOR_smaxv4df3 = 6432,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 6433,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 6434,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 6435,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 6436,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 6437,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 6438,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 6439,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse3_haddv2df3 = 6440,
  CODE_FOR_reduc_plus_scal_v2df = 6441,
  CODE_FOR_reduc_plus_scal_v4sf = 6442,
  CODE_FOR_reduc_plus_scal_v4df = 6443,
  CODE_FOR_reduc_plus_scal_v8sf = 6444,
  CODE_FOR_reduc_plus_scal_v8df = 6445,
  CODE_FOR_reduc_plus_scal_v16sf = 6446,
  CODE_FOR_reduc_smax_scal_v4sf = 6447,
  CODE_FOR_reduc_smin_scal_v4sf = 6448,
  CODE_FOR_reduc_smax_scal_v2df = 6449,
  CODE_FOR_reduc_smin_scal_v2df = 6450,
  CODE_FOR_reduc_smax_scal_v2di = 6451,
  CODE_FOR_reduc_smin_scal_v2di = 6452,
  CODE_FOR_reduc_smax_scal_v4si = 6453,
  CODE_FOR_reduc_smin_scal_v4si = 6454,
  CODE_FOR_reduc_smax_scal_v8hi = 6455,
  CODE_FOR_reduc_smin_scal_v8hi = 6456,
  CODE_FOR_reduc_smax_scal_v16qi = 6457,
  CODE_FOR_reduc_smin_scal_v16qi = 6458,
  CODE_FOR_reduc_smax_scal_v32qi = 6459,
  CODE_FOR_reduc_smin_scal_v32qi = 6460,
  CODE_FOR_reduc_smax_scal_v16hi = 6461,
  CODE_FOR_reduc_smin_scal_v16hi = 6462,
  CODE_FOR_reduc_smax_scal_v8si = 6463,
  CODE_FOR_reduc_smin_scal_v8si = 6464,
  CODE_FOR_reduc_smax_scal_v4di = 6465,
  CODE_FOR_reduc_smin_scal_v4di = 6466,
  CODE_FOR_reduc_smax_scal_v8sf = 6467,
  CODE_FOR_reduc_smin_scal_v8sf = 6468,
  CODE_FOR_reduc_smax_scal_v4df = 6469,
  CODE_FOR_reduc_smin_scal_v4df = 6470,
  CODE_FOR_reduc_smax_scal_v64qi = 6471,
  CODE_FOR_reduc_smin_scal_v64qi = 6472,
  CODE_FOR_reduc_smax_scal_v32hi = 6473,
  CODE_FOR_reduc_smin_scal_v32hi = 6474,
  CODE_FOR_reduc_smax_scal_v16si = 6475,
  CODE_FOR_reduc_smin_scal_v16si = 6476,
  CODE_FOR_reduc_smax_scal_v8di = 6477,
  CODE_FOR_reduc_smin_scal_v8di = 6478,
  CODE_FOR_reduc_smax_scal_v16sf = 6479,
  CODE_FOR_reduc_smin_scal_v16sf = 6480,
  CODE_FOR_reduc_smax_scal_v8df = 6481,
  CODE_FOR_reduc_smin_scal_v8df = 6482,
  CODE_FOR_reduc_umax_scal_v16si = 6483,
  CODE_FOR_reduc_umin_scal_v16si = 6484,
  CODE_FOR_reduc_umax_scal_v8di = 6485,
  CODE_FOR_reduc_umin_scal_v8di = 6486,
  CODE_FOR_reduc_umax_scal_v32hi = 6487,
  CODE_FOR_reduc_umin_scal_v32hi = 6488,
  CODE_FOR_reduc_umax_scal_v64qi = 6489,
  CODE_FOR_reduc_umin_scal_v64qi = 6490,
  CODE_FOR_reduc_umax_scal_v32qi = 6491,
  CODE_FOR_reduc_umin_scal_v32qi = 6492,
  CODE_FOR_reduc_umax_scal_v16hi = 6493,
  CODE_FOR_reduc_umin_scal_v16hi = 6494,
  CODE_FOR_reduc_umax_scal_v8si = 6495,
  CODE_FOR_reduc_umin_scal_v8si = 6496,
  CODE_FOR_reduc_umax_scal_v4di = 6497,
  CODE_FOR_reduc_umin_scal_v4di = 6498,
  CODE_FOR_reduc_umin_scal_v8hi = 6499,
  CODE_FOR_vec_cmpv16sihi = 6500,
  CODE_FOR_vec_cmpv8siqi = 6501,
  CODE_FOR_vec_cmpv4siqi = 6502,
  CODE_FOR_vec_cmpv8diqi = 6503,
  CODE_FOR_vec_cmpv4diqi = 6504,
  CODE_FOR_vec_cmpv2diqi = 6505,
  CODE_FOR_vec_cmpv16sfhi = 6506,
  CODE_FOR_vec_cmpv8sfqi = 6507,
  CODE_FOR_vec_cmpv4sfqi = 6508,
  CODE_FOR_vec_cmpv8dfqi = 6509,
  CODE_FOR_vec_cmpv4dfqi = 6510,
  CODE_FOR_vec_cmpv2dfqi = 6511,
  CODE_FOR_vec_cmpv64qidi = 6512,
  CODE_FOR_vec_cmpv16qihi = 6513,
  CODE_FOR_vec_cmpv32qisi = 6514,
  CODE_FOR_vec_cmpv32hisi = 6515,
  CODE_FOR_vec_cmpv16hihi = 6516,
  CODE_FOR_vec_cmpv8hiqi = 6517,
  CODE_FOR_vec_cmpv32qiv32qi = 6518,
  CODE_FOR_vec_cmpv16hiv16hi = 6519,
  CODE_FOR_vec_cmpv8siv8si = 6520,
  CODE_FOR_vec_cmpv4div4di = 6521,
  CODE_FOR_vec_cmpv16qiv16qi = 6522,
  CODE_FOR_vec_cmpv8hiv8hi = 6523,
  CODE_FOR_vec_cmpv4siv4si = 6524,
  CODE_FOR_vec_cmpv2div2di = 6525,
  CODE_FOR_vec_cmpv8sfv8si = 6526,
  CODE_FOR_vec_cmpv4dfv4di = 6527,
  CODE_FOR_vec_cmpv4sfv4si = 6528,
  CODE_FOR_vec_cmpv2dfv2di = 6529,
  CODE_FOR_vec_cmpuv16sihi = 6530,
  CODE_FOR_vec_cmpuv8siqi = 6531,
  CODE_FOR_vec_cmpuv4siqi = 6532,
  CODE_FOR_vec_cmpuv8diqi = 6533,
  CODE_FOR_vec_cmpuv4diqi = 6534,
  CODE_FOR_vec_cmpuv2diqi = 6535,
  CODE_FOR_vec_cmpuv64qidi = 6536,
  CODE_FOR_vec_cmpuv16qihi = 6537,
  CODE_FOR_vec_cmpuv32qisi = 6538,
  CODE_FOR_vec_cmpuv32hisi = 6539,
  CODE_FOR_vec_cmpuv16hihi = 6540,
  CODE_FOR_vec_cmpuv8hiqi = 6541,
  CODE_FOR_vec_cmpuv32qiv32qi = 6542,
  CODE_FOR_vec_cmpuv16hiv16hi = 6543,
  CODE_FOR_vec_cmpuv8siv8si = 6544,
  CODE_FOR_vec_cmpuv4div4di = 6545,
  CODE_FOR_vec_cmpuv16qiv16qi = 6546,
  CODE_FOR_vec_cmpuv8hiv8hi = 6547,
  CODE_FOR_vec_cmpuv4siv4si = 6548,
  CODE_FOR_vec_cmpuv2div2di = 6549,
  CODE_FOR_vec_cmpeqv2div2di = 6550,
  CODE_FOR_vcondv64qiv16sf = 6551,
  CODE_FOR_vcondv32hiv16sf = 6552,
  CODE_FOR_vcondv16siv16sf = 6553,
  CODE_FOR_vcondv8div16sf = 6554,
  CODE_FOR_vcondv16sfv16sf = 6555,
  CODE_FOR_vcondv8dfv16sf = 6556,
  CODE_FOR_vcondv64qiv8df = 6557,
  CODE_FOR_vcondv32hiv8df = 6558,
  CODE_FOR_vcondv16siv8df = 6559,
  CODE_FOR_vcondv8div8df = 6560,
  CODE_FOR_vcondv16sfv8df = 6561,
  CODE_FOR_vcondv8dfv8df = 6562,
  CODE_FOR_vcondv32qiv8sf = 6563,
  CODE_FOR_vcondv16hiv8sf = 6564,
  CODE_FOR_vcondv8siv8sf = 6565,
  CODE_FOR_vcondv4div8sf = 6566,
  CODE_FOR_vcondv8sfv8sf = 6567,
  CODE_FOR_vcondv4dfv8sf = 6568,
  CODE_FOR_vcondv32qiv4df = 6569,
  CODE_FOR_vcondv16hiv4df = 6570,
  CODE_FOR_vcondv8siv4df = 6571,
  CODE_FOR_vcondv4div4df = 6572,
  CODE_FOR_vcondv8sfv4df = 6573,
  CODE_FOR_vcondv4dfv4df = 6574,
  CODE_FOR_vcondv16qiv4sf = 6575,
  CODE_FOR_vcondv16qiv2df = 6576,
  CODE_FOR_vcondv8hiv4sf = 6577,
  CODE_FOR_vcondv8hiv2df = 6578,
  CODE_FOR_vcondv4siv4sf = 6579,
  CODE_FOR_vcondv4siv2df = 6580,
  CODE_FOR_vcondv2div4sf = 6581,
  CODE_FOR_vcondv2div2df = 6582,
  CODE_FOR_vcondv4sfv4sf = 6583,
  CODE_FOR_vcondv4sfv2df = 6584,
  CODE_FOR_vcondv2dfv4sf = 6585,
  CODE_FOR_vcondv2dfv2df = 6586,
  CODE_FOR_vcond_mask_v16sihi = 6587,
  CODE_FOR_vcond_mask_v8siqi = 6588,
  CODE_FOR_vcond_mask_v4siqi = 6589,
  CODE_FOR_vcond_mask_v8diqi = 6590,
  CODE_FOR_vcond_mask_v4diqi = 6591,
  CODE_FOR_vcond_mask_v2diqi = 6592,
  CODE_FOR_vcond_mask_v16sfhi = 6593,
  CODE_FOR_vcond_mask_v8sfqi = 6594,
  CODE_FOR_vcond_mask_v4sfqi = 6595,
  CODE_FOR_vcond_mask_v8dfqi = 6596,
  CODE_FOR_vcond_mask_v4dfqi = 6597,
  CODE_FOR_vcond_mask_v2dfqi = 6598,
  CODE_FOR_vcond_mask_v64qidi = 6599,
  CODE_FOR_vcond_mask_v16qihi = 6600,
  CODE_FOR_vcond_mask_v32qisi = 6601,
  CODE_FOR_vcond_mask_v32hisi = 6602,
  CODE_FOR_vcond_mask_v16hihi = 6603,
  CODE_FOR_vcond_mask_v8hiqi = 6604,
  CODE_FOR_vcond_mask_v32qiv32qi = 6605,
  CODE_FOR_vcond_mask_v16hiv16hi = 6606,
  CODE_FOR_vcond_mask_v8siv8si = 6607,
  CODE_FOR_vcond_mask_v4div4di = 6608,
  CODE_FOR_vcond_mask_v16qiv16qi = 6609,
  CODE_FOR_vcond_mask_v8hiv8hi = 6610,
  CODE_FOR_vcond_mask_v4siv4si = 6611,
  CODE_FOR_vcond_mask_v2div2di = 6612,
  CODE_FOR_vcond_mask_v8sfv8si = 6613,
  CODE_FOR_vcond_mask_v4dfv4di = 6614,
  CODE_FOR_vcond_mask_v4sfv4si = 6615,
  CODE_FOR_vcond_mask_v2dfv2di = 6616,
  CODE_FOR_andv8sf3 = 6617,
  CODE_FOR_andv8sf3_mask = 6618,
  CODE_FOR_iorv8sf3 = 6619,
  CODE_FOR_iorv8sf3_mask = 6620,
  CODE_FOR_xorv8sf3 = 6621,
  CODE_FOR_xorv8sf3_mask = 6622,
  CODE_FOR_andv4sf3 = 6623,
  CODE_FOR_andv4sf3_mask = 6624,
  CODE_FOR_iorv4sf3 = 6625,
  CODE_FOR_iorv4sf3_mask = 6626,
  CODE_FOR_xorv4sf3 = 6627,
  CODE_FOR_xorv4sf3_mask = 6628,
  CODE_FOR_andv4df3 = 6629,
  CODE_FOR_andv4df3_mask = 6630,
  CODE_FOR_iorv4df3 = 6631,
  CODE_FOR_iorv4df3_mask = 6632,
  CODE_FOR_xorv4df3 = 6633,
  CODE_FOR_xorv4df3_mask = 6634,
  CODE_FOR_andv2df3 = 6635,
  CODE_FOR_andv2df3_mask = 6636,
  CODE_FOR_iorv2df3 = 6637,
  CODE_FOR_iorv2df3_mask = 6638,
  CODE_FOR_xorv2df3 = 6639,
  CODE_FOR_xorv2df3_mask = 6640,
  CODE_FOR_andv16sf3 = 6641,
  CODE_FOR_andv16sf3_mask = 6642,
  CODE_FOR_iorv16sf3 = 6643,
  CODE_FOR_iorv16sf3_mask = 6644,
  CODE_FOR_xorv16sf3 = 6645,
  CODE_FOR_xorv16sf3_mask = 6646,
  CODE_FOR_andv8df3 = 6647,
  CODE_FOR_andv8df3_mask = 6648,
  CODE_FOR_iorv8df3 = 6649,
  CODE_FOR_iorv8df3_mask = 6650,
  CODE_FOR_xorv8df3 = 6651,
  CODE_FOR_xorv8df3_mask = 6652,
  CODE_FOR_copysignv16sf3 = 6653,
  CODE_FOR_copysignv8sf3 = 6654,
  CODE_FOR_copysignv4sf3 = 6655,
  CODE_FOR_copysignv8df3 = 6656,
  CODE_FOR_copysignv4df3 = 6657,
  CODE_FOR_copysignv2df3 = 6658,
  CODE_FOR_xorsignv16sf3 = 6659,
  CODE_FOR_xorsignv8sf3 = 6660,
  CODE_FOR_xorsignv4sf3 = 6661,
  CODE_FOR_xorsignv8df3 = 6662,
  CODE_FOR_xorsignv4df3 = 6663,
  CODE_FOR_xorsignv2df3 = 6664,
  CODE_FOR_andtf3 = 6665,
  CODE_FOR_iortf3 = 6666,
  CODE_FOR_xortf3 = 6667,
  CODE_FOR_fmasf4 = 6668,
  CODE_FOR_fmadf4 = 6669,
  CODE_FOR_fmav4sf4 = 6670,
  CODE_FOR_fmav2df4 = 6671,
  CODE_FOR_fmav8sf4 = 6672,
  CODE_FOR_fmav4df4 = 6673,
  CODE_FOR_fmav16sf4 = 6674,
  CODE_FOR_fmav8df4 = 6675,
  CODE_FOR_fmssf4 = 6676,
  CODE_FOR_fmsdf4 = 6677,
  CODE_FOR_fmsv4sf4 = 6678,
  CODE_FOR_fmsv2df4 = 6679,
  CODE_FOR_fmsv8sf4 = 6680,
  CODE_FOR_fmsv4df4 = 6681,
  CODE_FOR_fmsv16sf4 = 6682,
  CODE_FOR_fmsv8df4 = 6683,
  CODE_FOR_fnmasf4 = 6684,
  CODE_FOR_fnmadf4 = 6685,
  CODE_FOR_fnmav4sf4 = 6686,
  CODE_FOR_fnmav2df4 = 6687,
  CODE_FOR_fnmav8sf4 = 6688,
  CODE_FOR_fnmav4df4 = 6689,
  CODE_FOR_fnmav16sf4 = 6690,
  CODE_FOR_fnmav8df4 = 6691,
  CODE_FOR_fnmssf4 = 6692,
  CODE_FOR_fnmsdf4 = 6693,
  CODE_FOR_fnmsv4sf4 = 6694,
  CODE_FOR_fnmsv2df4 = 6695,
  CODE_FOR_fnmsv8sf4 = 6696,
  CODE_FOR_fnmsv4df4 = 6697,
  CODE_FOR_fnmsv16sf4 = 6698,
  CODE_FOR_fnmsv8df4 = 6699,
  CODE_FOR_fma4i_fmadd_sf = 6700,
  CODE_FOR_fma4i_fmadd_df = 6701,
  CODE_FOR_fma4i_fmadd_v4sf = 6702,
  CODE_FOR_fma4i_fmadd_v2df = 6703,
  CODE_FOR_fma4i_fmadd_v8sf = 6704,
  CODE_FOR_fma4i_fmadd_v4df = 6705,
  CODE_FOR_fma4i_fmadd_v16sf = 6706,
  CODE_FOR_fma4i_fmadd_v8df = 6707,
  CODE_FOR_fma4i_fmsub_sf = 6708,
  CODE_FOR_fma4i_fmsub_df = 6709,
  CODE_FOR_fma4i_fmsub_v4sf = 6710,
  CODE_FOR_fma4i_fmsub_v2df = 6711,
  CODE_FOR_fma4i_fmsub_v8sf = 6712,
  CODE_FOR_fma4i_fmsub_v4df = 6713,
  CODE_FOR_fma4i_fmsub_v16sf = 6714,
  CODE_FOR_fma4i_fmsub_v8df = 6715,
  CODE_FOR_fma4i_fnmadd_sf = 6716,
  CODE_FOR_fma4i_fnmadd_df = 6717,
  CODE_FOR_fma4i_fnmadd_v4sf = 6718,
  CODE_FOR_fma4i_fnmadd_v2df = 6719,
  CODE_FOR_fma4i_fnmadd_v8sf = 6720,
  CODE_FOR_fma4i_fnmadd_v4df = 6721,
  CODE_FOR_fma4i_fnmadd_v16sf = 6722,
  CODE_FOR_fma4i_fnmadd_v8df = 6723,
  CODE_FOR_fma4i_fnmsub_sf = 6724,
  CODE_FOR_fma4i_fnmsub_df = 6725,
  CODE_FOR_fma4i_fnmsub_v4sf = 6726,
  CODE_FOR_fma4i_fnmsub_v2df = 6727,
  CODE_FOR_fma4i_fnmsub_v8sf = 6728,
  CODE_FOR_fma4i_fnmsub_v4df = 6729,
  CODE_FOR_fma4i_fnmsub_v16sf = 6730,
  CODE_FOR_fma4i_fnmsub_v8df = 6731,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 6732,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 6733,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 6734,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 6735,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 6736,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 6737,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 6738,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 6739,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 6740,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 6741,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 6742,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 6743,
  CODE_FOR_avx512f_fmsub_v16sf_maskz = 6744,
  CODE_FOR_avx512f_fmsub_v16sf_maskz_round = 6745,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz = 6746,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz_round = 6747,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz = 6748,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz_round = 6749,
  CODE_FOR_avx512f_fmsub_v8df_maskz = 6750,
  CODE_FOR_avx512f_fmsub_v8df_maskz_round = 6751,
  CODE_FOR_avx512vl_fmsub_v4df_maskz = 6752,
  CODE_FOR_avx512vl_fmsub_v4df_maskz_round = 6753,
  CODE_FOR_avx512vl_fmsub_v2df_maskz = 6754,
  CODE_FOR_avx512vl_fmsub_v2df_maskz_round = 6755,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz = 6756,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz_round = 6757,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz = 6758,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz_round = 6759,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz = 6760,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz_round = 6761,
  CODE_FOR_avx512f_fnmadd_v8df_maskz = 6762,
  CODE_FOR_avx512f_fnmadd_v8df_maskz_round = 6763,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz = 6764,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz_round = 6765,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz = 6766,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz_round = 6767,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz = 6768,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz_round = 6769,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz = 6770,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz_round = 6771,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz = 6772,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz_round = 6773,
  CODE_FOR_avx512f_fnmsub_v8df_maskz = 6774,
  CODE_FOR_avx512f_fnmsub_v8df_maskz_round = 6775,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz = 6776,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz_round = 6777,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz = 6778,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz_round = 6779,
  CODE_FOR_fmaddsub_v16sf = 6780,
  CODE_FOR_fmaddsub_v8sf = 6781,
  CODE_FOR_fmaddsub_v4sf = 6782,
  CODE_FOR_fmaddsub_v8df = 6783,
  CODE_FOR_fmaddsub_v4df = 6784,
  CODE_FOR_fmaddsub_v2df = 6785,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 6786,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 6787,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 6788,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 6789,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 6790,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 6791,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 6792,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 6793,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 6794,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 6795,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 6796,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 6797,
  CODE_FOR_fmai_vmfmadd_v4sf = 6798,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 6799,
  CODE_FOR_fmai_vmfmadd_v2df = 6800,
  CODE_FOR_fmai_vmfmadd_v2df_round = 6801,
  CODE_FOR_fmai_vmfmsub_v4sf = 6802,
  CODE_FOR_fmai_vmfmsub_v4sf_round = 6803,
  CODE_FOR_fmai_vmfmsub_v2df = 6804,
  CODE_FOR_fmai_vmfmsub_v2df_round = 6805,
  CODE_FOR_fmai_vmfnmadd_v4sf = 6806,
  CODE_FOR_fmai_vmfnmadd_v4sf_round = 6807,
  CODE_FOR_fmai_vmfnmadd_v2df = 6808,
  CODE_FOR_fmai_vmfnmadd_v2df_round = 6809,
  CODE_FOR_fmai_vmfnmsub_v4sf = 6810,
  CODE_FOR_fmai_vmfnmsub_v4sf_round = 6811,
  CODE_FOR_fmai_vmfnmsub_v2df = 6812,
  CODE_FOR_fmai_vmfnmsub_v2df_round = 6813,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz = 6814,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_round = 6815,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz = 6816,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_round = 6817,
  CODE_FOR_fma4i_vmfmadd_v4sf = 6818,
  CODE_FOR_fma4i_vmfmadd_v2df = 6819,
  CODE_FOR_floatunsv16siv16sf2 = 6820,
  CODE_FOR_floatunsv8siv8sf2 = 6821,
  CODE_FOR_floatunsv4siv4sf2 = 6822,
  CODE_FOR_fixuns_truncv16sfv16si2 = 6823,
  CODE_FOR_fixuns_truncv8sfv8si2 = 6824,
  CODE_FOR_fixuns_truncv4sfv4si2 = 6825,
  CODE_FOR_floatv2div2sf2 = 6826,
  CODE_FOR_floatunsv2div2sf2 = 6827,
  CODE_FOR_vec_packs_float_v8di = 6828,
  CODE_FOR_vec_packu_float_v8di = 6829,
  CODE_FOR_vec_packs_float_v4di = 6830,
  CODE_FOR_vec_packu_float_v4di = 6831,
  CODE_FOR_vec_packs_float_v2di = 6832,
  CODE_FOR_vec_packu_float_v2di = 6833,
  CODE_FOR_floatv2div2sf2_mask = 6834,
  CODE_FOR_floatunsv2div2sf2_mask = 6835,
  CODE_FOR_avx_cvtpd2dq256_2 = 6836,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v16sf = 6837,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v16sf = 6838,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v8sf = 6839,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v8sf = 6840,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v4sf = 6841,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v4sf = 6842,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v16sf = 6843,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v16sf = 6844,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v8sf = 6845,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v8sf = 6846,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v4sf = 6847,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v4sf = 6848,
  CODE_FOR_avx_cvttpd2dq256_2 = 6849,
  CODE_FOR_sse2_cvtpd2ps = 6850,
  CODE_FOR_sse2_cvtpd2ps_mask = 6851,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 6852,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 6853,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 6854,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 6855,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 6856,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 6857,
  CODE_FOR_avx512f_cvtmask2dv16si = 6858,
  CODE_FOR_avx512vl_cvtmask2dv8si = 6859,
  CODE_FOR_avx512vl_cvtmask2dv4si = 6860,
  CODE_FOR_avx512f_cvtmask2qv8di = 6861,
  CODE_FOR_avx512vl_cvtmask2qv4di = 6862,
  CODE_FOR_avx512vl_cvtmask2qv2di = 6863,
  CODE_FOR_vec_unpacks_hi_v4sf = 6864,
  CODE_FOR_vec_unpacks_hi_v8sf = 6865,
  CODE_FOR_vec_unpacks_hi_v16sf = 6866,
  CODE_FOR_vec_unpacks_lo_v4sf = 6867,
  CODE_FOR_vec_unpacks_lo_v8sf = 6868,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 6869,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 6870,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 6871,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 6872,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 6873,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 6874,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 6875,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 6876,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 6877,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 6878,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 6879,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 6880,
  CODE_FOR_vec_unpacks_float_hi_v4si = 6881,
  CODE_FOR_vec_unpacks_float_lo_v4si = 6882,
  CODE_FOR_vec_unpacks_float_hi_v8si = 6883,
  CODE_FOR_vec_unpacks_float_lo_v8si = 6884,
  CODE_FOR_vec_unpacks_float_hi_v16si = 6885,
  CODE_FOR_vec_unpacks_float_lo_v16si = 6886,
  CODE_FOR_vec_unpacku_float_hi_v4si = 6887,
  CODE_FOR_vec_unpacku_float_lo_v4si = 6888,
  CODE_FOR_vec_unpacku_float_hi_v8si = 6889,
  CODE_FOR_vec_unpacku_float_hi_v16si = 6890,
  CODE_FOR_vec_unpacku_float_lo_v8si = 6891,
  CODE_FOR_vec_unpacku_float_lo_v16si = 6892,
  CODE_FOR_vec_pack_trunc_v8df = 6893,
  CODE_FOR_vec_pack_trunc_v4df = 6894,
  CODE_FOR_vec_pack_trunc_v2df = 6895,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 6896,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 6897,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 6898,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 6899,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 6900,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 6901,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 6902,
  CODE_FOR_vec_pack_sfix_v4df = 6903,
  CODE_FOR_vec_pack_sfix_v2df = 6904,
  CODE_FOR_sse_movhlps_exp = 6905,
  CODE_FOR_sse_movlhps_exp = 6906,
  CODE_FOR_vec_interleave_highv8sf = 6907,
  CODE_FOR_vec_interleave_lowv8sf = 6908,
  CODE_FOR_avx_shufps256 = 6909,
  CODE_FOR_avx_shufps256_mask = 6910,
  CODE_FOR_sse_shufps = 6911,
  CODE_FOR_sse_shufps_mask = 6912,
  CODE_FOR_sse_loadhps_exp = 6913,
  CODE_FOR_sse_loadlps_exp = 6914,
  CODE_FOR_vec_setv64qi = 6915,
  CODE_FOR_vec_setv32qi = 6916,
  CODE_FOR_vec_setv16qi = 6917,
  CODE_FOR_vec_setv32hi = 6918,
  CODE_FOR_vec_setv16hi = 6919,
  CODE_FOR_vec_setv8hi = 6920,
  CODE_FOR_vec_setv16si = 6921,
  CODE_FOR_vec_setv8si = 6922,
  CODE_FOR_vec_setv4si = 6923,
  CODE_FOR_vec_setv8di = 6924,
  CODE_FOR_vec_setv4di = 6925,
  CODE_FOR_vec_setv2di = 6926,
  CODE_FOR_vec_setv16sf = 6927,
  CODE_FOR_vec_setv8sf = 6928,
  CODE_FOR_vec_setv4sf = 6929,
  CODE_FOR_vec_setv8df = 6930,
  CODE_FOR_vec_setv4df = 6931,
  CODE_FOR_vec_setv2df = 6932,
  CODE_FOR_avx512dq_vextractf64x2_mask = 6933,
  CODE_FOR_avx512dq_vextracti64x2_mask = 6934,
  CODE_FOR_avx512f_vextractf32x4_mask = 6935,
  CODE_FOR_avx512f_vextracti32x4_mask = 6936,
  CODE_FOR_avx512dq_vextractf32x8_mask = 6937,
  CODE_FOR_avx512dq_vextracti32x8_mask = 6938,
  CODE_FOR_avx512f_vextractf64x4_mask = 6939,
  CODE_FOR_avx512f_vextracti64x4_mask = 6940,
  CODE_FOR_avx512vl_vextractf128v8si = 6941,
  CODE_FOR_avx512vl_vextractf128v8sf = 6942,
  CODE_FOR_avx512vl_vextractf128v4di = 6943,
  CODE_FOR_avx512vl_vextractf128v4df = 6944,
  CODE_FOR_avx_vextractf128v32qi = 6945,
  CODE_FOR_avx_vextractf128v16hi = 6946,
  CODE_FOR_avx_vextractf128v8si = 6947,
  CODE_FOR_avx_vextractf128v4di = 6948,
  CODE_FOR_avx_vextractf128v8sf = 6949,
  CODE_FOR_avx_vextractf128v4df = 6950,
  CODE_FOR_vec_extractv64qiqi = 6951,
  CODE_FOR_vec_extractv32qiqi = 6952,
  CODE_FOR_vec_extractv16qiqi = 6953,
  CODE_FOR_vec_extractv32hihi = 6954,
  CODE_FOR_vec_extractv16hihi = 6955,
  CODE_FOR_vec_extractv8hihi = 6956,
  CODE_FOR_vec_extractv16sisi = 6957,
  CODE_FOR_vec_extractv8sisi = 6958,
  CODE_FOR_vec_extractv4sisi = 6959,
  CODE_FOR_vec_extractv8didi = 6960,
  CODE_FOR_vec_extractv4didi = 6961,
  CODE_FOR_vec_extractv2didi = 6962,
  CODE_FOR_vec_extractv16sfsf = 6963,
  CODE_FOR_vec_extractv8sfsf = 6964,
  CODE_FOR_vec_extractv4sfsf = 6965,
  CODE_FOR_vec_extractv8dfdf = 6966,
  CODE_FOR_vec_extractv4dfdf = 6967,
  CODE_FOR_vec_extractv2dfdf = 6968,
  CODE_FOR_vec_extractv4titi = 6969,
  CODE_FOR_vec_extractv2titi = 6970,
  CODE_FOR_vec_extractv32qiv16qi = 6971,
  CODE_FOR_vec_extractv16hiv8hi = 6972,
  CODE_FOR_vec_extractv8siv4si = 6973,
  CODE_FOR_vec_extractv4div2di = 6974,
  CODE_FOR_vec_extractv8sfv4sf = 6975,
  CODE_FOR_vec_extractv4dfv2df = 6976,
  CODE_FOR_vec_extractv64qiv32qi = 6977,
  CODE_FOR_vec_extractv32hiv16hi = 6978,
  CODE_FOR_vec_extractv16siv8si = 6979,
  CODE_FOR_vec_extractv8div4di = 6980,
  CODE_FOR_vec_extractv16sfv8sf = 6981,
  CODE_FOR_vec_extractv8dfv4df = 6982,
  CODE_FOR_vec_interleave_highv4df = 6983,
  CODE_FOR_vec_interleave_highv2df = 6984,
  CODE_FOR_avx512f_movddup512 = 6985,
  CODE_FOR_avx512f_movddup512_mask = 6986,
  CODE_FOR_avx512f_unpcklpd512 = 6987,
  CODE_FOR_avx512f_unpcklpd512_mask = 6988,
  CODE_FOR_avx_movddup256 = 6989,
  CODE_FOR_avx_movddup256_mask = 6990,
  CODE_FOR_avx_unpcklpd256 = 6991,
  CODE_FOR_avx_unpcklpd256_mask = 6992,
  CODE_FOR_vec_interleave_lowv4df = 6993,
  CODE_FOR_vec_interleave_lowv2df = 6994,
  CODE_FOR_avx512f_vternlogv16si_maskz = 6995,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 6996,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 6997,
  CODE_FOR_avx512f_vternlogv8di_maskz = 6998,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 6999,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 7000,
  CODE_FOR_avx512f_shufps512_mask = 7001,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 7002,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 7003,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 7004,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 7005,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 7006,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 7007,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 7008,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 7009,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 7010,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 7011,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 7012,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 7013,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 7014,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 7015,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 7016,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 7017,
  CODE_FOR_avx512f_shufpd512_mask = 7018,
  CODE_FOR_avx_shufpd256 = 7019,
  CODE_FOR_avx_shufpd256_mask = 7020,
  CODE_FOR_sse2_shufpd = 7021,
  CODE_FOR_sse2_shufpd_mask = 7022,
  CODE_FOR_sse2_loadhpd_exp = 7023,
  CODE_FOR_sse2_loadlpd_exp = 7024,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 7025,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 7026,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 7027,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 7028,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 7029,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 7030,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 7031,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 7032,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 7033,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 7034,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 7035,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 7036,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 7037,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 7038,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 7039,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 7040,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 7041,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 7042,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 7043,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 7044,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 7045,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 7046,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 7047,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 7048,
  CODE_FOR_negv64qi2 = 7049,
  CODE_FOR_negv32qi2 = 7050,
  CODE_FOR_negv16qi2 = 7051,
  CODE_FOR_negv32hi2 = 7052,
  CODE_FOR_negv16hi2 = 7053,
  CODE_FOR_negv8hi2 = 7054,
  CODE_FOR_negv16si2 = 7055,
  CODE_FOR_negv8si2 = 7056,
  CODE_FOR_negv4si2 = 7057,
  CODE_FOR_negv8di2 = 7058,
  CODE_FOR_negv4di2 = 7059,
  CODE_FOR_negv2di2 = 7060,
  CODE_FOR_addv64qi3 = 7061,
  CODE_FOR_subv64qi3 = 7062,
  CODE_FOR_addv32qi3 = 7063,
  CODE_FOR_subv32qi3 = 7064,
  CODE_FOR_addv16qi3 = 7065,
  CODE_FOR_subv16qi3 = 7066,
  CODE_FOR_addv32hi3 = 7067,
  CODE_FOR_subv32hi3 = 7068,
  CODE_FOR_addv16hi3 = 7069,
  CODE_FOR_subv16hi3 = 7070,
  CODE_FOR_addv8hi3 = 7071,
  CODE_FOR_subv8hi3 = 7072,
  CODE_FOR_addv16si3 = 7073,
  CODE_FOR_subv16si3 = 7074,
  CODE_FOR_addv8si3 = 7075,
  CODE_FOR_subv8si3 = 7076,
  CODE_FOR_addv4si3 = 7077,
  CODE_FOR_subv4si3 = 7078,
  CODE_FOR_addv8di3 = 7079,
  CODE_FOR_subv8di3 = 7080,
  CODE_FOR_addv4di3 = 7081,
  CODE_FOR_subv4di3 = 7082,
  CODE_FOR_addv2di3 = 7083,
  CODE_FOR_subv2di3 = 7084,
  CODE_FOR_addv16si3_mask = 7085,
  CODE_FOR_subv16si3_mask = 7086,
  CODE_FOR_addv8si3_mask = 7087,
  CODE_FOR_subv8si3_mask = 7088,
  CODE_FOR_addv4si3_mask = 7089,
  CODE_FOR_subv4si3_mask = 7090,
  CODE_FOR_addv8di3_mask = 7091,
  CODE_FOR_subv8di3_mask = 7092,
  CODE_FOR_addv4di3_mask = 7093,
  CODE_FOR_subv4di3_mask = 7094,
  CODE_FOR_addv2di3_mask = 7095,
  CODE_FOR_subv2di3_mask = 7096,
  CODE_FOR_addv64qi3_mask = 7097,
  CODE_FOR_subv64qi3_mask = 7098,
  CODE_FOR_addv16qi3_mask = 7099,
  CODE_FOR_subv16qi3_mask = 7100,
  CODE_FOR_addv32qi3_mask = 7101,
  CODE_FOR_subv32qi3_mask = 7102,
  CODE_FOR_addv32hi3_mask = 7103,
  CODE_FOR_subv32hi3_mask = 7104,
  CODE_FOR_addv16hi3_mask = 7105,
  CODE_FOR_subv16hi3_mask = 7106,
  CODE_FOR_addv8hi3_mask = 7107,
  CODE_FOR_subv8hi3_mask = 7108,
  CODE_FOR_avx512bw_ssaddv64qi3 = 7109,
  CODE_FOR_avx512bw_ssaddv64qi3_mask = 7110,
  CODE_FOR_avx512bw_usaddv64qi3 = 7111,
  CODE_FOR_avx512bw_usaddv64qi3_mask = 7112,
  CODE_FOR_avx512bw_sssubv64qi3 = 7113,
  CODE_FOR_avx512bw_sssubv64qi3_mask = 7114,
  CODE_FOR_avx512bw_ussubv64qi3 = 7115,
  CODE_FOR_avx512bw_ussubv64qi3_mask = 7116,
  CODE_FOR_avx2_ssaddv32qi3 = 7117,
  CODE_FOR_avx2_ssaddv32qi3_mask = 7118,
  CODE_FOR_avx2_usaddv32qi3 = 7119,
  CODE_FOR_avx2_usaddv32qi3_mask = 7120,
  CODE_FOR_avx2_sssubv32qi3 = 7121,
  CODE_FOR_avx2_sssubv32qi3_mask = 7122,
  CODE_FOR_avx2_ussubv32qi3 = 7123,
  CODE_FOR_avx2_ussubv32qi3_mask = 7124,
  CODE_FOR_sse2_ssaddv16qi3 = 7125,
  CODE_FOR_sse2_ssaddv16qi3_mask = 7126,
  CODE_FOR_sse2_usaddv16qi3 = 7127,
  CODE_FOR_sse2_usaddv16qi3_mask = 7128,
  CODE_FOR_sse2_sssubv16qi3 = 7129,
  CODE_FOR_sse2_sssubv16qi3_mask = 7130,
  CODE_FOR_sse2_ussubv16qi3 = 7131,
  CODE_FOR_sse2_ussubv16qi3_mask = 7132,
  CODE_FOR_avx512bw_ssaddv32hi3 = 7133,
  CODE_FOR_avx512bw_ssaddv32hi3_mask = 7134,
  CODE_FOR_avx512bw_usaddv32hi3 = 7135,
  CODE_FOR_avx512bw_usaddv32hi3_mask = 7136,
  CODE_FOR_avx512bw_sssubv32hi3 = 7137,
  CODE_FOR_avx512bw_sssubv32hi3_mask = 7138,
  CODE_FOR_avx512bw_ussubv32hi3 = 7139,
  CODE_FOR_avx512bw_ussubv32hi3_mask = 7140,
  CODE_FOR_avx2_ssaddv16hi3 = 7141,
  CODE_FOR_avx2_ssaddv16hi3_mask = 7142,
  CODE_FOR_avx2_usaddv16hi3 = 7143,
  CODE_FOR_avx2_usaddv16hi3_mask = 7144,
  CODE_FOR_avx2_sssubv16hi3 = 7145,
  CODE_FOR_avx2_sssubv16hi3_mask = 7146,
  CODE_FOR_avx2_ussubv16hi3 = 7147,
  CODE_FOR_avx2_ussubv16hi3_mask = 7148,
  CODE_FOR_sse2_ssaddv8hi3 = 7149,
  CODE_FOR_sse2_ssaddv8hi3_mask = 7150,
  CODE_FOR_sse2_usaddv8hi3 = 7151,
  CODE_FOR_sse2_usaddv8hi3_mask = 7152,
  CODE_FOR_sse2_sssubv8hi3 = 7153,
  CODE_FOR_sse2_sssubv8hi3_mask = 7154,
  CODE_FOR_sse2_ussubv8hi3 = 7155,
  CODE_FOR_sse2_ussubv8hi3_mask = 7156,
  CODE_FOR_mulv64qi3 = 7157,
  CODE_FOR_mulv64qi3_mask = 7158,
  CODE_FOR_mulv32qi3 = 7159,
  CODE_FOR_mulv32qi3_mask = 7160,
  CODE_FOR_mulv16qi3 = 7161,
  CODE_FOR_mulv16qi3_mask = 7162,
  CODE_FOR_mulv32hi3 = 7163,
  CODE_FOR_mulv32hi3_mask = 7164,
  CODE_FOR_mulv16hi3 = 7165,
  CODE_FOR_mulv16hi3_mask = 7166,
  CODE_FOR_mulv8hi3 = 7167,
  CODE_FOR_mulv8hi3_mask = 7168,
  CODE_FOR_smulv32hi3_highpart = 7169,
  CODE_FOR_smulv32hi3_highpart_mask = 7170,
  CODE_FOR_umulv32hi3_highpart = 7171,
  CODE_FOR_umulv32hi3_highpart_mask = 7172,
  CODE_FOR_smulv16hi3_highpart = 7173,
  CODE_FOR_smulv16hi3_highpart_mask = 7174,
  CODE_FOR_umulv16hi3_highpart = 7175,
  CODE_FOR_umulv16hi3_highpart_mask = 7176,
  CODE_FOR_smulv8hi3_highpart = 7177,
  CODE_FOR_smulv8hi3_highpart_mask = 7178,
  CODE_FOR_umulv8hi3_highpart = 7179,
  CODE_FOR_umulv8hi3_highpart_mask = 7180,
  CODE_FOR_vec_widen_umult_even_v16si = 7181,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 7182,
  CODE_FOR_vec_widen_umult_even_v8si = 7183,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 7184,
  CODE_FOR_vec_widen_umult_even_v4si = 7185,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 7186,
  CODE_FOR_vec_widen_smult_even_v16si = 7187,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 7188,
  CODE_FOR_vec_widen_smult_even_v8si = 7189,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 7190,
  CODE_FOR_sse4_1_mulv2siv2di3 = 7191,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 7192,
  CODE_FOR_avx2_pmaddwd = 7193,
  CODE_FOR_sse2_pmaddwd = 7194,
  CODE_FOR_mulv16si3 = 7195,
  CODE_FOR_mulv16si3_mask = 7196,
  CODE_FOR_mulv8si3 = 7197,
  CODE_FOR_mulv8si3_mask = 7198,
  CODE_FOR_mulv4si3 = 7199,
  CODE_FOR_mulv4si3_mask = 7200,
  CODE_FOR_mulv8di3 = 7201,
  CODE_FOR_mulv4di3 = 7202,
  CODE_FOR_mulv2di3 = 7203,
  CODE_FOR_vec_widen_smult_hi_v32qi = 7204,
  CODE_FOR_vec_widen_umult_hi_v32qi = 7205,
  CODE_FOR_vec_widen_smult_hi_v16qi = 7206,
  CODE_FOR_vec_widen_umult_hi_v16qi = 7207,
  CODE_FOR_vec_widen_smult_hi_v16hi = 7208,
  CODE_FOR_vec_widen_umult_hi_v16hi = 7209,
  CODE_FOR_vec_widen_smult_hi_v8hi = 7210,
  CODE_FOR_vec_widen_umult_hi_v8hi = 7211,
  CODE_FOR_vec_widen_smult_hi_v8si = 7212,
  CODE_FOR_vec_widen_umult_hi_v8si = 7213,
  CODE_FOR_vec_widen_smult_hi_v4si = 7214,
  CODE_FOR_vec_widen_umult_hi_v4si = 7215,
  CODE_FOR_vec_widen_smult_lo_v32qi = 7216,
  CODE_FOR_vec_widen_umult_lo_v32qi = 7217,
  CODE_FOR_vec_widen_smult_lo_v16qi = 7218,
  CODE_FOR_vec_widen_umult_lo_v16qi = 7219,
  CODE_FOR_vec_widen_smult_lo_v16hi = 7220,
  CODE_FOR_vec_widen_umult_lo_v16hi = 7221,
  CODE_FOR_vec_widen_smult_lo_v8hi = 7222,
  CODE_FOR_vec_widen_umult_lo_v8hi = 7223,
  CODE_FOR_vec_widen_smult_lo_v8si = 7224,
  CODE_FOR_vec_widen_umult_lo_v8si = 7225,
  CODE_FOR_vec_widen_smult_lo_v4si = 7226,
  CODE_FOR_vec_widen_umult_lo_v4si = 7227,
  CODE_FOR_vec_widen_smult_even_v4si = 7228,
  CODE_FOR_vec_widen_smult_odd_v16si = 7229,
  CODE_FOR_vec_widen_umult_odd_v16si = 7230,
  CODE_FOR_vec_widen_smult_odd_v8si = 7231,
  CODE_FOR_vec_widen_umult_odd_v8si = 7232,
  CODE_FOR_vec_widen_smult_odd_v4si = 7233,
  CODE_FOR_vec_widen_umult_odd_v4si = 7234,
  CODE_FOR_sdot_prodv32hi = 7235,
  CODE_FOR_sdot_prodv16hi = 7236,
  CODE_FOR_sdot_prodv8hi = 7237,
  CODE_FOR_sdot_prodv4si = 7238,
  CODE_FOR_uavgv64qi3_ceil = 7239,
  CODE_FOR_uavgv32qi3_ceil = 7240,
  CODE_FOR_uavgv16qi3_ceil = 7241,
  CODE_FOR_uavgv32hi3_ceil = 7242,
  CODE_FOR_uavgv16hi3_ceil = 7243,
  CODE_FOR_uavgv8hi3_ceil = 7244,
  CODE_FOR_usadv16qi = 7245,
  CODE_FOR_usadv32qi = 7246,
  CODE_FOR_usadv64qi = 7247,
  CODE_FOR_vec_shr_v16qi = 7248,
  CODE_FOR_vec_shr_v8hi = 7249,
  CODE_FOR_vec_shr_v4si = 7250,
  CODE_FOR_vec_shr_v2di = 7251,
  CODE_FOR_smaxv32qi3 = 7252,
  CODE_FOR_sminv32qi3 = 7253,
  CODE_FOR_umaxv32qi3 = 7254,
  CODE_FOR_uminv32qi3 = 7255,
  CODE_FOR_smaxv16hi3 = 7256,
  CODE_FOR_sminv16hi3 = 7257,
  CODE_FOR_umaxv16hi3 = 7258,
  CODE_FOR_uminv16hi3 = 7259,
  CODE_FOR_smaxv8si3 = 7260,
  CODE_FOR_sminv8si3 = 7261,
  CODE_FOR_umaxv8si3 = 7262,
  CODE_FOR_uminv8si3 = 7263,
  CODE_FOR_smaxv64qi3 = 7264,
  CODE_FOR_sminv64qi3 = 7265,
  CODE_FOR_umaxv64qi3 = 7266,
  CODE_FOR_uminv64qi3 = 7267,
  CODE_FOR_smaxv32hi3 = 7268,
  CODE_FOR_sminv32hi3 = 7269,
  CODE_FOR_umaxv32hi3 = 7270,
  CODE_FOR_uminv32hi3 = 7271,
  CODE_FOR_smaxv16si3 = 7272,
  CODE_FOR_sminv16si3 = 7273,
  CODE_FOR_umaxv16si3 = 7274,
  CODE_FOR_uminv16si3 = 7275,
  CODE_FOR_smaxv16si3_mask = 7276,
  CODE_FOR_sminv16si3_mask = 7277,
  CODE_FOR_umaxv16si3_mask = 7278,
  CODE_FOR_uminv16si3_mask = 7279,
  CODE_FOR_smaxv8si3_mask = 7280,
  CODE_FOR_sminv8si3_mask = 7281,
  CODE_FOR_umaxv8si3_mask = 7282,
  CODE_FOR_uminv8si3_mask = 7283,
  CODE_FOR_smaxv4si3_mask = 7284,
  CODE_FOR_sminv4si3_mask = 7285,
  CODE_FOR_umaxv4si3_mask = 7286,
  CODE_FOR_uminv4si3_mask = 7287,
  CODE_FOR_smaxv8di3_mask = 7288,
  CODE_FOR_sminv8di3_mask = 7289,
  CODE_FOR_umaxv8di3_mask = 7290,
  CODE_FOR_uminv8di3_mask = 7291,
  CODE_FOR_smaxv4di3_mask = 7292,
  CODE_FOR_sminv4di3_mask = 7293,
  CODE_FOR_umaxv4di3_mask = 7294,
  CODE_FOR_uminv4di3_mask = 7295,
  CODE_FOR_smaxv2di3_mask = 7296,
  CODE_FOR_sminv2di3_mask = 7297,
  CODE_FOR_umaxv2di3_mask = 7298,
  CODE_FOR_uminv2di3_mask = 7299,
  CODE_FOR_smaxv8di3 = 7300,
  CODE_FOR_sminv8di3 = 7301,
  CODE_FOR_umaxv8di3 = 7302,
  CODE_FOR_uminv8di3 = 7303,
  CODE_FOR_smaxv4di3 = 7304,
  CODE_FOR_sminv4di3 = 7305,
  CODE_FOR_umaxv4di3 = 7306,
  CODE_FOR_uminv4di3 = 7307,
  CODE_FOR_smaxv2di3 = 7308,
  CODE_FOR_sminv2di3 = 7309,
  CODE_FOR_umaxv2di3 = 7310,
  CODE_FOR_uminv2di3 = 7311,
  CODE_FOR_smaxv16qi3 = 7312,
  CODE_FOR_sminv16qi3 = 7313,
  CODE_FOR_smaxv8hi3 = 7314,
  CODE_FOR_sminv8hi3 = 7315,
  CODE_FOR_smaxv4si3 = 7316,
  CODE_FOR_sminv4si3 = 7317,
  CODE_FOR_umaxv16qi3 = 7318,
  CODE_FOR_uminv16qi3 = 7319,
  CODE_FOR_umaxv8hi3 = 7320,
  CODE_FOR_uminv8hi3 = 7321,
  CODE_FOR_umaxv4si3 = 7322,
  CODE_FOR_uminv4si3 = 7323,
  CODE_FOR_avx2_eqv32qi3 = 7324,
  CODE_FOR_avx2_eqv16hi3 = 7325,
  CODE_FOR_avx2_eqv8si3 = 7326,
  CODE_FOR_avx2_eqv4di3 = 7327,
  CODE_FOR_avx512bw_eqv64qi3 = 7328,
  CODE_FOR_avx512bw_eqv64qi3_mask = 7329,
  CODE_FOR_avx512vl_eqv16qi3 = 7330,
  CODE_FOR_avx512vl_eqv16qi3_mask = 7331,
  CODE_FOR_avx512vl_eqv32qi3 = 7332,
  CODE_FOR_avx512vl_eqv32qi3_mask = 7333,
  CODE_FOR_avx512bw_eqv32hi3 = 7334,
  CODE_FOR_avx512bw_eqv32hi3_mask = 7335,
  CODE_FOR_avx512vl_eqv16hi3 = 7336,
  CODE_FOR_avx512vl_eqv16hi3_mask = 7337,
  CODE_FOR_avx512vl_eqv8hi3 = 7338,
  CODE_FOR_avx512vl_eqv8hi3_mask = 7339,
  CODE_FOR_avx512f_eqv16si3 = 7340,
  CODE_FOR_avx512f_eqv16si3_mask = 7341,
  CODE_FOR_avx512vl_eqv8si3 = 7342,
  CODE_FOR_avx512vl_eqv8si3_mask = 7343,
  CODE_FOR_avx512vl_eqv4si3 = 7344,
  CODE_FOR_avx512vl_eqv4si3_mask = 7345,
  CODE_FOR_avx512f_eqv8di3 = 7346,
  CODE_FOR_avx512f_eqv8di3_mask = 7347,
  CODE_FOR_avx512vl_eqv4di3 = 7348,
  CODE_FOR_avx512vl_eqv4di3_mask = 7349,
  CODE_FOR_avx512vl_eqv2di3 = 7350,
  CODE_FOR_avx512vl_eqv2di3_mask = 7351,
  CODE_FOR_sse2_eqv16qi3 = 7352,
  CODE_FOR_sse2_eqv8hi3 = 7353,
  CODE_FOR_sse2_eqv4si3 = 7354,
  CODE_FOR_sse4_1_eqv2di3 = 7355,
  CODE_FOR_vcondv64qiv16si = 7356,
  CODE_FOR_vcondv32hiv16si = 7357,
  CODE_FOR_vcondv16siv16si = 7358,
  CODE_FOR_vcondv8div16si = 7359,
  CODE_FOR_vcondv16sfv16si = 7360,
  CODE_FOR_vcondv8dfv16si = 7361,
  CODE_FOR_vcondv64qiv8di = 7362,
  CODE_FOR_vcondv32hiv8di = 7363,
  CODE_FOR_vcondv16siv8di = 7364,
  CODE_FOR_vcondv8div8di = 7365,
  CODE_FOR_vcondv16sfv8di = 7366,
  CODE_FOR_vcondv8dfv8di = 7367,
  CODE_FOR_vcondv64qiv32hi = 7368,
  CODE_FOR_vcondv32hiv32hi = 7369,
  CODE_FOR_vcondv16siv32hi = 7370,
  CODE_FOR_vcondv8div32hi = 7371,
  CODE_FOR_vcondv16sfv32hi = 7372,
  CODE_FOR_vcondv8dfv32hi = 7373,
  CODE_FOR_vcondv64qiv64qi = 7374,
  CODE_FOR_vcondv32hiv64qi = 7375,
  CODE_FOR_vcondv16siv64qi = 7376,
  CODE_FOR_vcondv8div64qi = 7377,
  CODE_FOR_vcondv16sfv64qi = 7378,
  CODE_FOR_vcondv8dfv64qi = 7379,
  CODE_FOR_vcondv32qiv32qi = 7380,
  CODE_FOR_vcondv16hiv32qi = 7381,
  CODE_FOR_vcondv8siv32qi = 7382,
  CODE_FOR_vcondv4div32qi = 7383,
  CODE_FOR_vcondv8sfv32qi = 7384,
  CODE_FOR_vcondv4dfv32qi = 7385,
  CODE_FOR_vcondv32qiv16hi = 7386,
  CODE_FOR_vcondv16hiv16hi = 7387,
  CODE_FOR_vcondv8siv16hi = 7388,
  CODE_FOR_vcondv4div16hi = 7389,
  CODE_FOR_vcondv8sfv16hi = 7390,
  CODE_FOR_vcondv4dfv16hi = 7391,
  CODE_FOR_vcondv32qiv8si = 7392,
  CODE_FOR_vcondv16hiv8si = 7393,
  CODE_FOR_vcondv8siv8si = 7394,
  CODE_FOR_vcondv4div8si = 7395,
  CODE_FOR_vcondv8sfv8si = 7396,
  CODE_FOR_vcondv4dfv8si = 7397,
  CODE_FOR_vcondv32qiv4di = 7398,
  CODE_FOR_vcondv16hiv4di = 7399,
  CODE_FOR_vcondv8siv4di = 7400,
  CODE_FOR_vcondv4div4di = 7401,
  CODE_FOR_vcondv8sfv4di = 7402,
  CODE_FOR_vcondv4dfv4di = 7403,
  CODE_FOR_vcondv16qiv16qi = 7404,
  CODE_FOR_vcondv8hiv16qi = 7405,
  CODE_FOR_vcondv4siv16qi = 7406,
  CODE_FOR_vcondv2div16qi = 7407,
  CODE_FOR_vcondv4sfv16qi = 7408,
  CODE_FOR_vcondv2dfv16qi = 7409,
  CODE_FOR_vcondv16qiv8hi = 7410,
  CODE_FOR_vcondv8hiv8hi = 7411,
  CODE_FOR_vcondv4siv8hi = 7412,
  CODE_FOR_vcondv2div8hi = 7413,
  CODE_FOR_vcondv4sfv8hi = 7414,
  CODE_FOR_vcondv2dfv8hi = 7415,
  CODE_FOR_vcondv16qiv4si = 7416,
  CODE_FOR_vcondv8hiv4si = 7417,
  CODE_FOR_vcondv4siv4si = 7418,
  CODE_FOR_vcondv2div4si = 7419,
  CODE_FOR_vcondv4sfv4si = 7420,
  CODE_FOR_vcondv2dfv4si = 7421,
  CODE_FOR_vcondv2div2di = 7422,
  CODE_FOR_vcondv2dfv2di = 7423,
  CODE_FOR_vconduv64qiv16si = 7424,
  CODE_FOR_vconduv32hiv16si = 7425,
  CODE_FOR_vconduv16siv16si = 7426,
  CODE_FOR_vconduv8div16si = 7427,
  CODE_FOR_vconduv16sfv16si = 7428,
  CODE_FOR_vconduv8dfv16si = 7429,
  CODE_FOR_vconduv64qiv8di = 7430,
  CODE_FOR_vconduv32hiv8di = 7431,
  CODE_FOR_vconduv16siv8di = 7432,
  CODE_FOR_vconduv8div8di = 7433,
  CODE_FOR_vconduv16sfv8di = 7434,
  CODE_FOR_vconduv8dfv8di = 7435,
  CODE_FOR_vconduv64qiv32hi = 7436,
  CODE_FOR_vconduv32hiv32hi = 7437,
  CODE_FOR_vconduv16siv32hi = 7438,
  CODE_FOR_vconduv8div32hi = 7439,
  CODE_FOR_vconduv16sfv32hi = 7440,
  CODE_FOR_vconduv8dfv32hi = 7441,
  CODE_FOR_vconduv64qiv64qi = 7442,
  CODE_FOR_vconduv32hiv64qi = 7443,
  CODE_FOR_vconduv16siv64qi = 7444,
  CODE_FOR_vconduv8div64qi = 7445,
  CODE_FOR_vconduv16sfv64qi = 7446,
  CODE_FOR_vconduv8dfv64qi = 7447,
  CODE_FOR_vconduv32qiv32qi = 7448,
  CODE_FOR_vconduv16hiv32qi = 7449,
  CODE_FOR_vconduv8siv32qi = 7450,
  CODE_FOR_vconduv4div32qi = 7451,
  CODE_FOR_vconduv8sfv32qi = 7452,
  CODE_FOR_vconduv4dfv32qi = 7453,
  CODE_FOR_vconduv32qiv16hi = 7454,
  CODE_FOR_vconduv16hiv16hi = 7455,
  CODE_FOR_vconduv8siv16hi = 7456,
  CODE_FOR_vconduv4div16hi = 7457,
  CODE_FOR_vconduv8sfv16hi = 7458,
  CODE_FOR_vconduv4dfv16hi = 7459,
  CODE_FOR_vconduv32qiv8si = 7460,
  CODE_FOR_vconduv16hiv8si = 7461,
  CODE_FOR_vconduv8siv8si = 7462,
  CODE_FOR_vconduv4div8si = 7463,
  CODE_FOR_vconduv8sfv8si = 7464,
  CODE_FOR_vconduv4dfv8si = 7465,
  CODE_FOR_vconduv32qiv4di = 7466,
  CODE_FOR_vconduv16hiv4di = 7467,
  CODE_FOR_vconduv8siv4di = 7468,
  CODE_FOR_vconduv4div4di = 7469,
  CODE_FOR_vconduv8sfv4di = 7470,
  CODE_FOR_vconduv4dfv4di = 7471,
  CODE_FOR_vconduv16qiv16qi = 7472,
  CODE_FOR_vconduv8hiv16qi = 7473,
  CODE_FOR_vconduv4siv16qi = 7474,
  CODE_FOR_vconduv2div16qi = 7475,
  CODE_FOR_vconduv4sfv16qi = 7476,
  CODE_FOR_vconduv2dfv16qi = 7477,
  CODE_FOR_vconduv16qiv8hi = 7478,
  CODE_FOR_vconduv8hiv8hi = 7479,
  CODE_FOR_vconduv4siv8hi = 7480,
  CODE_FOR_vconduv2div8hi = 7481,
  CODE_FOR_vconduv4sfv8hi = 7482,
  CODE_FOR_vconduv2dfv8hi = 7483,
  CODE_FOR_vconduv16qiv4si = 7484,
  CODE_FOR_vconduv8hiv4si = 7485,
  CODE_FOR_vconduv4siv4si = 7486,
  CODE_FOR_vconduv2div4si = 7487,
  CODE_FOR_vconduv4sfv4si = 7488,
  CODE_FOR_vconduv2dfv4si = 7489,
  CODE_FOR_vconduv2div2di = 7490,
  CODE_FOR_vconduv2dfv2di = 7491,
  CODE_FOR_vcondeqv2div2di = 7492,
  CODE_FOR_vcondeqv2dfv2di = 7493,
  CODE_FOR_vec_permv16qi = 7494,
  CODE_FOR_vec_permv8hi = 7495,
  CODE_FOR_vec_permv4si = 7496,
  CODE_FOR_vec_permv2di = 7497,
  CODE_FOR_vec_permv4sf = 7498,
  CODE_FOR_vec_permv2df = 7499,
  CODE_FOR_vec_permv32qi = 7500,
  CODE_FOR_vec_permv16hi = 7501,
  CODE_FOR_vec_permv8si = 7502,
  CODE_FOR_vec_permv4di = 7503,
  CODE_FOR_vec_permv8sf = 7504,
  CODE_FOR_vec_permv4df = 7505,
  CODE_FOR_vec_permv16sf = 7506,
  CODE_FOR_vec_permv8df = 7507,
  CODE_FOR_vec_permv16si = 7508,
  CODE_FOR_vec_permv8di = 7509,
  CODE_FOR_vec_permv32hi = 7510,
  CODE_FOR_vec_permv64qi = 7511,
  CODE_FOR_one_cmplv16si2 = 7512,
  CODE_FOR_one_cmplv8di2 = 7513,
  CODE_FOR_one_cmplv64qi2 = 7514,
  CODE_FOR_one_cmplv32qi2 = 7515,
  CODE_FOR_one_cmplv16qi2 = 7516,
  CODE_FOR_one_cmplv32hi2 = 7517,
  CODE_FOR_one_cmplv16hi2 = 7518,
  CODE_FOR_one_cmplv8hi2 = 7519,
  CODE_FOR_one_cmplv8si2 = 7520,
  CODE_FOR_one_cmplv4si2 = 7521,
  CODE_FOR_one_cmplv4di2 = 7522,
  CODE_FOR_one_cmplv2di2 = 7523,
  CODE_FOR_avx512bw_andnotv64qi3 = 7524,
  CODE_FOR_avx2_andnotv32qi3 = 7525,
  CODE_FOR_sse2_andnotv16qi3 = 7526,
  CODE_FOR_avx512bw_andnotv32hi3 = 7527,
  CODE_FOR_avx2_andnotv16hi3 = 7528,
  CODE_FOR_sse2_andnotv8hi3 = 7529,
  CODE_FOR_avx512f_andnotv16si3 = 7530,
  CODE_FOR_avx2_andnotv8si3 = 7531,
  CODE_FOR_sse2_andnotv4si3 = 7532,
  CODE_FOR_avx512f_andnotv8di3 = 7533,
  CODE_FOR_avx2_andnotv4di3 = 7534,
  CODE_FOR_sse2_andnotv2di3 = 7535,
  CODE_FOR_avx512f_andnotv16si3_mask = 7536,
  CODE_FOR_avx2_andnotv8si3_mask = 7537,
  CODE_FOR_sse2_andnotv4si3_mask = 7538,
  CODE_FOR_avx512f_andnotv8di3_mask = 7539,
  CODE_FOR_avx2_andnotv4di3_mask = 7540,
  CODE_FOR_sse2_andnotv2di3_mask = 7541,
  CODE_FOR_avx512bw_andnotv64qi3_mask = 7542,
  CODE_FOR_sse2_andnotv16qi3_mask = 7543,
  CODE_FOR_avx2_andnotv32qi3_mask = 7544,
  CODE_FOR_avx512bw_andnotv32hi3_mask = 7545,
  CODE_FOR_avx2_andnotv16hi3_mask = 7546,
  CODE_FOR_sse2_andnotv8hi3_mask = 7547,
  CODE_FOR_andv16si3 = 7548,
  CODE_FOR_iorv16si3 = 7549,
  CODE_FOR_xorv16si3 = 7550,
  CODE_FOR_andv8di3 = 7551,
  CODE_FOR_iorv8di3 = 7552,
  CODE_FOR_xorv8di3 = 7553,
  CODE_FOR_andv64qi3 = 7554,
  CODE_FOR_iorv64qi3 = 7555,
  CODE_FOR_xorv64qi3 = 7556,
  CODE_FOR_andv32qi3 = 7557,
  CODE_FOR_iorv32qi3 = 7558,
  CODE_FOR_xorv32qi3 = 7559,
  CODE_FOR_andv16qi3 = 7560,
  CODE_FOR_iorv16qi3 = 7561,
  CODE_FOR_xorv16qi3 = 7562,
  CODE_FOR_andv32hi3 = 7563,
  CODE_FOR_iorv32hi3 = 7564,
  CODE_FOR_xorv32hi3 = 7565,
  CODE_FOR_andv16hi3 = 7566,
  CODE_FOR_iorv16hi3 = 7567,
  CODE_FOR_xorv16hi3 = 7568,
  CODE_FOR_andv8hi3 = 7569,
  CODE_FOR_iorv8hi3 = 7570,
  CODE_FOR_xorv8hi3 = 7571,
  CODE_FOR_andv8si3 = 7572,
  CODE_FOR_iorv8si3 = 7573,
  CODE_FOR_xorv8si3 = 7574,
  CODE_FOR_andv4si3 = 7575,
  CODE_FOR_iorv4si3 = 7576,
  CODE_FOR_xorv4si3 = 7577,
  CODE_FOR_andv4di3 = 7578,
  CODE_FOR_iorv4di3 = 7579,
  CODE_FOR_xorv4di3 = 7580,
  CODE_FOR_andv2di3 = 7581,
  CODE_FOR_iorv2di3 = 7582,
  CODE_FOR_xorv2di3 = 7583,
  CODE_FOR_vec_pack_trunc_v32hi = 7584,
  CODE_FOR_vec_pack_trunc_v16hi = 7585,
  CODE_FOR_vec_pack_trunc_v8hi = 7586,
  CODE_FOR_vec_pack_trunc_v16si = 7587,
  CODE_FOR_vec_pack_trunc_v8si = 7588,
  CODE_FOR_vec_pack_trunc_v4si = 7589,
  CODE_FOR_vec_pack_trunc_v8di = 7590,
  CODE_FOR_vec_pack_trunc_v4di = 7591,
  CODE_FOR_vec_pack_trunc_v2di = 7592,
  CODE_FOR_vec_pack_trunc_qi = 7593,
  CODE_FOR_vec_pack_trunc_hi = 7594,
  CODE_FOR_vec_pack_trunc_si = 7595,
  CODE_FOR_vec_pack_sbool_trunc_qi = 7596,
  CODE_FOR_vec_interleave_highv32qi = 7597,
  CODE_FOR_vec_interleave_highv16hi = 7598,
  CODE_FOR_vec_interleave_highv8si = 7599,
  CODE_FOR_vec_interleave_highv4di = 7600,
  CODE_FOR_vec_interleave_lowv32qi = 7601,
  CODE_FOR_vec_interleave_lowv16hi = 7602,
  CODE_FOR_vec_interleave_lowv8si = 7603,
  CODE_FOR_vec_interleave_lowv4di = 7604,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 7605,
  CODE_FOR_avx512dq_vinserti64x2_mask = 7606,
  CODE_FOR_avx512f_vinsertf32x4_mask = 7607,
  CODE_FOR_avx512f_vinserti32x4_mask = 7608,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 7609,
  CODE_FOR_avx512dq_vinserti32x8_mask = 7610,
  CODE_FOR_avx512f_vinsertf64x4_mask = 7611,
  CODE_FOR_avx512f_vinserti64x4_mask = 7612,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 7613,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 7614,
  CODE_FOR_avx512f_shuf_f64x2_mask = 7615,
  CODE_FOR_avx512f_shuf_i64x2_mask = 7616,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 7617,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 7618,
  CODE_FOR_avx512f_shuf_f32x4_mask = 7619,
  CODE_FOR_avx512f_shuf_i32x4_mask = 7620,
  CODE_FOR_avx512f_pshufdv3_mask = 7621,
  CODE_FOR_avx512vl_pshufdv3_mask = 7622,
  CODE_FOR_avx2_pshufdv3 = 7623,
  CODE_FOR_avx512vl_pshufd_mask = 7624,
  CODE_FOR_sse2_pshufd = 7625,
  CODE_FOR_avx512vl_pshuflwv3_mask = 7626,
  CODE_FOR_avx2_pshuflwv3 = 7627,
  CODE_FOR_avx512vl_pshuflw_mask = 7628,
  CODE_FOR_sse2_pshuflw = 7629,
  CODE_FOR_avx2_pshufhwv3 = 7630,
  CODE_FOR_avx512vl_pshufhwv3_mask = 7631,
  CODE_FOR_avx512vl_pshufhw_mask = 7632,
  CODE_FOR_sse2_pshufhw = 7633,
  CODE_FOR_sse2_loadd = 7634,
  CODE_FOR_vec_unpacks_lo_v64qi = 7635,
  CODE_FOR_vec_unpacks_lo_v32qi = 7636,
  CODE_FOR_vec_unpacks_lo_v16qi = 7637,
  CODE_FOR_vec_unpacks_lo_v32hi = 7638,
  CODE_FOR_vec_unpacks_lo_v16hi = 7639,
  CODE_FOR_vec_unpacks_lo_v8hi = 7640,
  CODE_FOR_vec_unpacks_lo_v16si = 7641,
  CODE_FOR_vec_unpacks_lo_v8si = 7642,
  CODE_FOR_vec_unpacks_lo_v4si = 7643,
  CODE_FOR_vec_unpacks_hi_v64qi = 7644,
  CODE_FOR_vec_unpacks_hi_v32qi = 7645,
  CODE_FOR_vec_unpacks_hi_v16qi = 7646,
  CODE_FOR_vec_unpacks_hi_v32hi = 7647,
  CODE_FOR_vec_unpacks_hi_v16hi = 7648,
  CODE_FOR_vec_unpacks_hi_v8hi = 7649,
  CODE_FOR_vec_unpacks_hi_v16si = 7650,
  CODE_FOR_vec_unpacks_hi_v8si = 7651,
  CODE_FOR_vec_unpacks_hi_v4si = 7652,
  CODE_FOR_vec_unpacku_lo_v64qi = 7653,
  CODE_FOR_vec_unpacku_lo_v32qi = 7654,
  CODE_FOR_vec_unpacku_lo_v16qi = 7655,
  CODE_FOR_vec_unpacku_lo_v32hi = 7656,
  CODE_FOR_vec_unpacku_lo_v16hi = 7657,
  CODE_FOR_vec_unpacku_lo_v8hi = 7658,
  CODE_FOR_vec_unpacku_lo_v16si = 7659,
  CODE_FOR_vec_unpacku_lo_v8si = 7660,
  CODE_FOR_vec_unpacku_lo_v4si = 7661,
  CODE_FOR_vec_unpacks_sbool_lo_qi = 7662,
  CODE_FOR_vec_unpacks_lo_hi = 7663,
  CODE_FOR_vec_unpacks_lo_si = 7664,
  CODE_FOR_vec_unpacks_lo_di = 7665,
  CODE_FOR_vec_unpacku_hi_v64qi = 7666,
  CODE_FOR_vec_unpacku_hi_v32qi = 7667,
  CODE_FOR_vec_unpacku_hi_v16qi = 7668,
  CODE_FOR_vec_unpacku_hi_v32hi = 7669,
  CODE_FOR_vec_unpacku_hi_v16hi = 7670,
  CODE_FOR_vec_unpacku_hi_v8hi = 7671,
  CODE_FOR_vec_unpacku_hi_v16si = 7672,
  CODE_FOR_vec_unpacku_hi_v8si = 7673,
  CODE_FOR_vec_unpacku_hi_v4si = 7674,
  CODE_FOR_vec_unpacks_sbool_hi_qi = 7675,
  CODE_FOR_vec_unpacks_hi_hi = 7676,
  CODE_FOR_vec_unpacks_hi_si = 7677,
  CODE_FOR_vec_unpacks_hi_di = 7678,
  CODE_FOR_avx512bw_uavgv64qi3 = 7679,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 7680,
  CODE_FOR_avx2_uavgv32qi3 = 7681,
  CODE_FOR_avx2_uavgv32qi3_mask = 7682,
  CODE_FOR_sse2_uavgv16qi3 = 7683,
  CODE_FOR_sse2_uavgv16qi3_mask = 7684,
  CODE_FOR_avx512bw_uavgv32hi3 = 7685,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 7686,
  CODE_FOR_avx2_uavgv16hi3 = 7687,
  CODE_FOR_avx2_uavgv16hi3_mask = 7688,
  CODE_FOR_sse2_uavgv8hi3 = 7689,
  CODE_FOR_sse2_uavgv8hi3_mask = 7690,
  CODE_FOR_sse2_maskmovdqu = 7691,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 7692,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 7693,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 7694,
  CODE_FOR_avx2_pmulhrswv16hi3 = 7695,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 7696,
  CODE_FOR_absv64qi2 = 7697,
  CODE_FOR_absv32qi2 = 7698,
  CODE_FOR_absv16qi2 = 7699,
  CODE_FOR_absv32hi2 = 7700,
  CODE_FOR_absv16hi2 = 7701,
  CODE_FOR_absv8hi2 = 7702,
  CODE_FOR_absv16si2 = 7703,
  CODE_FOR_absv8si2 = 7704,
  CODE_FOR_absv4si2 = 7705,
  CODE_FOR_absv8di2 = 7706,
  CODE_FOR_absv4di2 = 7707,
  CODE_FOR_absv2di2 = 7708,
  CODE_FOR_avx2_pblendw = 7709,
  CODE_FOR_avx_roundps_sfix256 = 7710,
  CODE_FOR_sse4_1_roundps_sfix = 7711,
  CODE_FOR_avx512f_roundps512 = 7712,
  CODE_FOR_avx512f_roundpd512 = 7713,
  CODE_FOR_avx512f_roundps512_sfix = 7714,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 7715,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 7716,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 7717,
  CODE_FOR_roundv16sf2 = 7718,
  CODE_FOR_roundv8sf2 = 7719,
  CODE_FOR_roundv4sf2 = 7720,
  CODE_FOR_roundv8df2 = 7721,
  CODE_FOR_roundv4df2 = 7722,
  CODE_FOR_roundv2df2 = 7723,
  CODE_FOR_roundv16sf2_sfix = 7724,
  CODE_FOR_roundv8sf2_sfix = 7725,
  CODE_FOR_roundv4sf2_sfix = 7726,
  CODE_FOR_roundv8df2_vec_pack_sfix = 7727,
  CODE_FOR_roundv4df2_vec_pack_sfix = 7728,
  CODE_FOR_roundv2df2_vec_pack_sfix = 7729,
  CODE_FOR_avx512pf_gatherpfv16sisf = 7730,
  CODE_FOR_avx512pf_gatherpfv8disf = 7731,
  CODE_FOR_avx512pf_gatherpfv8sidf = 7732,
  CODE_FOR_avx512pf_gatherpfv8didf = 7733,
  CODE_FOR_avx512pf_scatterpfv16sisf = 7734,
  CODE_FOR_avx512pf_scatterpfv8disf = 7735,
  CODE_FOR_avx512pf_scatterpfv8sidf = 7736,
  CODE_FOR_avx512pf_scatterpfv8didf = 7737,
  CODE_FOR_rotlv16qi3 = 7738,
  CODE_FOR_rotlv8hi3 = 7739,
  CODE_FOR_rotlv4si3 = 7740,
  CODE_FOR_rotlv2di3 = 7741,
  CODE_FOR_rotrv16qi3 = 7742,
  CODE_FOR_rotrv8hi3 = 7743,
  CODE_FOR_rotrv4si3 = 7744,
  CODE_FOR_rotrv2di3 = 7745,
  CODE_FOR_vrotrv16qi3 = 7746,
  CODE_FOR_vrotrv8hi3 = 7747,
  CODE_FOR_vrotrv4si3 = 7748,
  CODE_FOR_vrotrv2di3 = 7749,
  CODE_FOR_vrotlv16qi3 = 7750,
  CODE_FOR_vrotlv8hi3 = 7751,
  CODE_FOR_vrotlv4si3 = 7752,
  CODE_FOR_vrotlv2di3 = 7753,
  CODE_FOR_vlshrv16qi3 = 7754,
  CODE_FOR_vlshrv8hi3 = 7755,
  CODE_FOR_vlshrv4si3 = 7756,
  CODE_FOR_vlshrv2di3 = 7757,
  CODE_FOR_vlshrv16si3 = 7758,
  CODE_FOR_vlshrv8di3 = 7759,
  CODE_FOR_vlshrv8si3 = 7760,
  CODE_FOR_vlshrv4di3 = 7761,
  CODE_FOR_vashrv8hi3 = 7762,
  CODE_FOR_vashrv8hi3_mask = 7763,
  CODE_FOR_vashrv16qi3 = 7764,
  CODE_FOR_vashrv2di3 = 7765,
  CODE_FOR_vashrv2di3_mask = 7766,
  CODE_FOR_vashrv4si3 = 7767,
  CODE_FOR_vashrv16si3 = 7768,
  CODE_FOR_vashrv8si3 = 7769,
  CODE_FOR_vashlv16qi3 = 7770,
  CODE_FOR_vashlv8hi3 = 7771,
  CODE_FOR_vashlv4si3 = 7772,
  CODE_FOR_vashlv2di3 = 7773,
  CODE_FOR_vashlv16si3 = 7774,
  CODE_FOR_vashlv8di3 = 7775,
  CODE_FOR_vashlv8si3 = 7776,
  CODE_FOR_vashlv4di3 = 7777,
  CODE_FOR_ashlv64qi3 = 7778,
  CODE_FOR_lshrv64qi3 = 7779,
  CODE_FOR_ashrv64qi3 = 7780,
  CODE_FOR_ashlv32qi3 = 7781,
  CODE_FOR_lshrv32qi3 = 7782,
  CODE_FOR_ashrv32qi3 = 7783,
  CODE_FOR_ashlv16qi3 = 7784,
  CODE_FOR_lshrv16qi3 = 7785,
  CODE_FOR_ashrv16qi3 = 7786,
  CODE_FOR_ashrv2di3 = 7787,
  CODE_FOR_xop_vmfrczv4sf2 = 7788,
  CODE_FOR_xop_vmfrczv2df2 = 7789,
  CODE_FOR_avx_vzeroall = 7790,
  CODE_FOR_avx2_permv4di = 7791,
  CODE_FOR_avx2_permv4df = 7792,
  CODE_FOR_avx512vl_permv4di_mask = 7793,
  CODE_FOR_avx512vl_permv4df_mask = 7794,
  CODE_FOR_avx512f_permv8df = 7795,
  CODE_FOR_avx512f_permv8di = 7796,
  CODE_FOR_avx512f_permv8df_mask = 7797,
  CODE_FOR_avx512f_permv8di_mask = 7798,
  CODE_FOR_avx512f_vpermilv8df = 7799,
  CODE_FOR_avx512f_vpermilv8df_mask = 7800,
  CODE_FOR_avx_vpermilv4df = 7801,
  CODE_FOR_avx_vpermilv4df_mask = 7802,
  CODE_FOR_avx_vpermilv2df = 7803,
  CODE_FOR_avx_vpermilv2df_mask = 7804,
  CODE_FOR_avx512f_vpermilv16sf = 7805,
  CODE_FOR_avx512f_vpermilv16sf_mask = 7806,
  CODE_FOR_avx_vpermilv8sf = 7807,
  CODE_FOR_avx_vpermilv8sf_mask = 7808,
  CODE_FOR_avx_vpermilv4sf = 7809,
  CODE_FOR_avx_vpermilv4sf_mask = 7810,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 7811,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 7812,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 7813,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 7814,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 7815,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 7816,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 7817,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 7818,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 7819,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 7820,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 7821,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 7822,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 7823,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 7824,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 7825,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 7826,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 7827,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 7828,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 7829,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 7830,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 7831,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 7832,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 7833,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 7834,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 7835,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 7836,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 7837,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 7838,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 7839,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 7840,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 7841,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 7842,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 7843,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 7844,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 7845,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 7846,
  CODE_FOR_avx_vperm2f128v8si3 = 7847,
  CODE_FOR_avx_vperm2f128v8sf3 = 7848,
  CODE_FOR_avx_vperm2f128v4df3 = 7849,
  CODE_FOR_avx512vl_vinsertv8si = 7850,
  CODE_FOR_avx512vl_vinsertv8sf = 7851,
  CODE_FOR_avx512vl_vinsertv4di = 7852,
  CODE_FOR_avx512vl_vinsertv4df = 7853,
  CODE_FOR_avx_vinsertf128v32qi = 7854,
  CODE_FOR_avx_vinsertf128v16hi = 7855,
  CODE_FOR_avx_vinsertf128v8si = 7856,
  CODE_FOR_avx_vinsertf128v4di = 7857,
  CODE_FOR_avx_vinsertf128v8sf = 7858,
  CODE_FOR_avx_vinsertf128v4df = 7859,
  CODE_FOR_maskloadv4sfv4si = 7860,
  CODE_FOR_maskloadv2dfv2di = 7861,
  CODE_FOR_maskloadv8sfv8si = 7862,
  CODE_FOR_maskloadv4dfv4di = 7863,
  CODE_FOR_maskloadv4siv4si = 7864,
  CODE_FOR_maskloadv2div2di = 7865,
  CODE_FOR_maskloadv8siv8si = 7866,
  CODE_FOR_maskloadv4div4di = 7867,
  CODE_FOR_maskloadv16sihi = 7868,
  CODE_FOR_maskloadv8siqi = 7869,
  CODE_FOR_maskloadv4siqi = 7870,
  CODE_FOR_maskloadv8diqi = 7871,
  CODE_FOR_maskloadv4diqi = 7872,
  CODE_FOR_maskloadv2diqi = 7873,
  CODE_FOR_maskloadv16sfhi = 7874,
  CODE_FOR_maskloadv8sfqi = 7875,
  CODE_FOR_maskloadv4sfqi = 7876,
  CODE_FOR_maskloadv8dfqi = 7877,
  CODE_FOR_maskloadv4dfqi = 7878,
  CODE_FOR_maskloadv2dfqi = 7879,
  CODE_FOR_maskloadv64qidi = 7880,
  CODE_FOR_maskloadv16qihi = 7881,
  CODE_FOR_maskloadv32qisi = 7882,
  CODE_FOR_maskloadv32hisi = 7883,
  CODE_FOR_maskloadv16hihi = 7884,
  CODE_FOR_maskloadv8hiqi = 7885,
  CODE_FOR_maskstorev4sfv4si = 7886,
  CODE_FOR_maskstorev2dfv2di = 7887,
  CODE_FOR_maskstorev8sfv8si = 7888,
  CODE_FOR_maskstorev4dfv4di = 7889,
  CODE_FOR_maskstorev4siv4si = 7890,
  CODE_FOR_maskstorev2div2di = 7891,
  CODE_FOR_maskstorev8siv8si = 7892,
  CODE_FOR_maskstorev4div4di = 7893,
  CODE_FOR_maskstorev16sihi = 7894,
  CODE_FOR_maskstorev8siqi = 7895,
  CODE_FOR_maskstorev4siqi = 7896,
  CODE_FOR_maskstorev8diqi = 7897,
  CODE_FOR_maskstorev4diqi = 7898,
  CODE_FOR_maskstorev2diqi = 7899,
  CODE_FOR_maskstorev16sfhi = 7900,
  CODE_FOR_maskstorev8sfqi = 7901,
  CODE_FOR_maskstorev4sfqi = 7902,
  CODE_FOR_maskstorev8dfqi = 7903,
  CODE_FOR_maskstorev4dfqi = 7904,
  CODE_FOR_maskstorev2dfqi = 7905,
  CODE_FOR_maskstorev64qidi = 7906,
  CODE_FOR_maskstorev16qihi = 7907,
  CODE_FOR_maskstorev32qisi = 7908,
  CODE_FOR_maskstorev32hisi = 7909,
  CODE_FOR_maskstorev16hihi = 7910,
  CODE_FOR_maskstorev8hiqi = 7911,
  CODE_FOR_cbranchv4si4 = 7912,
  CODE_FOR_cbranchv2di4 = 7913,
  CODE_FOR_cbranchv8si4 = 7914,
  CODE_FOR_cbranchv4di4 = 7915,
  CODE_FOR_vec_initv64qiqi = 7916,
  CODE_FOR_vec_initv32qiqi = 7917,
  CODE_FOR_vec_initv16qiqi = 7918,
  CODE_FOR_vec_initv32hihi = 7919,
  CODE_FOR_vec_initv16hihi = 7920,
  CODE_FOR_vec_initv8hihi = 7921,
  CODE_FOR_vec_initv16sisi = 7922,
  CODE_FOR_vec_initv8sisi = 7923,
  CODE_FOR_vec_initv4sisi = 7924,
  CODE_FOR_vec_initv8didi = 7925,
  CODE_FOR_vec_initv4didi = 7926,
  CODE_FOR_vec_initv2didi = 7927,
  CODE_FOR_vec_initv16sfsf = 7928,
  CODE_FOR_vec_initv8sfsf = 7929,
  CODE_FOR_vec_initv4sfsf = 7930,
  CODE_FOR_vec_initv8dfdf = 7931,
  CODE_FOR_vec_initv4dfdf = 7932,
  CODE_FOR_vec_initv2dfdf = 7933,
  CODE_FOR_vec_initv4titi = 7934,
  CODE_FOR_vec_initv2titi = 7935,
  CODE_FOR_vec_initv64qiv32qi = 7936,
  CODE_FOR_vec_initv32qiv16qi = 7937,
  CODE_FOR_vec_initv16qiv8qi = 7938,
  CODE_FOR_vec_initv32hiv16hi = 7939,
  CODE_FOR_vec_initv16hiv8hi = 7940,
  CODE_FOR_vec_initv8hiv4hi = 7941,
  CODE_FOR_vec_initv16siv8si = 7942,
  CODE_FOR_vec_initv8siv4si = 7943,
  CODE_FOR_vec_initv4siv2si = 7944,
  CODE_FOR_vec_initv8div4di = 7945,
  CODE_FOR_vec_initv4div2di = 7946,
  CODE_FOR_vec_initv16sfv8sf = 7947,
  CODE_FOR_vec_initv8sfv4sf = 7948,
  CODE_FOR_vec_initv4sfv2sf = 7949,
  CODE_FOR_vec_initv8dfv4df = 7950,
  CODE_FOR_vec_initv4dfv2df = 7951,
  CODE_FOR_vec_initv4tiv2ti = 7952,
  CODE_FOR_vcvtps2ph_mask = 7953,
  CODE_FOR_vcvtps2ph = 7954,
  CODE_FOR_avx2_gathersiv2di = 7955,
  CODE_FOR_avx2_gathersiv2df = 7956,
  CODE_FOR_avx2_gathersiv4di = 7957,
  CODE_FOR_avx2_gathersiv4df = 7958,
  CODE_FOR_avx2_gathersiv4si = 7959,
  CODE_FOR_avx2_gathersiv4sf = 7960,
  CODE_FOR_avx2_gathersiv8si = 7961,
  CODE_FOR_avx2_gathersiv8sf = 7962,
  CODE_FOR_avx2_gatherdiv2di = 7963,
  CODE_FOR_avx2_gatherdiv2df = 7964,
  CODE_FOR_avx2_gatherdiv4di = 7965,
  CODE_FOR_avx2_gatherdiv4df = 7966,
  CODE_FOR_avx2_gatherdiv4si = 7967,
  CODE_FOR_avx2_gatherdiv4sf = 7968,
  CODE_FOR_avx2_gatherdiv8si = 7969,
  CODE_FOR_avx2_gatherdiv8sf = 7970,
  CODE_FOR_avx512f_gathersiv16si = 7971,
  CODE_FOR_avx512f_gathersiv16sf = 7972,
  CODE_FOR_avx512f_gathersiv8di = 7973,
  CODE_FOR_avx512f_gathersiv8df = 7974,
  CODE_FOR_avx512vl_gathersiv8si = 7975,
  CODE_FOR_avx512vl_gathersiv8sf = 7976,
  CODE_FOR_avx512vl_gathersiv4di = 7977,
  CODE_FOR_avx512vl_gathersiv4df = 7978,
  CODE_FOR_avx512vl_gathersiv4si = 7979,
  CODE_FOR_avx512vl_gathersiv4sf = 7980,
  CODE_FOR_avx512vl_gathersiv2di = 7981,
  CODE_FOR_avx512vl_gathersiv2df = 7982,
  CODE_FOR_avx512f_gatherdiv16si = 7983,
  CODE_FOR_avx512f_gatherdiv16sf = 7984,
  CODE_FOR_avx512f_gatherdiv8di = 7985,
  CODE_FOR_avx512f_gatherdiv8df = 7986,
  CODE_FOR_avx512vl_gatherdiv8si = 7987,
  CODE_FOR_avx512vl_gatherdiv8sf = 7988,
  CODE_FOR_avx512vl_gatherdiv4di = 7989,
  CODE_FOR_avx512vl_gatherdiv4df = 7990,
  CODE_FOR_avx512vl_gatherdiv4si = 7991,
  CODE_FOR_avx512vl_gatherdiv4sf = 7992,
  CODE_FOR_avx512vl_gatherdiv2di = 7993,
  CODE_FOR_avx512vl_gatherdiv2df = 7994,
  CODE_FOR_avx512f_scattersiv16si = 7995,
  CODE_FOR_avx512f_scattersiv16sf = 7996,
  CODE_FOR_avx512f_scattersiv8di = 7997,
  CODE_FOR_avx512f_scattersiv8df = 7998,
  CODE_FOR_avx512vl_scattersiv8si = 7999,
  CODE_FOR_avx512vl_scattersiv8sf = 8000,
  CODE_FOR_avx512vl_scattersiv4di = 8001,
  CODE_FOR_avx512vl_scattersiv4df = 8002,
  CODE_FOR_avx512vl_scattersiv4si = 8003,
  CODE_FOR_avx512vl_scattersiv4sf = 8004,
  CODE_FOR_avx512vl_scattersiv2di = 8005,
  CODE_FOR_avx512vl_scattersiv2df = 8006,
  CODE_FOR_avx512f_scatterdiv16si = 8007,
  CODE_FOR_avx512f_scatterdiv16sf = 8008,
  CODE_FOR_avx512f_scatterdiv8di = 8009,
  CODE_FOR_avx512f_scatterdiv8df = 8010,
  CODE_FOR_avx512vl_scatterdiv8si = 8011,
  CODE_FOR_avx512vl_scatterdiv8sf = 8012,
  CODE_FOR_avx512vl_scatterdiv4di = 8013,
  CODE_FOR_avx512vl_scatterdiv4df = 8014,
  CODE_FOR_avx512vl_scatterdiv4si = 8015,
  CODE_FOR_avx512vl_scatterdiv4sf = 8016,
  CODE_FOR_avx512vl_scatterdiv2di = 8017,
  CODE_FOR_avx512vl_scatterdiv2df = 8018,
  CODE_FOR_avx512f_expandv16si_maskz = 8019,
  CODE_FOR_avx512f_expandv16sf_maskz = 8020,
  CODE_FOR_avx512f_expandv8di_maskz = 8021,
  CODE_FOR_avx512f_expandv8df_maskz = 8022,
  CODE_FOR_avx512vl_expandv8si_maskz = 8023,
  CODE_FOR_avx512vl_expandv8sf_maskz = 8024,
  CODE_FOR_avx512vl_expandv4di_maskz = 8025,
  CODE_FOR_avx512vl_expandv4df_maskz = 8026,
  CODE_FOR_avx512vl_expandv4si_maskz = 8027,
  CODE_FOR_avx512vl_expandv4sf_maskz = 8028,
  CODE_FOR_avx512vl_expandv2di_maskz = 8029,
  CODE_FOR_avx512vl_expandv2df_maskz = 8030,
  CODE_FOR_expandv64qi_maskz = 8031,
  CODE_FOR_expandv16qi_maskz = 8032,
  CODE_FOR_expandv32qi_maskz = 8033,
  CODE_FOR_expandv32hi_maskz = 8034,
  CODE_FOR_expandv16hi_maskz = 8035,
  CODE_FOR_expandv8hi_maskz = 8036,
  CODE_FOR_vpamdd52huqv8di_maskz = 8037,
  CODE_FOR_vpamdd52huqv4di_maskz = 8038,
  CODE_FOR_vpamdd52huqv2di_maskz = 8039,
  CODE_FOR_vpamdd52luqv8di_maskz = 8040,
  CODE_FOR_vpamdd52luqv4di_maskz = 8041,
  CODE_FOR_vpamdd52luqv2di_maskz = 8042,
  CODE_FOR_movv64sf = 8043,
  CODE_FOR_movv64si = 8044,
  CODE_FOR_vpshrdv_v32hi_maskz = 8045,
  CODE_FOR_vpshrdv_v16si_maskz = 8046,
  CODE_FOR_vpshrdv_v8di_maskz = 8047,
  CODE_FOR_vpshrdv_v16hi_maskz = 8048,
  CODE_FOR_vpshrdv_v8si_maskz = 8049,
  CODE_FOR_vpshrdv_v4di_maskz = 8050,
  CODE_FOR_vpshrdv_v8hi_maskz = 8051,
  CODE_FOR_vpshrdv_v4si_maskz = 8052,
  CODE_FOR_vpshrdv_v2di_maskz = 8053,
  CODE_FOR_vpshldv_v32hi_maskz = 8054,
  CODE_FOR_vpshldv_v16si_maskz = 8055,
  CODE_FOR_vpshldv_v8di_maskz = 8056,
  CODE_FOR_vpshldv_v16hi_maskz = 8057,
  CODE_FOR_vpshldv_v8si_maskz = 8058,
  CODE_FOR_vpshldv_v4di_maskz = 8059,
  CODE_FOR_vpshldv_v8hi_maskz = 8060,
  CODE_FOR_vpshldv_v4si_maskz = 8061,
  CODE_FOR_vpshldv_v2di_maskz = 8062,
  CODE_FOR_vpdpbusd_v16si_maskz = 8063,
  CODE_FOR_vpdpbusd_v8si_maskz = 8064,
  CODE_FOR_vpdpbusd_v4si_maskz = 8065,
  CODE_FOR_vpdpbusds_v16si_maskz = 8066,
  CODE_FOR_vpdpbusds_v8si_maskz = 8067,
  CODE_FOR_vpdpbusds_v4si_maskz = 8068,
  CODE_FOR_vpdpwssd_v16si_maskz = 8069,
  CODE_FOR_vpdpwssd_v8si_maskz = 8070,
  CODE_FOR_vpdpwssd_v4si_maskz = 8071,
  CODE_FOR_vpdpwssds_v16si_maskz = 8072,
  CODE_FOR_vpdpwssds_v8si_maskz = 8073,
  CODE_FOR_vpdpwssds_v4si_maskz = 8074,
  CODE_FOR_sse2_lfence = 8075,
  CODE_FOR_sse_sfence = 8076,
  CODE_FOR_sse2_mfence = 8077,
  CODE_FOR_mem_thread_fence = 8078,
  CODE_FOR_atomic_loadqi = 8079,
  CODE_FOR_atomic_loadhi = 8080,
  CODE_FOR_atomic_loadsi = 8081,
  CODE_FOR_atomic_loaddi = 8082,
  CODE_FOR_atomic_storeqi = 8083,
  CODE_FOR_atomic_storehi = 8084,
  CODE_FOR_atomic_storesi = 8085,
  CODE_FOR_atomic_storedi = 8086,
  CODE_FOR_atomic_compare_and_swapqi = 8087,
  CODE_FOR_atomic_compare_and_swaphi = 8088,
  CODE_FOR_atomic_compare_and_swapsi = 8089,
  CODE_FOR_atomic_compare_and_swapdi = 8090,
  CODE_FOR_atomic_compare_and_swapti = 8091,
  CODE_FOR_atomic_bit_test_and_sethi = 8092,
  CODE_FOR_atomic_bit_test_and_setsi = 8093,
  CODE_FOR_atomic_bit_test_and_setdi = 8094,
  CODE_FOR_atomic_bit_test_and_complementhi = 8095,
  CODE_FOR_atomic_bit_test_and_complementsi = 8096,
  CODE_FOR_atomic_bit_test_and_complementdi = 8097,
  CODE_FOR_atomic_bit_test_and_resethi = 8098,
  CODE_FOR_atomic_bit_test_and_resetsi = 8099,
  CODE_FOR_atomic_bit_test_and_resetdi = 8100
};

const unsigned int NUM_INSN_CODES = 8101;
#endif /* GCC_INSN_CODES_H */
